// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
//
// OLEDディスプレーとAE-GNSSの組合せ最終バージョン（一点物）
// 2022/01/30: STM32_GPSCOUNTER_V50_60yo_ol-ae
// 2022/03/23: STM32_GPSCOUNTER_V50_60y_OL-GN
//             OLEDディスプレーの表示タイミングを調整　(2154行目辺り)
//             PICをリセットするボタンを廃止する。（特に影響なし）
//             連続計測回数を99999とする。（最短約55時間）
//
// 2022/04/11: STM32_GPSCOUNTER_V50_61    ここからTFT & UBLOX_NEO_M8N のみの更新とする。
//
// 2022/04/14: STM32F411を試そうとライブラリー等を更新したところ、約２か月前にボード情報が
//             更新されていて、それを導入すると大きなフォントが出てこない。
//                 Arduino_STM32-master.zip
//             ここのこと  https://github.com/rogerclarkmelbourne/Arduino_STM32
//             このファイルを展開し、hardwareフォルダーに入れて使う。
//             多分フォントの切り替えをした時に弾かれたのではないかと思われる。
//             そのため、過去のバックアップからSTM32F のボード情報を復帰させた。
//             もし、新規に導入するときは、画面右に V1.0.0beta が有るのでそれを
//             使用すると上手くいくと思われる。
//                 Arduino_STM32-1.0.0.zip が旧ファイル
// 
// 2022/09/16: STM32_GPSCOUNTER_V50_62 表示処理を遅らせる処理と、軽微なレベルの修正。
//             ＜微修正＞連続計測回数　99999 => 100000 に変更。オーバー後は再度 1 から
//             ＜微修正＞「I2Cが未接続なら、VconAUTO を強制的にOFF」(Vcon使用時のみ関連)
//             ＜微修正＞ dmHz 値の無効判定条件式を改定(ROMが初期値の時のみ該当、Vcon時のみ関連)
//             ＜余裕度＞$GNGGAの受信衛星数がNWO-M8Nのバラツキにより取り切れないことがあったので、
//             　　表示処理を200mS遅らせる遅延を挿入。
//                 無操作時に、画面２行目左の衛星受信数（通常はMax12)が点滅する場合は効果が見込める。
//             ＜拡張性＞PCB V5.1用にBuzzerポートを追加できるようにした。
// 
// 
const char verDate[] = "V50_62 & 2022/09/16";   // ★★この行は文頭の変更履歴直下に置く★★
// 
// 
// 
// 
// GPSをゲート信号に使用した周波数カウンター
// 用途：OCXO Calibrator
//
// 対応するPIC側ソフトは、A6_V41calib_TIM1_U2TX_24FV32KA301.X
//
// Arduino IDEのボードマネージャにて追加 v1.9.0まで
// ■Arduino SAM Boards (32-bits ARM Cortex-M3)
// https://github.com/stm32duino/BoardManagerFiles/raw/master/STM32/package_stm_index.json
//
// 以下のボードライブラリーV2.0.0 以降は、ボード選択が多岐にわたるため導入を止めている。(2022/04/14)
// xxxxxhttps://raw.githubusercontent.com/stm32duino/BoardManagerFiles/main/package_stmicroelectronics_index.json
//
//
//  主要なライブラリの導入状況　2021/02/01現在
// ■Adafruit_ILI9341  1.5.5  ==> 1.5.6 ==> 1.7.5(○)
//     Adafruit_GFX_Library 1.7.5(○) ==> アップすると全エラー
//     最新 Adafruit_GFX_Library 1.10.4を試すがエラーのため 1.7.5に戻す(2021/02/01)
//     Arduino IDE 1.8.15 (2021/05/20)でも同様に 1.7.5(○) ==> アップすると全エラー
//
// ■U8g2_for_Adafruit_GFX  1.7.0(○) ==> 1.8.0(○)   (2020/02/01)
// ■U8g2  2.28.10
// ■Adafruit_ILI9341  1.5.6
// ■u8g2_fonts  2.27.6 ==> 2.28.10  (2020/02/01)
// ■XPT2046_touch  1.3.0
//
//
//  (c) 2020-2022 jlb.jp
//
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
/*

  ■2019/12/26最初の記述
  STM32F103F103C8
  //----------------------------------------------
  　表示出力は  OLED  SSD1309
  　2.42" Inch 12864 128 * 64 OLED Display Module SPI

  OLED Pin UNO________STM32
  GND      GND________GND
  VCC      5v_________3V3
  SCL      13 (SCK)___PA5
  SDA      11 (MOSI)__PA7
  RES      8__________PA3
  DC       9__________PA2
  CS       10 (SS)____PA4
  Note:MISO(PA6) is No connection

  Arduino UNO u8g2
  U8G2_SSD1309_128X64_NONAME0_F_4W_HW_SPI u8g2(U8G2_R0,  10,9, 8);
  STM32duino u8g2
  U8G2_SSD1309_128X64_NONAME0_F_4W_HW_SPI u8g2(U8G2_R0, PA4,  PA2,  PA3);
  //----------------------------------------------
  C++の仕様により、
  警告：文字列定数から 'char *'への非推奨の変換　が発生する場合が有る。
  その時は、　char* str ===> const char* str　とする。
  //----------------------------------------------
  ＧＰＳは、秋月電子扱いの太陽誘電社GNSSを使用。
  　[AE_GNSS-EXTANT]　通販コード　K-13849
  　・1PPS出力：C-MOSロジック(3.3V)レベル、パルス幅：100mS(アクティブLow)
  //----------------------------------------------

  ■2020/01/27：
  ・パラメータ設定・編集処理完了

  ■2020/01/29：
  ・外部EEPROM 24C256等の読み出し・書込み処理完了

  ■2020/02/05:
  ・シリアル速度変更が失敗する時が有ったので、コマンドを２回送信に変更。
  ・GPSとの通信速度選択で57600bpsを止め、38600,19200,9600bpsの３種から選択とする。
  ・UTC時間をJSTに変更。
  ・年月日もJSTに変更した。（LOGに使用）

  ■2020/03/25:
  ・GPS信号が受信できなくなった時に、計測を止めて信号が復帰するのを待つ。
    そのために、Timer2を使用してPPS信号の間隔が1.2秒以上開くと、再開モードをセットする。
  ・現在４bitの前置カウンタを入れている状態
    最高周波数が、２．７MHzで74HC161のキャリーを取り逃がすような現象のため、
    入力を２．５MHzで検証中。（8bit 換算でMax40MHz）
  ・割込み(int)内の変数に=volatile を付ける。（volatile = 変わり易い, 揮発性の意味）
  ・LOGの出力を整理した。＝＝＞CSV出力（漢字コードがUTF-8のため、ASCII記述のみに変更）
  ・LOGの最後に入力パルスカウントのLSB側16ビットを出力するように変更
    （ビット化け・欠落の確認用）
  ・再設計時には、SW-AD入力用(現PA1)を空き番として、アナログ入力はPB0又はPB1に変更し、
    PA0・PA1を空ける。

  ■2020/05/21:
  ・U8g2_for_Adafruit_GFX フォントを採用する。

    ＜評価＞__ _______________フォント______速度______上書き_____
    Adafruit_GFX　　　　　 ｜　　×　　｜　　◎　　｜　　×　　｜
    -----------------------------------------------------------
    Ucg　　　　　　　　　　｜　　〇　　｜　　×　　｜　　〇　　｜
    -----------------------------------------------------------
    U8g2_for_Adafruit_GFX　｜　　〇　　｜　　〇　　｜　　〇　　｜
    -----------------------------------------------------------

    ・XPT2046_Touchscreen ライブラリー導入
    ▲XXX STM32_ILI9341_U8g2_for_Adafruit_GFX_Touch　"XPT2046_touch.h" 自前の割込み
      ・・・・・2度目の割込み受け付けない。
    ▲STM32_ILI9341_U8g2_for_Adafruit_GFX_Touch2 <XPT2046_Touchscreen.h> 割込み無し
      Zあり、動きはOK
    ▲STM32_ILI9341_U8g2_for_Adafruit_GFX_TouchIRQ <XPT2046_Touchscreen.h> 割込み有り
      Zあり、動きはOK
    ▲STM32_ILI9341_U8g2_for_Adafruit_GFX_Touch3 "XPT2046_touch.h"  割込み無し
      Z無し、動きはOK  (STM32のサンプルに有ったので、これを採用する。
      STM32サンプルの場所  \Documents\ArduinoIDE1.8\hardware\Arduino_STM32\STM32F1\
                           libraries\Serasidis_XPT2046_touch\examples\TouchTest
        Z:押し圧
  ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

  ■2020/06/24:
  ・初回公開版  STM32_GPSCOUNTER_V50_10    V5.0の基板用　 (c)2020 jlb.jp
  ・OLEDの表示タイミングを、非同期の5回／秒からPPS信号に同期した2回に変更。
    （ciPPS信号がLOWになった瞬間と500mS後の2回）
  ・TFT側も2回／秒表示に切り替えたが、その後OLEDとTFTともに、カウント中の表示更新は
    1回／秒に、待機中は2回／秒に変更した。
  ・NEOに対してボーレート変更の指示も、AE-GNSS同様にSTM32F103より送出するように変更した。
  ・buttonSW() のスイッチ判定定数見直し。
  ・V5用にBUZZERを追加(TFT)(OLED)
  ・TFT時のファンクション表示(最下行）のｙ位置を修正。（2ドット）

  ■2020/07/26:
  ・＄GxGGAより抽出の衛星数を、計測中の最低衛星数と最大衛星数に分けてLOG記録する。
  ・PicReset()を1543行に追加。
  ・LM61&MPC9700Aの温度計算式：修正。

  ■2020/08/31:V50_51
  ・TFT用タッチ状態確認BUZZERの発音を、押している間の連続発音から固定長(50mSec)に変更。
  ・OLED用ボタンSWの発音は、上記と同様に固定長(50mSec)に変更と、
    1秒以上の長押し時に10mSの単音を50ｍS間隔で発音する処理に変更。

  ■2020/09/14:V50_51
  ・OLED用[F3]ボタンSWの検出範囲を、変更した。（誤検出対応）

  ■2020/10/19:V50_51
  ・LM61センサーの温度が上下に変動(0.2-3度)するため ==> MPC9700Aに交換。
    後で分かったこと（アナログ入力処理の共用のため！？）
  ・ハンダ付け直後は、センサー温度が下がらない。（多分、数時間ぐらい）
  ・OLEDの構成（アナログSW)では、温度センサーの読み出しが安定しないことが判明。
    対応を試みたが、明確な解決策は無く、「OLEDに温度センサーは採用しない」こととした。
  ・IncDec の計算式のbug修正（周波数カウントが完了した時に前回との差を求める処理)
    // if (CountTime == 1) {   // 1回目のカウントは、プラスマイナスはゼロ //

  ■2020/10/23:V50_52
  ・温度センサーの定義を、
    センサーを使用する＝定義とセンサーの種類に分けた。
  ・SENSOR_MPC9700A を取り止め、TMP36 (ANALOG DEVICES)に変更。
      理由：温度変化への追従性が良さそう。
  ・温度センサーの仕様に準じた計算式へ修正（傾き=10, 誤差=±○○℃)

  ■2020/11/24:V50_52
  ・PCB頒布のため、記述内容を再確認。

  ■2020/12/13:V50_53
  ・GNGGA が受け取れなかったときのため、「使用GPS数」を表示後毎回クリヤーする。
  ・画面表示回数を1回に固定する。ただし、GATE時間の切り換えは即時更新とする。
  ・BUZZER の記述を整理し、Google翻訳で英表記の見直しを実施。
  ・最小周波数の処理を修正。

  ■2020/12/15:V50_54
  ・GATEタイムが1秒の時、[STOP][STRT]を繰り返すとGNGGA等の信号が受けきれない時が
    有ったので、[STOP]後の[STRT]を「タイマー割込み２」で遅らせる処理を追加した。(TFT)
    遅らせる時間「stopTime(1100 mSec)」,（早押し抑制とも表記している）

  ■2020/12/17:V50_54a
  ・PIC側のLEDがカウント毎に点滅を繰り返さなかったので、picRESET()を一ヶ所
    コメントアウトした。
  ・変更履歴のダブりを修正。

  ■2020/12/28:V50_54b
  ・[Agin]を[AGIN]に変更。
  ・1回以上カウントした後の停止で、「CountTime」を画面上で消さずに残すよう変更。
  ・GATE=1-^4 は、アベレージを少数以下5桁まで表示する。（TFTのみ）

  ■2021/02/08:V50_60e
  ・OCXO周波数微調整のVconをコントロールする機能追加。MCP4725
  ・　#define OPTION_OCXO_Vcon  (TFT限定）
  ・DEVICE_ADDRESS==>EEPROM_ADDRESS に変更。
  ・パラメータ入力用の10Key入力を採用。
  ・変数名を変更　form1st ==> formDisp, func1st ==> funcDisp, disp1st ==> dispOnce
  ・ #define SOFT_RESET  リセットをMENUに追加（PB0にトランジスタ回路追加）
  ・ #define BUZZER_PORT_PB13  リセット利用時は、BUZZERの利用をPB13に切り替えること。

  ■2021/03/31:V50_60f　非公開
  ・VconのチップをMCP4725とMCP4726の2通りを使えるように変更。
  ・Vconの補正計算を見直し（誤差の計算式）
  ・VconのLOGフォーマットにAUTO/MANUALを追加した。
  ・char logheader[]の3つの"Loss"を"E"に変更して文字数を縮小した。
  ・変化量抑制のため、理想値付近の補正量を(tmpdac / 2)としていたのを
  　(tmpdac )に変更した。50% ==> 100%

  ■2021/07/05:V50_60g
  ・TOYOCOM(TCO-6920N)の1LSBが0.1mHz以下なので、Vcon自動補正の見直し。
  ・数か月更新が無いので公開

  ■2021/11/19:V50_60h
  ・OLEDタイプの入力ＳＷのアナログ値範囲を変更（各キーの判断）
  ・TFTは変更なし。


   ■■■これ以降の変更履歴は文頭に記述する。■■■


  TFT色指定のワードメモ
  ILI9341_BLACK, NAVY, DARKGREEN, DARKCYAN, MAROON, PURPLE, OLIVE
  , LIGHTGREY, DARKGREY, BLUE, GREEN, CYAN, RED, MAGENTA, YELLOW
  , WHITE, ORANGE, GREENYELLOW, PINK

  I2C EEPROM 24C256 Address MAP(2021/01/28)
  ------------------------------------------------------------------------------------------------
   00 ｜ 01 ｜ 02 ｜ 03 ｜ 04 ｜ 05 ｜ 06 ｜ 07 ｜ 08 ｜ 09 ｜ 0A ｜ 0B ｜ 0C ｜ 0D ｜ 0E ｜ 0F
  bps ｜host｜gate｜-tchXmin- ｜-tchYmin- ｜-tchXmax- ｜-tchYmax- ｜    ｜    ｜    ｜    ｜
  ------------------------------------------------------------------------------------------------
   10 ｜ 11 ｜ 12 ｜ 13 ｜ 14 ｜ 15 ｜ 16 ｜ 17 ｜ 18 ｜ 19 ｜ 1A ｜ 1B ｜ 1C ｜ 1D ｜ 1E ｜ 1F
   ============ double UNI.dmHz =============== ｜MODE｜    ｜    ｜    ｜    ｜    ｜    ｜
  ------------------------------------------------------------------------------------------------

 ************************************************/

/////////////////////////////////////////////////////////////////////////////

#include "STM32_GPSCOUNTER_V50.h" // 表示・ＧＰＳ等のオプション選択
#include <Wire.h>
#include <SPI.h>

// I2C EEPROM アドレス
#define EEPROM_ADDRESS 0x50 //I2C EEPROM スレーブアドレス 24C256等
#define Vcon_ADDRESS 0x60   //I2C 12bit DAC MCP4726A0T-E/CH

//#include <wirish_time.h>          // STM32 delay()に

/*
   timer.h ファイルは、
   \Documents\ArduinoIDE1.8\hardware\Arduino_STM32\STM32F1\system\libmaple
   \stm32f1\include\libmaple
   （\libmaple フォルダは追加作成）
   に保存した。
*/
//#include "libmaple/timer.h"        // STM32 GitHub より入手、
/////////////////////////////////////////////////////////////////////////////

#ifdef BOARD_V5_BUZZER
#ifdef BUZZER_PORT_PB0
#define BUZZER PB0
#endif
#ifdef BUZZER_PORT_PB13
#define BUZZER PB13
#endif
#ifdef BUZZER_PORT_PB5
#define BUZZER PB5
#endif
#endif

#ifdef SOFT_RESET // =================
#define RESET_PORT PB0
#endif            // =================
// =======================================================================
#ifdef OPTION_SSD1309_OLED

#include <U8g2lib.h>
// OLED SSD1309 Board Pin definitions
#define SSD_MOSI PA7      // Serial DATA Pin(out) SPI:MOSI
#define SSD_MISO PA6      // Serial DATA Pin(in)  SPI:MISO
#define SSD_SCK  PA5      // Serial CLOCK Pin(out)
#define SSD_CS   PA4      // Chip Select Active Low(out) 
#define SSD_RST  PA3      // Reset(out) Active Low(out) 
#define SSD_DC   PA2      // Data/Command(out)
U8G2_SSD1309_128X64_NONAME0_F_4W_HW_SPI u8g2(U8G2_R0, SSD_CS, SSD_DC, SSD_RST);

#define ADC_In  PA0     // アナログ入力ピンの定義（予備）
#define SWa_In  PA1     // コントロールＳＷ アナログ入力ピンの定義

uint8_t dispOnce = 0;
uint8_t dispGo = 0;

#endif
// --------------------------------
#ifdef OPTION_ILI9341_TFT

// Adafruit_GFX_Libraryは、 1.7.5を超えるとエラーになるので V1.7.5 のままとする。V50_53
// 以下の #include "Adafruit_GFX.h" は、コメントアウトしても影響が無かった(2021/02/02)
//#include "Adafruit_GFX.h"
#include "Adafruit_ILI9341.h"
//#include <u8g2_fonts.h>
#include <U8g2_for_Adafruit_GFX.h>
// For the Adafruit shield, these are the default.
#define TFT_MOSI PA7      // Serial DATA Pin(out) SPI:MOSI
#define TFT_MISO PA6      // Serial DATA Pin(in)  SPI:MISO
#define TFT_SCK  PA5      // Serial CLOCK Pin(out)
#define TFT_CS   PA4      // Chip Select Active Low(out) 
#define TFT_RST  PA3      // Reset(out) Active Low(out) 
#define TFT_DC   PA2      // Data/Command(out)

//Adafruit_ILI9341 display = Adafruit_ILI9341(TFT_CS, TFT_DC);
Adafruit_ILI9341 display = Adafruit_ILI9341(TFT_CS, TFT_DC, TFT_RST);

U8G2_FOR_ADAFRUIT_GFX u8g2_for_adafruit_gfx;

#include "XPT2046_touch.h"
#define CS_PIN  PA1 // Chip Select pin

SPIClass mySPI(1); //Create an SPI instance on SPI1 port.
XPT2046_touch ts(CS_PIN, mySPI); // Chip Select pin, SPI port

uint16_t xy[2];
char tmpX[10];
char tmpY[10];

#define InitXmin 372
#define InitYmin 260
#define InitXmax 3907
#define InitYmax 3775
uint16_t tchXmin = InitXmin;  //仮のタッチセンサー初期値(左上x)
uint16_t tchYmin = InitYmin;  //仮のタッチセンサー初期値(左上y)
uint16_t tchXmax = InitXmax;  //仮のタッチセンサー初期値(右下x)
uint16_t tchYmax = InitYmax;  //仮のタッチセンサー初期値(右下y)
const uint16_t pixWidth = 320;    // TFT横ドット数
const uint16_t pixHeight = 240;   // TFT縦ドット数
int8_t ans;
uint8_t formDisp = 0;    // 計測中に変化しない内容を１回だけ表示するために使用
uint8_t funcDisp = 0;    // 計測中に変化しないFUNC内容を１回だけ表示するために使用
uint8_t dispOnce = 0;    // 1回／秒、時刻や温度等を表示することに使用
uint8_t dispGo = 0;

#endif
// =======================================================================
// ■■共通設定■■

#define ciGATE2 PB15    // 
#define coGSET  PA8     // Counter Gate Active High(out)Low edge
#define ciGATE  PA15    // Counter Gate ON
#define coPicREST  PB3     // Counter reset Active High(out) 
#define coRSET  PB4     // Counter reset Active LOW(out)
#define LED_OUT PB8     // 
#define ciPPS   PB9     // Counter Input 1sec Active High edge(increment)
#define LED_PIN PC13    // STM32F103C8T6 PC13 or 32 or D32
#define stopTime 1100   // 入力抑制の時間 mSec.（速いボタン操作の抑制）

// ==============================================
// タイマー割り込み関係
volatile int32_t tIntCount1;  // Timer1:割込み回数
volatile int32_t tIntCount2;  // Timer2:割込み回数
volatile int32_t tIntCount3;  // Timer3:割込み回数（int内で使用=volatile を付ける）
// ==============================================
// シリアル受信処理関係
uint16_t RXposi = 0;
uint8_t RCVok = 0;
uint8_t PICposi = 0;
uint8_t PICok = 0;
byte RXchar, PCchar, PICchar;
char RXbuff[255];
char RXbuff2[255];
char PICbuff[32];     // '[',16文字,SUMx2,']'PICからのカウンターデータ受取
char tmpbuff[64];
char useGPS[5];
char nowTime[22];
char nowDate[22];
char logdata[256];
char tmptmp[32];

// uint16_t SSt;                 // キー操作用変数
int8_t Keycnt = 0;
uint16_t FuncEnable = 0;       // Function キー有効ビット F0:bit4, F1:bit3, F2:bit2, F3:bit1,
uint8_t jobPRM = 0;           // 編集画面の処理No
uint8_t bpsPRM = 0;           // BPSの指数
uint8_t vconPRM = 0;           // vconの指数
uint8_t hostPRM = 0;          // HOSTへ送出するデータ選択　0:GPS Data, 1:LOG Data, 2:None
uint8_t gatePRM = 0;          // GATE時間の指数
uint8_t gatePRMon = 0;          // 電源ON時に指定するGATE時間
uint8_t dispNo = 0;           // 表示番号　0:初期画面ならびに計測中, 1:各パラメータ表示中
uint8_t Counting = 0;         // 計数中 = 1;
uint16_t gpsLoss = 0;         // GPSエラー回数
uint16_t ppsLoss = 0;         // PPS信号間隔のエラー回数
uint32_t CountTime = 0;       // 計測回数
uint32_t picLoss = 0;        // PICデータ受信エラー回数
uint8_t SATmax;               // 最大衛星数
uint8_t SATmin;               // 最小衛星数

#ifdef OPTION_AE_GNSS
// GNSSとの通信速度変更用（チェックSUM(27:2桁)は、実測値）　\r\n は、Serial.println で付加する。
const char bps96set[] = "$PMTK251,9600*17";     // （1秒中のデータ占有率 65.1%）(default)
const char bps192set[] = "$PMTK251,19200*22";   // （1秒中のデータ占有率 32.6%）
const char bps384set[] = "$PMTK251,38400*27";   // （1秒中のデータ占有率 16.3%）
const char bps576set[] = "$PMTK251,57600*2C";   // （1秒中のデータ占有率 10.9%）
const char bps1152set[] = "$PMTK251,115200*1F"; // （1秒中のデータ占有率 5.4%）
#endif

#ifdef OPTION_UBLOX_NEO
// NEO-M8N/7M（共通）のスピード指示とBatteryBackupRam への書込みUBXコマンド
// Serial.write(buf, len) でバイト数指定で送出する。
const char bps96set[] = {0xB5, 0x62, 0x06, 0x00, 0x14, 0x00, 0x01, 0x00,    // 28バイト
                         0x00, 0x00, 0xD0, 0x08, 0x00, 0x00, 0x80, 0x25,
                         0x00, 0x00, 0x07, 0x00, 0x03, 0x00, 0x00, 0x00,
                         0x00, 0x00, 0xA2, 0xB5
                        };
const char bps384set[] = {0xB5, 0x62, 0x06, 0x00, 0x14, 0x00, 0x01, 0x00,    // 28バイト
                          0x00, 0x00, 0xD0, 0x08, 0x00, 0x00, 0x00, 0x96,
                          0x00, 0x00, 0x07, 0x00, 0x03, 0x00, 0x00, 0x00,
                          0x00, 0x00, 0x93, 0x90
                         };
const char bps1152set[] = {0xB5, 0x62, 0x06, 0x00, 0x14, 0x00, 0x01, 0x00,    // 28バイト
                           0x00, 0x00, 0xD0, 0x08, 0x00, 0x00, 0x00, 0xC2,
                           0x01, 0x00, 0x07, 0x00, 0x03, 0x00, 0x00, 0x00,
                           0x00, 0x00, 0xC0, 0x7E
                          };
const char bpsBBRset[] = {0xB5, 0x62, 0x06, 0x09, 0x0D, 0x00, 0x00, 0x00,    // 21バイト
                          0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00,
                          0x00, 0x00, 0x03, 0x1D, 0xAB
                         };
#endif

// カウンター用レジスタ？
volatile uint64_t ui64Reg = 0;       // 符号無し(64bit) 周波数カウント用
//uint64_t ui64PicReg = 0;
//uint64_t ui64tmp;
uint8_t sumchk;

double dFrqReg = 0;         // 浮動小数点(64bit) 周波数計算用
double dAvgReg = 0;         // 浮動小数点(64bit) アベレージ計算用
double dMaxReg = 0;         // 浮動小数点(64bit) 最高値保持
double dMinReg = 0;         // 浮動小数点(64bit) 最小値保持
double dTmpReg = 0;         // 浮動小数点(64bit) 汎用
double dP_PReg = 0;         // 浮動小数点(64bit) p-p 振れ幅
int16_t AddCnt = 0;         // 加算回数
char fmtFrq[22];
double B4dFrqReg = 0;       // 浮動小数点(64bit) 周波数計算用
float IncDec = 0;           // 増減計算用

// （int内で使用=volatile を付ける）
volatile int16_t ppsCnt = 0;  // ゲートON時のPPSカウント用
uint8_t countDone = 0;        // カウント完了
volatile uint8_t gateON = 0;  // ゲートの状態
//uint8_t Running = 0;          // カウント中
volatile uint8_t reStart = 0; // PPS信号が欠落したときの再開フラグ

char tmpsec[3] = "00";  // ■臨時■　秒単位でのLOG送信に使用する。

#ifdef OPTION_USE_TEMPERATURE_SENSOR

#define Tsensor  PB1     // 温度センサー　LM61 or TMP36   or .......

char dosuu = 0xb0;        // 温度記号  「°」
char temptuText[10];      // 温度を文字列で記憶しておく
uint8_t temptuRead = 0;
// CSVのヘッダー（1行目）のデータ列（2020/09/07 現在）
// Num=連番, YYMMDD=年月日,HHMMSS=時分秒 ,StMin=最小衛星捕捉数, StMax=最大衛星捕捉数,
// Freq=周波数, (+-)=前差, gpsLoss=GPS信号ロス回数, ppsLoss=PPS信号ロス回数,
// picLoss=PICからSTM32に送られる計測データの転送エラー,TEMP=温度
// 文字列を追加したときは、配列の文字数に注意！！
char logheader[90] = "Num,YYMMDD,HHMMSS,StMin,StMax,Freq,(+-),gpsE,ppsE,picE,TEMP";
#else
char logheader[85] = "Num,YYMMDD,HHMMSS,StMin,StMax,Freq,(+-),gpsE,ppsE,picE";
#endif

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ◆◆◆◆◆電源投入時の初期画面◆◆◆◆◆
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
//
// □■■■□□■■■■□□■■■■■□■□□□■
// ■□□□■□■□□□■□■□□□□□■■□□■
// ■□□□■□■□□□■□■□□□□□■■□□■
// ■□□□■□■■■■□□■■■■□□■□■□■
// ■□□□■□■□□□□□■□□□□□■□□■■
// ■□□□■□■□□□□□■□□□□□■□□■■
// □■■■□□■□□□□□■■■■■□■□□□■

const char head1[] = "OCXO Calibrator (GPS)";
// const char verDate[] = "V50_62 & 2022/09/XX";   // ★★この行は文頭の変更履歴直下に置く★★

#ifdef OPTION_SSD1309_OLED

void Opening_Text()
{
  u8g2.clearBuffer();          // clear

  Text_setCursor(0, 0);     // 桁・行
  u8g2.print(head1);
  Text_setCursor(1, 1);   // 桁・行
  u8g2.print(verDate);
  Text_setCursor(3, 2);   // 桁・行
  // =======================================================================
#ifdef OPTION_AE_GNSS
  u8g2.print("AE_GNSS   OLED");
#endif  //end of OPTION_AE_GNSS
#if defined(OPTION_UBLOX_NEO) && defined(SEL_UBLOX_NEO_M8N)
  u8g2.print("NEO-M8N   OLED");
#endif
#if defined(OPTION_UBLOX_NEO) && defined(SEL_UBLOX_NEO_7M)
  u8g2.print("NEO-7M    OLED");
#endif  //end of OPTION_UBLOX_NEO
  Text_setCursor(0, 3);     // 桁・行
  u8g2.print("(c) 2020-2022, jlb.jp");
  FuncEnable = B00001111;
  func_print(0, "MENU");
  func_print(1, "GATE");
  func_print(2, "STRT");
  func_print(3, "CngD");
  u8g2.sendBuffer();          // OLEDに送信（描画する）
}
#endif  // end of  OPTION_SSD1309_OLED
// =======================================================================
#ifdef OPTION_ILI9341_TFT
void Opening_Text()
{
  display.fillScreen(ILI9341_BLACK);

  TFT_setCursor(1, 1);   // 桁・行
  u8g2_for_adafruit_gfx.print(head1);
  TFT_setCursor(2, 3);   // 桁・行
  u8g2_for_adafruit_gfx.print(verDate);
  TFT_setCursor(3, 4);   // 桁・行
#ifdef OPTION_AE_GNSS
  u8g2_for_adafruit_gfx.print("AE_GNSS  TFT");
#endif  //end of OPTION_AE_GNSS
#if defined(OPTION_UBLOX_NEO) && defined(SEL_UBLOX_NEO_M8N)
  u8g2_for_adafruit_gfx.print("NEO-M8N  TFT");
#endif
#if defined(OPTION_UBLOX_NEO) && defined(SEL_UBLOX_NEO_7M)
  u8g2_for_adafruit_gfx.print("NEO-7M   TFT");
#endif  //end of OPTION_UBLOX_NEO

#if defined(OPTION_USE_TEMPERATURE_SENSOR) && defined(OPTION_TEMPERATURE_SENSOR_LM61)
  TFT_setCursor(3, 5);   // 桁・行
  u8g2_for_adafruit_gfx.print("TEMP. LM61");
#endif
#if defined(OPTION_USE_TEMPERATURE_SENSOR) && defined(OPTION_TEMPERATURE_SENSOR_TMP36)
  TFT_setCursor(3, 5);   // 桁・行
  u8g2_for_adafruit_gfx.print("TEMP. TMP36");
#endif

  TFT_setCursor(1, 6);     // 桁・行
  u8g2_for_adafruit_gfx.print("(c) 2020-2022, jlb.jp");

}
#endif

// =======================================================================
#ifdef OPTION_OCXO_Vcon  // I2C 0x60
// Vcon
//
// ■□□□■□□□□□□□□□□□□□□□□□□□
// ■□□□■□□□□□□□□□□□□□□□□□□□
// ■□□□■□□■■■□□□■■■□□■■■■□□
// ■□□□■□■□□□■□■□□□■□■□□□■□
// ■□□□■□■□□□□□■□□□■□■□□□■□
// □■□■□□■□□□■□■□□□■□■□□□■□
// □□■□□□□■■■□□□■■■□□■□□□■□
static union {                // 共用体の記述。
  double dmHz;                // 倍精度浮動小数点(STM32)
  char intrC[8];              // 8バイトに分割
} UNI;

// MCP4725 & MCP4726
uint8_t VconI2C = 1;          // 0 = Vcon I2C 接続中、 4 = 未接続
uint8_t VconAUTO = 0;         // AUTO = 1 / MANUAL = 0
uint16_t VconValue = 4096;    //  4096 = P.ON後、最初の接続の判定に使用
uint16_t B4VconValue;
double tenkeyVal = 0;         // 10Keyの返り値
double gosa;
double tmpdac;                // 誤差に基づくDAC量の計算用
int8_t stable;                // 許容値内の積算を計数し、安定後の単発の許容外値を見送る

uint8_t getMODE;              // MCP4725 & MCP4726
uint16_t getDAC;              // MCP4725 & MCP4726
uint16_t getEPROM;            // MCP4725 & MCP4726



#ifdef OPTION_VCON_MCP4726

void vconDACset(uint16_t SetVal) {
  uint8_t value1 = 0x59;
  if (SetVal > 4095) SetVal = 4095;
  Wire.beginTransmission(Vcon_ADDRESS);
  Wire.write(value1);
  Wire.write(SetVal >> 4);
  Wire.write(SetVal << 4);
  Wire.endTransmission();
  delay(10);
}


void vconALLMEMset(uint16_t SetVal) {    // MCP4726
  /*
    // MCP4726 の場合は、このコマンドで動作モードをセットする。
    // このデバイス(MCP4726)を初めて使うときは、本コマンドを最初に
    // 実行し、デバイスを再起動すること。
    // また、他の箇所で以下のモードを変更する記述をしては成らない。
    // Write All Memory: (C2, C1, C0) = (0,1,1)
    // Vref1, Vref2 = 11 = VREF pin 2.50V(Buffered), 10 unBufferd
    // PD1, PD2 = 0, 0
    // Gain  = 1    1 = 2x (gain of 2)
    // 0111 1001 0x79（固定）
  */
  uint8_t value1 = 0x79;
  if (SetVal > 4095) SetVal = 4095;
  Wire.beginTransmission(Vcon_ADDRESS);
  Wire.write(value1);
  Wire.write(SetVal >> 4);
  Wire.write(SetVal << 4);
  Wire.endTransmission();
  delay(200);
}

uint8_t vconALLread() {        // MCP4726
  uint8_t ret;
  if (Wire.requestFrom(Vcon_ADDRESS, 6) == 6) {
    getMODE = Wire.read();
    getDAC = Wire.read();
    getDAC <<= 4;
    getDAC |= Wire.read() >> 4;
    getMODE = Wire.read();    // 2回目
    getEPROM = Wire.read();
    getEPROM <<= 4;
    getEPROM |= Wire.read() >> 4;
    ret = 0;
  } else {
    ret = 1;
  }
  return ret;   // 0 = OK
}

#else     // MCP4725

void vconDACset(uint16_t SetVal) {
  Wire.beginTransmission(Vcon_ADDRESS);
  Wire.write((SetVal >> 8) & 0x0F);
  Wire.write(SetVal);
  Wire.endTransmission();
  delay(10);
}
void vconALLMEMset(uint16_t SetVal) {    // MCP4725
  uint8_t value1 = 0x60; // Write DAC Register and EEPROM: (C2, C1, C0) = (0,1,1)
  if (SetVal > 4095) SetVal = 4095;
  Wire.beginTransmission(Vcon_ADDRESS);
  Wire.write(value1);
  Wire.write(SetVal >> 4);
  Wire.write(SetVal << 4);
  Wire.endTransmission();
  delay(200);
}

uint8_t vconALLread() {        // MCP4725
  uint8_t ret;
  if (Wire.requestFrom(Vcon_ADDRESS, 5) == 5) {
    getMODE = Wire.read();
    getDAC = Wire.read();
    getDAC <<= 4;
    getDAC |= Wire.read() >> 4;
    getEPROM = Wire.read();
    getEPROM <<= 8;
    getEPROM |= Wire.read();
    ret = 0;
  } else {
    ret = 1;
  }
  return ret;   // 0 = OK
}


#endif

uint8_t vconStartCHK() {
  uint8_t ret = 0;
  //  Wire.begin();    // （仮）ストール対策
  //  delay(20);

  // Vcon のI2C接続(有無)確認 if (ret == 0) 接続中なら
  // VconのEEPROMからDACの初期値を読み出してセット
  // 返り値＝０なら、接続中
  // 以後のI2Cコマンドの発出許可にも使用
  Wire.beginTransmission(Vcon_ADDRESS);
  ret = Wire.endTransmission();

  if (ret == 0) {
    if (vconALLread() == 0) {

      if (VconValue == 4096) VconValue = getDAC; // 今まで未接続だったらROM読み込み

    } else {
      // VconのEEPROMが読めなかったらエラー表示（接続OKの時、在りえないかも？）
      errDisp("#00 vconStartCHK   ");   // Max19文字
    }
  }
  return ret;   // 0 = Vcon 接続中
}

// =======================================================================

#endif

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// カウンター用レジスタのゼロクリヤー
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void RegZero() {
  ui64Reg = 0;
  dFrqReg = 0;
  dAvgReg = 0;
  dMaxReg = 0;
  dMinReg = 0;
  dTmpReg = 0;
  dP_PReg = 0;
  AddCnt = 0;
  ppsCnt = 0;
  gateON = 0;
  gpsLoss = 0;
  ppsLoss = 0;
  tIntCount3 = 0;
  //Running = 0;  // 未使用？
  B4dFrqReg = 0;       // 浮動小数点(64bit) 周波数計算用
  IncDec = 0;           // 増減計算用
  picLoss = 0;      // PICデータ受信エラー回数

  // 重要　入れないとPICからの古いデータが残る場合が有る。
  while (Serial3.available()) {
    PICchar = Serial3.read();
    delay(2);   // 9600bps時に1文字1.04mSsec
  }
  PICok = 0;
  satReset();
}

void satReset() {
  SATmax = 0;       // 最大衛星数を初期セット
  SATmin = 99;       // 最小衛星数を初期セット
}

// GATE用D-F/Fのリセット    V41時は反転(LOW アクティブ)
void PortReset() {
  digitalWrite(coRSET, LOW);
  delay(1);                         // 1mS
  digitalWrite(coRSET, HIGH);
  delay(1);                         // 1mS
}
// PICのリセット
void PicReset() {
  digitalWrite(coPicREST, HIGH);
  delay(1);                         // 1mS
  digitalWrite(coPicREST, LOW);
  delay(10);                         // 10mS
}

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// PPS 割り込みのカウント
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void intPps() {     // PPS 割込み

  if (tIntCount3 >= 12) {        // 前回のPPS信号から1.2秒以上経過している。

    digitalWrite(coGSET, LOW);
    gateON = 0;                 // GATEの状態OFF
    reStart = 1;                // PPS信号が欠落した後の再開フラグ SET

  } else {
    // 正常処理
    if (digitalRead(coGSET) == HIGH) {
      gateON = 1;     // GATEの状態ON

      // 最終手前のPPS(次のPPSでカウント完了)なら、GSETをOFFする。
      if (ppsCnt <= 1) digitalWrite(coGSET, LOW);

      ppsCnt--;
    } else {
      gateON = 0;     // GATEの状態OFF

      if (Counting == 1) countDone = 1;  // カウント完了
      ppsCnt = 0;     // PPS Count Zero
      // delay(1);     // delay() は機能しない。

    }
  }
  tIntCount3 = 0;       // PPS信号の間隔を監視するカウンタ
}


// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
//  PPS パラメータセット　＜＜共通使用＞＞
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void PpsSet() {
  switch (gatePRM) {
    case 1:
      ppsCnt = 10;      // 11秒／回
      break;
    case 2:
      ppsCnt = 100;     // 1分41秒／回
      break;
    case 3:
      ppsCnt = 1000;     // 16分41秒／回
      break;
    case 4:
      ppsCnt = 10000;     // 2時間46分41秒／回
      break;
    default:
      ppsCnt = 1;
      break;
  }
}

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// double の数値を周波数表記に変換する
//
// 小数点以下の位をGATE時間に応じて可変(Variable)する(2020/04/15)
//
//      01234567890123456
//      98765432.1234       ----sprintf 後の状態
//      98 765 432.1234Hz   ----書式変換後のイメージ
//                              10MHz台まで表示する。桁の先頭がzeroの場合は空白。
//                              3桁カンマは付けずに空白。少数以下4桁まで。
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void dFRQtoStrV(char* fmttmp, double freqIN) {
  double dtmp;
  char strtmp[22];
  int8_t i, j, ln;

  dtmp = freqIN;

  switch (gatePRM) {
    case 1:
      sprintf(strtmp, "%10.1lf   ", dtmp);
      break;
    case 2:
      sprintf(strtmp, "%11.2lf  ", dtmp);
      break;
    case 3:
      sprintf(strtmp, "%12.3lf ", dtmp);
      break;
    case 4:
      sprintf(strtmp, "%13.4lf", dtmp);
      break;
    default:
      sprintf(strtmp, "%10.1lf   ", dtmp);    // 小数点を表示するために
      strtmp[9] = ' ';
      break;
  }
  ln = strlen(strtmp) + 2;
  j = 0;
  for (i = 0; i < ln; i++) {
    switch (i) {
      case 2:
      case 5:
        fmttmp[j] = ' ';
        j++;
        fmttmp[j] = strtmp[i];
        j++;
        break;
      default:
        fmttmp[j] = strtmp[i];
        j++;
        break;
    }
  }
  fmttmp[15] = 0x0;
}

#ifdef OPTION_SSD1309_OLED

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// double の数値を周波数表記に変換する
//
// アベレージの表示専用にする(2020/04/15)
//
//      01234567890123456
//      98765432.1234       ----sprintf 後の状態
//      98 765 432.1234Hz   ----書式変換後のイメージ
//                              10MHz台まで表示する。桁の先頭がzeroの場合は空白。
//                              3桁カンマは付けずに空白。少数以下4桁まで。
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void dFRQtoStr(char* fmttmp, double freqIN) {
  double dtmp;
  char strtmp[22];
  int8_t i, j, ln;

  dtmp = freqIN;

  sprintf(strtmp, "%13.4lf", dtmp);

  ln = strlen(strtmp) + 2;
  j = 0;
  for (i = 0; i < ln; i++) {
    switch (i) {
      case 2:
      case 5:
        fmttmp[j] = ' ';
        j++;
        fmttmp[j] = strtmp[i];
        j++;
        break;
      default:
        fmttmp[j] = strtmp[i];
        j++;
        break;
    }
  }

  fmttmp[15] = 0x0;

}
#endif

#ifdef OPTION_ILI9341_TFT

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// double の数値を周波数表記に変換する
//
// アベレージの表示専用にする(2020/04/15)
//
//      01234567890123456
//      98765432.1234       ----sprintf 後の状態
//      98 765 432.1234Hz   ----書式変換後のイメージ
//                              10MHz台まで表示する。桁の先頭がzeroの場合は空白。
//                              3桁カンマは付けずに空白。少数以下*4桁まで。
//                              *GATE=1-^4 は、少数以下5桁(V50_54b)■TFTのみ■
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void dFRQtoStr(char* fmttmp, double freqIN) {
  double dtmp;
  char strtmp[22];
  int8_t i, j, ln;

  dtmp = freqIN;

  if (gatePRM <= 3) {
    sprintf(strtmp, "%13.4lf", dtmp);
  } else {
    sprintf(strtmp, "%14.5lf", dtmp);   // GATE=1-^4 は、少数以下5桁
  }
  ln = strlen(strtmp) + 2;
  j = 0;
  for (i = 0; i < ln; i++) {
    switch (i) {
      case 2:
      case 5:
        fmttmp[j] = ' ';
        j++;
        fmttmp[j] = strtmp[i];
        j++;
        break;
      default:
        fmttmp[j] = strtmp[i];
        j++;
        break;
    }
  }

  if (gatePRM <= 3) {
    fmttmp[15] = 0x20;  // GATE=1-^4 以外は、少数以下5桁目をスペースで埋める
  }
  fmttmp[16] = 0x0;     // GATE=1-^4 は、少数以下5桁

}
#endif

/*************************************************************
   受信した文字データ(SUM付)を uint64_t に変換し、SUM Check をする。
   戻り値 uint64_t  <==    char inHex[20];

   引数にSUM Checkの結果(sumFlag)を戻すため、アドレス渡しで関数を定義
   使用時は、'&' を付ける。　hex16Bto64bit( &sumFlag, char *inHex)

   sumFlag = 0;エラーなし
   sumFlag = 1;エラー有り（発生はしない予想）
 *************************************************************/
uint64_t hex16Bto64bit( uint8_t * sumFlag, char *inHex) {
  int8_t i;
  uint8_t tmpB1, tmpB2, tmpsum = 0;
  uint64_t out64val = 0;
  uint16_t sum = 0;
  for (i = 15; i >= 0; i--) { // 文字配列の後から取り出す
    tmpB1 = inHex[i];
    sum += tmpB1;
    if (tmpB1 > 0x39) {
      tmpB2 = tmpB1 - 55;
      tmpB2 &= 0x0F;
    } else {
      tmpB2 = tmpB1 & 0x0F;
    }
    out64val = out64val << 4;
    out64val = out64val | tmpB2;
  }
  for (i = 17; i >= 16; i--) {  // Check Sum
    tmpB1 = inHex[i];
    if (tmpB1 > 0x39) {
      tmpB2 = tmpB1 - 55;
      tmpB2 &= 0x0F;
    } else {
      tmpB2 = tmpB1 & 0x0F;
    }
    tmpsum <<= 4;
    tmpsum |= tmpB2;
  }

  if (tmpsum == (uint8_t)sum) { // LSB側8bitを照合
    *sumFlag = 0;
  } else {
    *sumFlag = 1;
  }
  return out64val;
}

////////////////////////////////////////////////////////////
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ◆◆◆◆◆タイマー割込み◆◆◆◆◆
// 0.1秒ごとに数を増分する　　
// OLED の表示サイクル用に使用する　       0.01Sec(timer1)
// PPS信号が欠落したことの検出に使用する。 0.1Sec(timer3)
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

void handle_timer1() {
  tIntCount1++;
  if (tIntCount1 > 10000) tIntCount1 = 0;    // 10000 x 0.01S = 100 Sec 以上はクリアー
}
// 早押し抑制用に使用する。
// 通常は、5000(5sec)から9000(9sec)までをカウントする。（あえて範囲外を）
// 早押し抑制期間は、0 からカウントUPして指定秒数(5sec以内)まで指定する。
void handle_timer2() {
  tIntCount2++;        // 1mSec毎にカウント
  if (tIntCount2 > 9000) tIntCount2 = 5000;    // 9000 x 1mS = 9 Sec 超えは5000
}
void handle_timer3() {
  if (gateON == 1)  tIntCount3++;        // GATEがONの間、0.1Sec毎にカウント
}

#ifdef OPTION_USE_TEMPERATURE_SENSOR
/*
  // ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
  // 温度測定　アナログ入力
  // 最初のセンス値は、そのまま変換する。
  // 2回目以降は、前回の最初の値・あるいは平均値を１回として、指定回数の
  // 平均値を求める。（2020/05/23）
  //
  // 2020/10/14: 両センサーを実測定で調整を実施。27℃から31℃で平行的に揃う(Hドライヤー使用)
  // 2020/10/17:  約25度で調整。OLEDは温度センサーを外す。
  // 2020/10/23:  秋月のSHT31(I2C)を基準にして「電気ひざ掛け」を使用して確認。
  //              （室温25℃と加熱33℃辺りで確認）
  // 2020/10/24:  温度係数 ＋10.0mV/℃ のままで、オフセット部で±補正をする。
  // ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
*/
float addSENSOR = 0;
int8_t countSENSOR = 0;
char temptVal[22];
uint16_t ana_tempt;

void tempt_chk(void) {

  ana_tempt = analogRead(Tsensor);
  // =======================================================================
#ifdef OPTION_TEMPERATURE_SENSOR_LM61       // 小TFT機でテスト
  // LM61     : offset = 600mV, Scale factor = +10mV/C,  ±3℃(@25℃)
  // センサー個別の調整は最後の値で調整する。
  float nowSENSOR = (ana_tempt *  3.3 / 4095.0 - 0.6 ) * 100 - 1.35;  // (0.021:18℃にて)
#endif  // end OPTION_TEMPERATURE_SENSOR_LM61

  // -----------------------------------------------------------
#ifdef OPTION_TEMPERATURE_SENSOR_TMP36        // 大TFT機でテスト
  // TMP36  : offset = 500mV, Scale factor = +10mV/C,  ±3℃(@25℃)
  // センサー個別の調整は最後の値で調整する。
  float nowSENSOR = (ana_tempt *  3.3 / 4095.0 - 0.5 ) * 100 - 1.5;  //  個体:No1
#endif  // end OPTION_TEMPERATURE_SENSOR_TMP36

  // =======================================================================
#ifdef SENSOR_TEST_MODE
  // ADC値をそのままmV単位で出力する。
  float nowSENSOR = ana_tempt *  3.3 / 4095.0 * 1000;  // デバッグ用

#endif

  // =======================================================================
  if (countSENSOR == 0) {
    addSENSOR = nowSENSOR;

#ifdef SENSOR_TEST_MODE
    sprintf(temptVal, "%7.1f", addSENSOR);//  "%+5.1f"  mV
#else
    sprintf(temptVal, "%+5.1f", addSENSOR);//  "%+5.1f"  C
#endif

    countSENSOR = 1;
  } else {
    addSENSOR += nowSENSOR;
    countSENSOR++;
    if (countSENSOR >= 3) {
      addSENSOR /= countSENSOR;

#ifdef SENSOR_TEST_MODE
      sprintf(temptVal, "%7.1f", addSENSOR);//  "%+5.1f"
#else
      sprintf(temptVal, "%+5.1f", addSENSOR);//  "%+5.1f"
#endif

      countSENSOR = 1;
    }
  }
}

#endif  // ■■■■　end #ifdef OPTION_USE_TEMPERATURE_SENSOR ■■■■■■■■■■

#ifdef OPTION_ILI9341_TFT
void fonSet_BIG() {
  u8g2_for_adafruit_gfx.setFont(u8g2_font_profont29_mr);  //  W16xH29 (A19) _mn数字だけ
  //     u8g2_for_adafruit_gfx.setFont(u8g2_font_courB18_tr);  //  W17xH26 (A19) _mn数字だけ
}

void fonSet_MID() {
  u8g2_for_adafruit_gfx.setFont(u8g2_font_profont22_mf);  // W12xH22  A14
  //      u8g2_for_adafruit_gfx.setFont(u8g2_font_courB14_tf);  // W12xH20  A11
}

//////////////////////////////////////////////////////////////////////////////
// 特定フォント専用　(u8g2_font_profont22_mr)　14 Pixel Height, 12x22
// ペンタッチ押して放した時の文字位置を返す。
// 返り値を引数を使って返すため、ポインター指定を使う(受取：*abc  呼ぶ時：&abc )
// 押されて無かった時は、留まらず return 0; を返す。
// 押された時は、return 1;  と　cc桁,  ll行,  に（文字換算の）位置情報を返す。
//
// ■■重要 Touchセンサーのxy情報を読み込んだ後は、すぐに別変数に移す■■
// xy情報は続けてすぐに別変数に移さないと、yの読取値が化ける。
//////////////////////////////////////////////////////////////////////////////
int8_t touchCCxLL(uint16_t *cc, uint16_t *ll) {
  uint16_t xtmp;
  uint16_t ytmp;
  uint16_t ptmp;
  float ftmp1;
  int16_t ftmp2;

  if (ts.read_XY(xy)) { // true = 1 / false = 0
    xtmp = xy[0];  // ■■重要 Touchセンサーのxy情報を読み込んだ後は、すぐに別変数に移す■■
    ytmp = xy[1];  // ■■重要 Touchセンサーのxy情報を読み込んだ後は、すぐに別変数に移す■■

#ifdef BOARD_V5_BUZZER
    beepms(150);    // キー動作のビープ音
#endif

    while (1) {   // この行以下4行を // コメントにすれば、押した時の取り込み。
      if ( !ts.read_XY(xy)) { // 放すのを待つ。
        break;
      }
    }

    if (xtmp <= tchXmin) xtmp = tchXmin + 1;  // タッチ位置が最小座標より小なら
    if (xtmp >= tchXmax) xtmp = tchXmax - 1;  // タッチ位置が最小座標より大なら
    ftmp1 = (float)(xtmp - tchXmin) / (float)(tchXmax - tchXmin); // 横、左から何％の位置
    ftmp2 = ftmp1 * (float)pixWidth - 1; // ＊桁の位置調整
    if (ftmp2 <= 0) ftmp2 = 1;      // ＊最左桁の調整
    if (ftmp2 >= 311) ftmp2 = 311;  // ＊最右桁の調整(27桁目を無くす)
    *cc = ftmp2 / 12;
    //
    if (ytmp <= tchYmin) ytmp = tchYmin + 1;  // タッチ位置が最小座標より小なら
    if (ytmp >= tchYmax) ytmp = tchYmax - 1;  // タッチ位置が最小座標より大なら
    ftmp1 = (float)(ytmp - tchYmin) / (float)(tchYmax - tchYmin); // 縦、上から何％の位置
    ftmp2 = ftmp1 * (float)pixHeight - 5; // ＊行の位置調整
    if (ftmp2 <= 0) ftmp2 = 1;      // ＊最上位行の調整
    if (ftmp2 >= 224) ftmp2 = 224;  // ＊最下行の調整(10行目を無くす)
    *ll = ftmp2 / 25;
    return 1;
  } else {
    return 0;
  }
}

#endif
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// beep sub
#ifdef BOARD_V5_BUZZER
void beepms(uint32_t msec)
{
  digitalWrite(BUZZER, HIGH);
  delay(msec);
  digitalWrite(BUZZER, LOW);
}
#endif

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
//
// □■■■□□■■■■■□■■■■■□■□□□■□■■■■□
// ■□□□■□■□□□□□□□■□□□■□□□■□■□□□■
// ■□□□□□■□□□□□□□■□□□■□□□■□■□□□■
// □■■■□□■■■■□□□□■□□□■□□□■□■■■■□
// □□□□■□■□□□□□□□■□□□■□□□■□■□□□□
// ■□□□■□■□□□□□□□■□□□■□□□■□■□□□□
// □■■■□□■■■■■□□□■□□□□■■■□□■□□□□
//
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

void setup(void) {
  disableDebugPorts();      // JTAG_Debug を使わないことで、PA15,PB3,PB4を使えるようにする。

#ifdef OPTION_SSD1309_OLED
  pinMode(SWa_In, INPUT_ANALOG);
#endif

  pinMode(ciGATE2, INPUT);    // 未使用（2020/06/11現在）
  pinMode(coGSET, OUTPUT);
  pinMode(ciGATE, INPUT);     // 未使用（2020/06/11現在）
  pinMode(coPicREST, OUTPUT);     // PIC AllRESET
  pinMode(coRSET, OUTPUT);      // Device Regi
  pinMode(ciPPS, INPUT);
  pinMode(LED_PIN, OUTPUT);
  pinMode(LED_OUT, OUTPUT);
  digitalWrite(coGSET, LOW);

#ifdef OPTION_USE_TEMPERATURE_SENSOR
  pinMode(Tsensor, INPUT_ANALOG);
#endif

#ifdef BOARD_V5_BUZZER
  pinMode(BUZZER, OUTPUT);
#endif

#ifdef SOFT_RESET // ================= PB0 HIGH ==> RESET
  pinMode(RESET_PORT, OUTPUT);
#endif            // =================

  digitalWrite(LED_PIN, HIGH);
  digitalWrite(LED_OUT, HIGH);


  PortReset();     // GATE用D-F/Fの初期化
  PicReset();     // PICの初期化

  attachInterrupt(ciPPS, intPps, RISING);    // PPS 割り込み関数の指定。RISING
  // attachInterrupt(ctQ7, intPictest, FALLING);    // 臨時テスト

  RegZero();      // レジスターの初期化

  // =======================================================================
#ifdef OPTION_SSD1309_OLED

  SPI.begin();
  u8g2.begin();
  //  u8g2.setFont(u8g2_font_6x12_tr);  // 7bit 等幅、21文字、5行(行12dot)、●●これが良さそう●●
  u8g2.setFont(u8g2_font_6x12_t_symbols); // ▶記号を使用するために、こちらのフォントを使う。
  u8g2.setFontMode(1);  /* activate transparent font mode */
#endif
  // --------------------------------
#ifdef OPTION_ILI9341_TFT

  display.begin();
  display.setRotation(3);
  display.fillScreen(ILI9341_BLACK);

  u8g2_for_adafruit_gfx.begin(display);                 // connect u8g2 procedures to Adafruit GFX
  // u8g2_for_adafruit_gfx.drawLine(0, 0, 320, 240);

  u8g2_for_adafruit_gfx.setFontMode(0);                 // use u8g2 none transparent mode
  u8g2_for_adafruit_gfx.setFontDirection(0);            // left to right (this is default)
  u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);      // apply Adafruit GFX color

  //  u8g2_for_adafruit_gfx.setFont(u8g2_font_profont22_mf);  //  OK (A14) Pixel Height, W12xH22
  fonSet_MID();


  ts.begin(); //Begin TouchScreen.

#endif
  // =======================================================================

  Wire.begin();           // I2C EEPROM用、他

  Opening_Text();         // 初期画面を表示

  // ■■■■■■■■■■■■■■■
  // EEPROM読み出し・初期化
  // ■■■■■■■■■■■■■■■
  uint8_t tmp;
  tmp = eeprom_read(0); // ROMからの読み出し Address = 0
  if (tmp >= 0 && tmp <= 2) {   // パラメータが範囲内   0 ～ 2
    bpsPRM = tmp;               //  = PRMへ読み出し値セット
  } else {                      // パラメータが範囲外 = ROM新品または、故障??
    eeprom_write(0, 0);               // 0 = (default) を書込み
  }
  tmp = eeprom_read(1); // ROMからの読み出し Address = 1
  if (tmp >= 0 && tmp <= 2) {
    hostPRM = tmp;
  } else {
    eeprom_write(1, 0);
  }
  tmp = eeprom_read(2);   // ROMからの読み出し Address = 2
  if (tmp >= 0 && tmp <= 2) {
    gatePRMon = tmp;
    gatePRM = tmp;
  } else {
    eeprom_write(2, 0);
  }

  // ■■■■■■■■■■■■■■■
  // ==========================================
#ifdef OPTION_ILI9341_TFT
  uint16_t tmp16;

  tmp16 = eeprom_read_uint16(3);

  if ((200 < tmp16) && (tmp16 < 550)) {   // ROMに書かれていたら
    tchXmin = tmp16;
    tchYmin = eeprom_read_uint16(5);
    tchXmax = eeprom_read_uint16(7);
    tchYmax = eeprom_read_uint16(9);
  } else {                                // 初めての書込み
    tchXmin = InitXmin;
    tchYmin = InitYmin;
    tchXmax = InitXmax;
    tchYmax = InitYmax;
    eeprom_write_uint16(3, tchXmin);
    eeprom_write_uint16(5, tchYmin);
    eeprom_write_uint16(7, tchXmax);
    eeprom_write_uint16(9, tchYmax);
  }

  // ==========================================
  // ■■■■■■■■■■■■■■■
#ifdef OPTION_OCXO_Vcon  // I2C 0x60

  eeprom_read_double(0x10);     // dmHzの読み出し

  if (!((UNI.dmHz < 10) && (UNI.dmHz >= 0))) {   // 2022/09/16:条件式を改定(ROMが初期値の時のみ該当)

    UNI.dmHz = 0.5;       // 仮の値
    eeprom_write_double(0x10, UNI.intrC);     // 書込んでおく
  }
  VconAUTO = eeprom_read(0x18);  // AUTO/MANUAL MODEの読み出し
  if (VconAUTO > 1) {
    VconAUTO = 0;
    eeprom_write(0x18, VconAUTO);      // 後でエラー判定を入れる
  }

  // Logのヘッダーに文字列追加      // デバッグ中★★★★
  strcat(logheader, ",Vcon,STABLE,(1LSB)");     // Logのヘッダーに文字列追加

  VconI2C = vconStartCHK();       // Vcon のPowerON時の接続チェック

#endif  // OPTION_OCXO_Vcon

#endif    // end OPTION_ILI9341_TFT

  // ■■■■■■■■■■■■■■■

  // ★★★タイマー割込み 1 の初期設定 START★★★▼▼▼▼▼▼(10mSec)汎用 0～100秒　ソフトリセットで
  Timer1.pause();                   // タイマー停止
  Timer1.setPrescaleFactor(7200);   // 分周値set 72MHz ÷ 7200 = 10KHz
  Timer1.setOverflow(100);          // 10KHz x 100 = 0.01秒(10mS)でオーバーフォロー割込み
  // ------
  Timer1.attachInterrupt(           // 割り込みハンドラの登録
    TIMER_UPDATE_INTERRUPT,         // 条件は、カウンターオーバーフロー更新時
    handle_timer1);                  // 呼び出す関数
  // ------
  Timer1.setCount(0);               // カウンタを0に設定
  Timer1.refresh();                 // タイマ更新
  Timer1.resume();                  // タイマースタート
  // --------------------------------------
  // ★★★タイマー割込み ２ の初期設定 START★★★▼▼▼▼▼▼(1mSec) STRT」早押し抑制
  Timer2.pause();                   // タイマー停止
  Timer2.setPrescaleFactor(7200);   // 分周値set 72MHz ÷ 7200 = 10KHz (uint32)
  Timer2.setOverflow(10);           // 10KHz x 10 = 1mSsecでオーバーフォロー割込み (uint32)
  // ------
  Timer2.attachInterrupt(           // 割り込みハンドラの登録
    TIMER_UPDATE_INTERRUPT,         // 条件は、カウンターオーバーフロー更新時
    handle_timer2);                  // 呼び出す関数
  // ------
  Timer2.setCount(0);               // カウンタを0に設定
  Timer2.refresh();                 // タイマ更新
  Timer2.resume();                  // タイマー２スタート
  // ★★★タイマー割込みの初期設定 END  ★★★▲▲▲▲▲▲
  // --------------------------------------
  // --------------------------------------
  // ★★★タイマー割込み 3 の初期設定 START★★★▼▼▼▼▼▼(100mSec) PPS信号の欠落検知用
  Timer3.pause();                   // タイマー停止
  Timer3.setPrescaleFactor(7200);   // 分周値set 72MHz ÷ 7200 = 10KHz (uint32)
  Timer3.setOverflow(1000);          // 10KHz x 1000 = 0.1秒でオーバーフォロー割込み (uint16)
  // ------
  Timer3.attachInterrupt(           // 割り込みハンドラの登録
    TIMER_UPDATE_INTERRUPT,         // 条件は、カウンターオーバーフロー更新時
    handle_timer3);                  // 呼び出す関数
  // ------
  Timer3.setCount(0);               // カウンタを0に設定
  Timer3.refresh();                 // タイマ更新
  Timer3.resume();                  // タイマー２スタート
  // ★★★タイマー割込みの初期設定 END  ★★★▲▲▲▲▲▲
  // --------------------------------------

  Serial.begin(9600);     // USBシリアル(PC側) STM32は、速度指定しても最高速度で動く。
  //while (!Serial);         // Arduino UNO 書込み後のUSBシリアルの解放を待つ。
  //while (!Serial.isConnected()) delay(10);  // STM32 USB-シリアル用 isConnected()を使う。
  /*
     Serial1は、GPS受信機の初期速度が9600bpsのため、9600(default)は、必ず指定できる
     ようにしておく必要がある。その他の速度は変更可能であるが、GPSからのデータを取り
     こぼす可能性が有るため、できるだけ高速(115200)で使用する方が良い。
  */

  // ここでROMからリードしたbpsをセット
  switch (bpsPRM) {
    case 1:
      Serial1.begin(38400);              // Serial1(PA9 PA10)
      break;
    case 2:
      Serial1.begin(115200);              // Serial1(PA9 PA10)
      break;
    default:
      Serial1.begin(9600);               // Serial1(PA9 PA10)
      break;
  }

  /*
     PICからのカウントデータ受信は、38400bps固定
  */

  Serial3.begin(38400); // PIC側と合わせる

  delay(3000);

#ifdef OPTION_ILI9341_TFT
  display.fillScreen(ILI9341_BLACK);

  tIntCount2 = 0;   // 早押し抑制
  formDisp = 0;
  funcDisp = 0;

#endif

}

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
//
// ■□□□□□□■■■□□□■■■□□■■■■□
// ■□□□□□■□□□■□■□□□■□■□□□■
// ■□□□□□■□□□■□■□□□■□■□□□■
// ■□□□□□■□□□■□■□□□■□■■■■□
// ■□□□□□■□□□■□■□□□■□■□□□□
// ■□□□□□■□□□■□■□□□■□■□□□□
// ■■■■■□□■■■□□□■■■□□■□□□□
//
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// OLEDを使用時のLoop一回の処理時間（タイマーで実測:2020/04/17）
// スタンバイ時　　：Max16mS(1mS～12mS)
// スタート時(瞬間)：Max194mS
// 計測中　　　　　：Max16mS(1mS～15mS)
// ストップ時(瞬間)：Max239mS
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

void loop(void) {

  uint8_t a0a;
#ifdef OPTION_SSD1309_OLED
  // digitalWrite(LED_OUT, !digitalRead (ciPPS));   // LED_PIN を PPS入力に同期させる
#endif

  if (digitalRead (ciPPS) == HIGH) dispOnce = 0;   // 1回表示／秒のためのクリヤー

  // =======================================================================
#ifdef OPTION_SSD1309_OLED

  switch (buttonSW()) {     // キーが押されているか確認。

    // パラメータ編集メニューへ
    case 10:              // 長押し(20)F0 ===>短押し(10)へ変更(2020/04/16)
      if ((FuncEnable & 0x08) > 0) {    // 許可フラグ
        jobPRM = jobSelect(jobPRM);        // ★★★設定処理等の選択画面に行く
        u8g2.clearBuffer();   // ★★★設定処理等の選択画面からの帰りにクリアー
        Text_setCursor(1, 1);   // 桁・行
        u8g2.print(" Config MENU END ");
        u8g2.sendBuffer();            // OLEDに送信（描画する）
        dispOnce = 0;
        delay(500);
      }
      break;
    // GATE time切換
    case 11:          // 短押しF1、長押しは無効として処理。
      if ((FuncEnable & 0x04) > 0) {    // 許可フラグ

        gatePRM++;
        if (gatePRM > 3) gatePRM = 0;    // GATE time を変更、通常0～3



        /* PICをリセットするボタンを廃止する。(2022/03/23)
                 if (dispNo == 0) {
                  gatePRM++;
                  if (gatePRM > 3) gatePRM = 0;    // GATE time を変更、通常0～3

                  //        gatePRMdisp(gatePRM);    // GATE時間の表示
                  //        u8g2.sendBuffer();          // OLEDに送信（描画する）ここだけは単独で送出。

                } else {
                  // CLEAR  第２画面
                  PicReset();   // PICをリセットする。（INT0割り込みで）
                  delay(1);
                }
        */
      }
      break;
    // GATE time切換
    case 21:          // 長押しF1で、10,000secの設定
      if ((FuncEnable & 0x04) > 0) {    // 許可フラグ
        if (dispNo == 0) {
          gatePRM = 4;
        }
      }
      break;
    // 計測スタート・ストップ
    case 12:          // 短押しF2、長押しは無効として処理。
      if ((FuncEnable & 0x02) > 0) {    // 許可フラグ
        if (Counting == 0) {
          Counting = 1;     // カウントスタート
          CountTime = 0;                   // V50_54b
          PortReset();
          PicReset();
          RegZero();

          // ゲート時間により、
          PpsSet();                       // gatePRMからppsCntへ時間をセット

          digitalWrite(coGSET, HIGH);
        } else {
          digitalWrite(coGSET, LOW);
          Counting = 0;     // カウントストップ
          // CountTime = 0;               // スタートで処理する。V50_54b
          delay(50);
        }
      }
      break;
    // 表示画面の切替
    case 13:          // 短押しF3、長押しは無効として処理。
      if ((FuncEnable & 0x01) > 0) {    // 許可フラグ
        if (dispNo == 0) {
          dispNo = 1;     // 2画面へ変更
        } else {
          dispNo = 0;     // 1画面へ変更
        }
      }
      break;
  } // switch() end
#endif

#ifdef OPTION_ILI9341_TFT

  a0a = touchSW();

  switch (a0a) {
    //  switch (touchSW()) {     // キーが押されているか確認。

    // パラメータ編集メニューへ
    case 10:
      if ((FuncEnable & 0x08) > 0) {    // 許可フラグ
        jobPRM = jobSelect(jobPRM);        // ★★★設定処理等の選択画面に行く
        display.fillScreen(ILI9341_BLACK);
        TFT_setCursor(1, 4);   // 桁・行
        u8g2_for_adafruit_gfx.print(" Config MENU END ");

        delay(500);
        display.fillScreen(ILI9341_BLACK);

        tIntCount2 = 0;   // 早押し抑制
        formDisp = 0;
        funcDisp = 0;
        dispOnce = 0;
      }
      break;
    // GATE time切換
    case 11:          // 短押しF1、長押しは無効として処理。
      if ((FuncEnable & 0x04) > 0) {    // 許可フラグ
        gatePRM++;
        if (gatePRM > 4) gatePRM = 0;    // GATE time を変更、通常0～3

        gatePRMdisp(gatePRM);    // GATE時間の表示

      }
      break;

    // 計測スタート
    case 12:          // 短押しF2、長押しは無効として処理。
      if ((FuncEnable & 0x02) > 0) {    // 許可フラグ


        if (Counting == 0) {
          Counting = 1;     // カウントスタート
          CountTime = 0;                   // V50_54b
          PortReset();
          PicReset();
          RegZero();

#ifdef OPTION_OCXO_Vcon
          stable = 0;     // Vconで使用、実験中。■★■★■★▼(TFT)
#endif

          // ゲート時間により、
          PpsSet();                       // gatePRMからppsCntへ時間をセット

          digitalWrite(coGSET, HIGH);

          //          display.fillScreen(ILI9341_BLACK);
          func_fillCler();    // 上記全画面消去から、ファンクション領域のみの消去に
          formDisp = 0;
          funcDisp = 0;
        }
      }

      break;
    // カウントストップ
    case 13:          // 短押しF3、長押しは無効として処理。
      if ((FuncEnable & 0x01) > 0) {    // 許可フラグ

        digitalWrite(coGSET, LOW);
        Counting = 0;     // カウントストップ
        // CountTime = 0;               // スタートで処理する。V50_54b
        tIntCount2 = 0;   // 早押し抑制
        // delay(50);    // BBB

        //        display.fillScreen(ILI9341_BLACK);
        func_fillCler();    // 上記全画面消去から、ファンクション領域のみの消去に
        formDisp = 0;
        funcDisp = 0;
      }
      break;

  } // switch() end

#endif
  // =======================================================================

  // ■■■もし、PPS信号が欠落したときの再開フラグが立ったら。■■■
  if (reStart != 0) {

    ppsLoss++;      // PPSのロス回数を＋１

    PortReset();    // GATE用D-F/Fのリセット

    PpsSet();                       // gatePRMからppsCntへ時間をセット
    ui64Reg = 0;                    // クリヤー
    reStart = 0;
    digitalWrite(coGSET, HIGH);

  }

  // ■■■PPSが長時間停止したときのカウント処理停止■■■
  if (tIntCount3 > 200000) { // タイマー割り込み 100mS * 20000 = 20000S(約33.3分) で
    // カウント処理を停止する。（PPS信号が約33.3分到達しなかった）

    digitalWrite(coGSET, LOW);
    Counting = 0;     // カウントストップ
    gateON = 0;
    tIntCount3 = 0;
  }

  // --------------------------------------
  if (Serial1.available()) { // ■Serial1(PA9 PA10)からUSBシリアル(PC側)■

    RXchar = Serial1.read();
    switch (RXposi) {
      case 0:
        if (RXchar == '$')
        {
          RXbuff[RXposi] = RXchar;
          RXposi++;
        } else {
          RXposi = 0;
          RCVok = 0;
        }
        break;
      case 255:                 // バッファ数 255 一杯なら吐き捨てる
        RXposi = 0;
        RCVok = 0;
        break;
      default:
        if (RXchar == '*')      // エンドマークなら '*'又は <lf>0x0a
        {
          RXbuff[RXposi] = 0x00;      // 文字配列に0x00を書き込む
          RCVok = 1;
          RXposi = 0;
        } else {
          RXbuff[RXposi] = RXchar;    // それ以外なら RXbuff[] に取り込む
          RXposi++;
        }
    }
    if (hostPRM == 0) {         // 0:ＧＰＳデータを受け付ける状態
      Serial.write(RXchar);
    }
  }
  // --------------------------------------
  if (RCVok == 1)
  {
    //  Text_setCursor(1, 3);     // 桁・行
    //  u8g2.print(RXbuff);

#ifdef OPTION_AE_GNSS
    if (strncmp(RXbuff, "$GPGGA", 6) == 0) {
      strcpy(RXbuff2, RXbuff);
      extractGPGGA(RXbuff2);        // GPS数を抽出
    }
    if (strncmp(RXbuff, "$GPZDA", 6) == 0) {
      strcpy(RXbuff2, RXbuff);
      extractGPZDA(RXbuff2);        // UTCと年月日を抽出してJSTに変換する
    }
#endif
    // UBlox NEO-M8N は、GNGGA GNRMCを使用している。
#if defined(OPTION_UBLOX_NEO) && defined(SEL_UBLOX_NEO_M8N)

    if (strncmp(RXbuff, "$GNGGA", 6) == 0) {
      strcpy(RXbuff2, RXbuff);
      extractGNGGA(RXbuff2);        // GPS数を抽出
    }
    if (strncmp(RXbuff, "$GNRMC", 6) == 0) {
      strcpy(RXbuff2, RXbuff);
      extractGNRMC(RXbuff2);        // UTCと年月日を抽出してJSTに変換する
    }
    //初期設定 CNGGA, GNGLL, GNGSA, GLGSV, GPGSV, GNRMC, GNVTG
    // 2020/05/30  GNGGA,GNRMC 戻す。
#endif

    // UBlox NEO-7M は、GPGGA GPRMCを使用している。
#if defined(OPTION_UBLOX_NEO) && defined(SEL_UBLOX_NEO_7M)
    if (strncmp(RXbuff, "$GPGGA", 6) == 0) {
      strcpy(RXbuff2, RXbuff);
      extractGPGGA(RXbuff2);        // GPS数を抽出
    }
    if (strncmp(RXbuff, "$GPRMC", 6) == 0) {
      strcpy(RXbuff2, RXbuff);
      extractGPRMC(RXbuff2);        // UTCと年月日を抽出してJSTに変換する
    }
    // 2020/06/07:追加
#endif

    RCVok = 0;
  }

  // --------------------------------------
  if (Serial.available()) {   // ■USBシリアル(PC側)からSerial1(PA9 PA10)■

    PCchar = Serial.read();
    Serial1.write(PCchar);
  }

  // --------------------------------------
  if (Serial3.available()) { // ■Serial3(PB11)からカウンターデータ受取■

    PICchar = Serial3.read();
    switch (PICposi) {
      case 0:
        if (PICchar == '[') // '['= START Mark
        {
          PICposi++;
        } else {
          PICposi = 0;
          PICok = 0;
        }
        break;
      case 19:
        if  (PICchar == ']') // 0から数えて19番目が ']'= END Mark　なら正常
        {
          PICbuff[PICposi - 1] = 0x00;
          PICok = 1;
        }
        PICposi = 0;
        break;
      default:
        if  (PICchar != ']') // ']'= END Mark 以外なら
        {
          PICbuff[PICposi - 1] = PICchar;
          PICposi++;
        } else {
          PICposi = 0;
          PICok = 0;
        }
        break;
    }

    // ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
    if (hostPRM == 2) {         // 2:PICデータをHOSTへ送信する状態■テスト■
      Serial.write(PICchar);
    }
    // ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

  }
  // --------------------------------------
  // ■■■■■■周波数カウントが完了した時の処理■■■■■■
  // ■■■■■■周波数カウントが完了した時の処理■■■■■■
  // ■■■■■■周波数カウントが完了した時の処理■■■■■■
  // ■■■■■■周波数カウントが完了した時の処理■■■■■■
  // ■■■■■■周波数カウントが完了した時の処理■■■■■■
  // ■■■■■■周波数カウントが完了した時の処理■■■■■■
  if (countDone == 1 && PICok == 1)  {
    if (Counting == 1) {
      if (CountTime >= 100000 ) {    // 連続計測回数を100000とする。（最短約55時間）
        CountTime = 1;              // 5桁オーバー時は '1' に戻る。
                                    // 2022/03/23:修正分の微修正(2022/09/16)
      }
      else {
        CountTime++;  // 通常は、計測回数を加算していく。
      }
    }

    // (sumchk != 0) SUM Check Error
    // 1～2cm間の伝送路のため、発生可能性はゼロに近いと思われるため
    // SUM Check Error 時の処理は記述せず。(2020/04/10)
    // そのまま処理して、picLoss を加算する。
    ui64Reg = hex16Bto64bit(&sumchk, PICbuff);
    if (sumchk > 0) picLoss++; // もし発生したら、ログ等への引き渡しのみ

    dFrqReg = (double)ui64Reg;      // double に変換

    ui64Reg = 0;                    // クリヤー

    // ゲート時間により、
    switch (gatePRM) {
      case 1:
        dFrqReg = dFrqReg / 10;
        break;
      case 2:
        dFrqReg = dFrqReg / 100;
        break;
      case 3:
        dFrqReg = dFrqReg / 1000;
        break;
      case 4:
        dFrqReg = dFrqReg / 10000;
        break;
      default:
        break;
    }

    // アベレージに加算
    dAvgReg += dFrqReg;
    AddCnt++;

    // 最高値の更新
    if (dMaxReg < dFrqReg) dMaxReg = dFrqReg;
    // 最低値の更新
    if (AddCnt == 1) {
      dMinReg = dFrqReg;
    } else {
      if (dMinReg > dFrqReg) dMinReg = dFrqReg;
    }
    // 振れ幅 p-p の更新
    dP_PReg = dMaxReg - dMinReg;

    // ■■■■■■■■■■■■■■■■■■■■■■■■
    // 一つ前の測定結果との誤差を求める。
    //
    // ■■■■■■■■■■■■■■■■■■■■■■■■
    if (CountTime <= 1) {   // 1回目のカウントは、プラスマイナスはゼロ
      IncDec = 0;
    } else {                // それ以外の回数時は前との変化量をプラスマイナスにセット
      IncDec = dFrqReg - B4dFrqReg;
    }
    B4dFrqReg = dFrqReg;    // 読み込んだカウント値を以前(B4)にセット

    PICok = 0;            // Serial3からの読み込みもクリヤー
    countDone = 0;       // 周波数カウントが完了した時の処理■■完了■■

    if (Counting == 1) {    // カウント継続中なら

      // ゲート時間により、
      PpsSet();                       // gatePRMからppsCntへ時間をセット
      //  PicReset();                   // V50_54a
      PortReset();    // GATE用D-F/Fのリセット

      digitalWrite(coGSET, HIGH);   // カウントの継続セット
    }

#ifdef OPTION_OCXO_Vcon  // I2C 0x60   ■■■■■■■■■■■■■■
    //
    //
    // ■■■■ VconValueが上がると周波数は下がる（逆特性） ■■■■
    // 誤差が有り過ぎるときの処理
    // その他を追加する
    //
    // Vcon MODE= Auto の場合で且つ GATE=1000Sec, 10^4Secの場合のみ自動補正する。
    // 許容値内（例）10--0.000Hz～9--9.999Hz の場合は、補正はせずSTABLE++をする。
    // 許容値±0.002 までの小誤差は、STABLEが9まで加算していれば見送り、
    //      それ以下の場合は値を縮小して補正する。ともに、STABLEを-2する。
    // 許容値±0.003 以上の誤差は、STABLEが10まで加算していれば見送り、
    //      それ以下の場合は計算値もって補正する。ともに、STABLEを-3する。
    // STABLEは、0から10の間の値とする。
    // 2021/02/24:上記の説明に合わせて、誤差の引数を一つ(10000000.000Hz)から
    // 許容値内基準(10--0.000Hz～9--9.999Hz)の二つに見直した。
    // gosa を求める時に絶対値に変換する(2021/04/29)

    B4VconValue = VconValue;
    if (VconAUTO == 1) {      // Vcon AUTO MODE の時に実行

      // ゲート時間により、
      switch (gatePRM) {
        case 1:
          if (dFrqReg >= 10000000.0) {
            gosa = fabs(dFrqReg - 10000000.0);    // gosa の絶対値を求める
            tmpdac = gosa * 1000 / UNI.dmHz;      // 変化させるDAC値を計算
            VconValue += uint16_t(tmpdac);
          } else {
            gosa = fabs(dFrqReg - 9999999.9);     // gosa の絶対値を求める
            tmpdac = gosa * 1000 / UNI.dmHz;      // 変化させるDAC値を計算
            VconValue -= uint16_t(tmpdac);
          }
          if (VconValue > 4095) VconValue = 4095;
          break;

        case 2:
          if (dFrqReg >= 10000000.00) {
            gosa = fabs(dFrqReg - 10000000.00);    // gosa の絶対値を求める
            tmpdac = gosa * 1000 / UNI.dmHz;
            VconValue += uint16_t(tmpdac);
          } else {
            gosa = fabs(dFrqReg - 9999999.99);    // gosa の絶対値を求める
            tmpdac = gosa * 1000 / UNI.dmHz;
            VconValue -= uint16_t(tmpdac);
          }
          if (VconValue > 4095) VconValue = 4095;
          break;

        case 3:
          if (dFrqReg >= 10000000.000) {      // ■■(dFrqReg >= 10000000.000)
            gosa = fabs(dFrqReg - 10000000.000);    // gosa の絶対値を求める
            tmpdac = gosa * 1000 / UNI.dmHz;
            if (gosa >= 0.003) {          // 絶対値、0.003以上の誤差

              if (stable < 10) VconValue += uint16_t(tmpdac);

              //　大幅な許容値外なので安定状態のカウントを-3する
              if (stable > 3) stable -= 3; else stable = 0;

            } else if (gosa >= 0.001) {   // 絶対値、0.002から0.001の誤差

              // 安定状態が9未満なら補正する（9or10なら補正を一回見送る：実験中）
              if (stable < 9) VconValue += uint16_t(tmpdac);   // ＊＊大幅な変化を抑制

              //　許容値外なので安定状態のカウントを-2する
              if (stable > 2) stable -= 2; else stable = 0;

            } else {
              if (stable < 10) {          // 10未満なら安定状態を加算
                stable++;
              }
            }
          } else {                            // ■■(dFrqReg <= 9999999.999)
            gosa = fabs(dFrqReg - 9999999.999);    // gosa の絶対値を求める
            tmpdac = gosa * 1000 / UNI.dmHz;

            if (gosa >= 0.003) {         // 絶対値、0.003以上の誤差
              if (stable < 10) VconValue -= uint16_t(tmpdac);
              //　許容値外なので安定状態のカウントを-3する
              if (stable > 3) stable -= 3; else stable = 0;

            } else if (gosa >= 0.001) {  // 絶対値、0.002から0.001の誤差

              // 安定状態が9未満なら補正する（9or10なら補正を一回見送る：実験中）
              if (stable < 9)  VconValue -= uint16_t(tmpdac);  // ＊＊大幅な変化を抑制

              //　大幅な許容値外なので安定状態のカウントを-2する
              if (stable > 2) stable -= 2; else stable = 0;

            } else {                      // +0.000から-0.001までの誤差（許容値内）
              if (stable < 10) {          // 10未満なら安定状態を加算
                stable++;
              }
            }
          }
          if (VconValue > 4095) VconValue = 4095;
          break;

        case 4:
          // 0.1mHz台はDACの最小分解能の限界を越している可能性が高いため、
          // 変則的な補正式とする。　
          if (dFrqReg >= 10000000.0000) {       // ■■(dFrqReg >= 10000000.0000)
            gosa = fabs(dFrqReg - 10000000.0000);    // gosa の絶対値を求める
            tmpdac = gosa * 1000 / UNI.dmHz;

            if (gosa >= 0.0003) {          // 絶対値、0.0003以上の誤差
              if (stable < 10) VconValue += uint16_t(tmpdac);

              //　大幅な許容値外なので安定状態のカウントを-3する
              if (stable > 3) stable -= 3; else stable = 0;

            } else if (gosa >= 0.0001) {   // 絶対値、0.0002から0.0001の誤差

              // 安定状態が9未満なら補正する（9or10なら補正を一回見送る：実験中）
              // 更にGATE=10^4Secだけ、１LSBだけを調整する。
              if (stable < 9) {
                if (UNI.dmHz <= 0.1) {   // UNI.dmHzの分解能が0.1mHz未満なら
                  VconValue += uint16_t(tmpdac);   // ＊そのまま加算
                } else {
                  VconValue += 1;   // 0.1mHz以上なら、1カウントの加算
                }
              }

              //　許容値外なので安定状態のカウントを-2する
              if (stable > 2) stable -= 2; else stable = 0;

            } else {
              if (stable < 10) {          // 10未満なら安定状態を加算
                stable++;
              }
            }
          } else {                             // (dFrqReg <= 9999999.9999)
            gosa = fabs(dFrqReg - 9999999.9999);    // gosa の絶対値を求める
            tmpdac = gosa * 1000 / UNI.dmHz;

            if (gosa >= 0.0003) {  // 絶対値、0.0003以上の誤差
              if (stable < 10) VconValue -= uint16_t(tmpdac);

              //　許容値外なので安定状態のカウントを-3する
              if (stable > 3) stable -= 3; else stable = 0;

            } else if (gosa <= -0.0001) {  // 絶対値、0.0002から0.0001の誤差

              // 安定状態が9未満なら補正する（9or10なら補正を一回見送る：実験中）
              // 更にGATE=10^4Secだけ、１LSBだけを調整する。
              if (stable < 9) {
                if (UNI.dmHz <= 0.1) {   // UNI.dmHzの分解能が0.1mHz未満なら
                  VconValue -= uint16_t(tmpdac);   // ＊そのまま減算
                } else {
                  VconValue -= 1;   // 0.1mHz以上なら、1カウントの減算
                }
              }

              //　大幅な許容値外なので安定状態のカウントを-2する
              if (stable > 2) stable -= 2; else stable = 0;

            } else {                      // +0.0000から-0.0001までの誤差（許容値内）
              if (stable < 10) {          // 10未満なら安定状態を加算
                stable++;
              }
            }
          }

          if (VconValue > 4095) VconValue = 4095;
          break;
        default:    // 0Hz台の測定中は自動加減しない。(I/F多回転VRで合わせる)
          //        gosa = 10000000 - dFrqReg;
          //        if (gosa >= 2) VconValue += 200;
          //        if (gosa <= -2) VconValue -= 200;
          break;
      }
      if (VconI2C == 0) {     // Vcon 接続中なら
        vconDACset(VconValue);   // ■■■■■ VconへDAC値をセット■■■■■■■■
      }
    }

#endif

    // ■■■■■■■■■■■■■■■■■■■■■■■■
    // カウント完了ごとに、（LOG指定なら）LOG出力
    // ■■■■■■■■■■■■■■■■■■■■■■■■

    if (hostPRM == 1) {         // 1:ＬＯＧデータをHOSTへ送信する状態■テスト■
      if (CountTime == 1 ) {        // もし計測1回目なら
        Serial.println(logheader); // <cr><lf>を付けてヘッダー出力
      }

      sprintf(tmptmp, "%d", CountTime);
      strcpy(logdata, tmptmp);
      strcat(logdata, ", ");
      strcat(logdata, nowDate);
      strcat(logdata, ", ");
      strcat(logdata, nowTime);
      strcat(logdata, ", ");
      sprintf(tmptmp, "%2d", SATmin);
      strcat(logdata, tmptmp);
      strcat(logdata, ",");
      sprintf(tmptmp, "%2d", SATmax);
      strcat(logdata, tmptmp);
      strcat(logdata, ",");

      switch (gatePRM) {    // GATE時間に応じて小数点以下を増減する(2020/04/29)
        case 1:
          sprintf(tmptmp, "%10.1lf   ", dFrqReg);
          break;
        case 2:
          sprintf(tmptmp, "%11.2lf   ", dFrqReg);
          break;
        case 3:
          sprintf(tmptmp, "%12.3lf  ", dFrqReg);
          break;
        case 4:
          sprintf(tmptmp, "%13.4lf ", dFrqReg);
          break;
        default:
          sprintf(tmptmp, "%8.0lf    ", dFrqReg);
          break;
      }
      strcat(logdata, tmptmp);
      strcat(logdata, ",");
      //      sprintf(tmptmp, "%+10.4f", IncDec);    // + - 記号付き、少数以下3桁、計８桁

      switch (gatePRM) {    // GATE時間に応じて小数点以下を増減する(2020/04/29)
        case 1:
          sprintf(tmptmp, "%+8.1f", IncDec);
          break;
        case 2:
          sprintf(tmptmp, "%+9.2f", IncDec);
          break;
        case 3:
          sprintf(tmptmp, "%+10.3f", IncDec);
          break;
        case 4:
          sprintf(tmptmp, "%+11.4f", IncDec);
          break;
        default:
          sprintf(tmptmp, "%+6.0f", IncDec);
          break;
      }
      strcat(logdata, tmptmp);

      strcat(logdata, ",");
      sprintf(tmptmp, "%d", gpsLoss);
      strcat(logdata, tmptmp);
      strcat(logdata, ",");
      sprintf(tmptmp, "%d", ppsLoss);
      strcat(logdata, tmptmp);
      strcat(logdata, ",");
      sprintf(tmptmp, "%d", picLoss); // PICデータのSUM Checkエラー回数 デバッグ用
      strcat(logdata, tmptmp);
#ifdef OPTION_USE_TEMPERATURE_SENSOR
      strcat(logdata, ",");
      strcat(logdata, temptVal);
#endif

#ifdef OPTION_OCXO_Vcon  // I2C 0x60
      //      strcat(logdata, ",");
      //      if (VconAUTO == 1) strcat(logdata, "A"); else strcat(logdata, "M");
      strcat(logdata, ",");
      sprintf(tmptmp, "%d", B4VconValue);   // 使用したVconの値をLOGに記録する。
      strcat(logdata, tmptmp);
      // デバッグ中★★★★
      strcat(logdata, ",");
      if (VconAUTO == 1) {      // Vcon AUTO MODE の時に（2021/04/27）
        sprintf(tmptmp, "%d", stable);   // STABLEの数値を記録
        strcat(logdata, tmptmp);
      } else {
        strcat(logdata, "M");     // MANUAL時は、M（2021/04/27）
      }
      strcat(logdata, ",");
      sprintf(tmptmp, "%8.5lf", UNI.dmHz);   // 許容値内を続けている数
      strcat(logdata, tmptmp);


#endif

      Serial.println(logdata);       // <cr><lf>を付けて出力
    }

    satReset();   // LOG出力後、衛星数MaxMinをリセット

  }   // ■■■■■■周波数カウントが完了処理　終了　■■■■■■

  // --------------------------------------

#ifdef OPTION_SSD1309_OLED


  // ■■■■■■■■■■■■■■■■■■■■■■■■
  // OLED   への表示処理
  // ■■■■■■■■■■■■■■■■■■■■■■■■

#ifdef OPTION_AE_GNSS
  // OLED と AE-GNSS の組合せの場合のみ表示タイミングを遅らせる。2022/03/22
  // AE-GNSS のPPS信号（LOWアクティブ100mS）直後に時間を表示すると、AE-GNSSの「$GPZDA」にある時間データを受けきれないため
  // 1秒遅れた表示に成る。そのため、tIntCount1 を使ってタイミングを遅らせる。（約350mS以上＝余裕を見て400mSとする）
  if (digitalRead (ciPPS) == HIGH) tIntCount1 = 0;

  // ciPPSが、LOWになった直後のタイミングで表示を実施する。（1回目）
  if ((digitalRead (ciPPS) == LOW) && (dispOnce == 0) && (tIntCount1 >= 40)) { // 40 = 遅らせる時間 400ｍS(10mS単位)
    dispOnce++;  // この時点では = 1
    dispGo = 1;
    // tIntCount1 = 0;
#else
  // こちらはUBLOX用の処理
  // ciPPSが、LOWになった直後のタイミングで表示を実施する。（1回目）
  if ((digitalRead (ciPPS) == LOW) && (dispOnce == 0)) { // 表示時間がかかるので、1回／秒の表示（TFTに揃えた）
    dispOnce++;  // この時点では = 1
    dispGo = 1;
    // tIntCount1 = 0;

#endif
  }


  if (dispGo == 1) {  // 表示ルーチンを実行する。
    dispGo = 0;
#ifdef OPTION_USE_TEMPERATURE_SENSOR        // ■■■■■■

#ifndef SENSOR_TEST_MODE    // テストモードで無ければ通常の処理
    if (temptuRead < 1) {  // 最初は処理
      tempt_chk();  // 温度を取得する
      temptuRead = 3;      // その後は、10回(10秒）毎に温度をチェック==>　3回
    } else {
      temptuRead--;
    }
#else                      // テストモードの処理　連続的に取得

    tempt_chk();  // 温度を取得する

#endif

#endif  // end ■■■　OPTION_USE_TEMPERATURE_SENSOR　■■■

    digitalWrite(LED_OUT, LOW);
    u8g2.clearBuffer();          // clear the internal memory

    if (dispNo == 0) {                // 基本画面
      Text_setCursor(0, 0);   // 桁・行
      u8g2.print("Frq:");
      dFRQtoStrV(fmtFrq, dFrqReg); // 周波数データを表示形式に変換
      Text_setCursor(4, 0);     // 桁・行
      u8g2.print(fmtFrq);
      Text_setCursor(19, 0);   // 桁・行
      u8g2.print("Hz");
      Text_setCursor(1, 1);     // 桁・行
      u8g2.print(useGPS);

      /* GNGGA が受け取れなかったときのため、「使用GPS数」を表示後毎回クリヤーする。*/
      strcpy(useGPS, "  ");        // スペース2文字

      Text_setCursor(3, 1);     // 桁・行
      u8g2.print("(");
      u8g2.print(gpsLoss);
      //      u8g2.print(ppsLoss);      //GPS信号が欠落したとき　　　デバッグ用
      //      u8g2.print(picLoss);
      u8g2.print(")");

      Text_setCursor(13, 1);    // 桁・行
      u8g2.print(nowTime);

      gatePRMdisp(gatePRM);    // GATE時間の表示

      sprintf(tmptmp, "%5d", CountTime);  // 計測回数（最小55時間連続でカウントできる桁数）
      Text_setCursor(11, 2);     // 桁・行
      u8g2.print(tmptmp);

      sprintf(tmptmp, "%5d", ppsCnt);   // GATEがOFFするまでの pps回数
      Text_setCursor(16, 2);     // 桁・行
      u8g2.print(tmptmp);

      Text_setCursor(1, 3);     // 桁・行
      u8g2.print("SEND=");
      Text_setCursor(6, 3);     // 桁・行
      switch (hostPRM) {
        case 0:
          u8g2.print("GPS");
          break;
        case 1:
          u8g2.print("LOG");
          break;
        case 2:
          u8g2.print("PIC");
          break;
      }

      // 仮■■　変更したパラメータを表示する。デバッグ用
      char aa = 0x30 | jobPRM;
      char bb = 0x30 | bpsPRM;
      char cc = 0x30 | hostPRM;
      char dd = 0x30 | gatePRMon;

      Text_setCursor(10, 3);    // 桁・行
      //      u8g2.print(aa);
      u8g2.print(bb);
      u8g2.print(cc);
      u8g2.print(dd);

#ifdef OPTION_USE_TEMPERATURE_SENSOR

#ifndef SENSOR_TEST_MODE      // Not define
      Text_setCursor(14, 3);     // 桁・行
      u8g2.print(temptVal);   // 温度
      u8g2.print(dosuu);   // 温度記号
      u8g2.print("C");   // 温度記号
#else
      Text_setCursor(12, 3);     // 桁・行
      u8g2.print(temptVal);   // 温度
      u8g2.print("mV");   // 電圧記号
#endif    // end ===  #ifndef SENSOR_TEST_MODE      // Not define

#endif  // defined(OPTION_TEMPERATURE_SENSOR_LM61) || defined...........

      // ************************************************************************
      //sprintf(tmptmp, "%2d", tIntCount3); // PPS信号の間隔(0.1Sec単位)デバッグ用
      //Text_setCursor(14, 3);     // 桁・行
      //  u8g2.print(tmptmp);
      // ************************************************************************
      //      sprintf(tmptmp, "%2d", picLoss); // PICデータのシリアル受信エラー回数 デバッグ用
      //      Text_setCursor(19, 3);     // 桁・行
      //      u8g2.print(tmptmp);

      if (Counting == 0) {
        FuncEnable = B00001111;
        func_print(0, "MENU");
        func_print(1, "GATE");
        func_print(2, "STRT");
        func_print(3, "CngD");
      } else {
        FuncEnable = B00000011;
        func_print(2, "STOP");
        func_print(3, "CngD");
      }

    } else {                      // 拡張　第２画面■■■■■■■■
      //
      Text_setCursor(0, 0);     // 桁・行
      u8g2.print("Ave:");
      if (AddCnt > 0) {
        dTmpReg = dAvgReg / (double)AddCnt;
        dFRQtoStr(fmtFrq, dTmpReg); // 周波数データを表示形式に変換
        Text_setCursor(4, 0);     // 桁・行
        u8g2.print(fmtFrq);
      }
      Text_setCursor(19, 0);   // 桁・行
      u8g2.print("Hz");
      //
      Text_setCursor(0, 1);     // 桁・行
      u8g2.print("Max:");
      dFRQtoStrV(fmtFrq, dMaxReg); // 周波数データを表示形式に変換
      Text_setCursor(4, 1);     // 桁・行
      u8g2.print(fmtFrq);
      Text_setCursor(19, 1);   // 桁・行
      u8g2.print("Hz");
      //
      Text_setCursor(0, 2);     // 桁・行
      u8g2.print("Min:");
      dFRQtoStrV(fmtFrq, dMinReg); // 周波数データを表示形式に変換
      Text_setCursor(4, 2);     // 桁・行
      u8g2.print(fmtFrq);
      Text_setCursor(19, 2);   // 桁・行
      u8g2.print("Hz");
      //
      Text_setCursor(0, 3);     // 桁・行
      u8g2.print("P-P:");
      dFRQtoStrV(fmtFrq, dP_PReg); // 周波数データを表示形式に変換
      Text_setCursor(4, 3);     // 桁・行
      u8g2.print(fmtFrq);
      Text_setCursor(19, 3);   // 桁・行
      u8g2.print("Hz");

      FuncEnable = B00000001;
      func_print(3, "CngD");

      /* PICをリセットするボタンを廃止する。(2022/03/23)
            if (Counting == 0) {
              FuncEnable = B00000101;
              func_print(1, "pCLR");
              func_print(3, "CngD");
            } else {
              FuncEnable = B00000001;
              func_print(3, "CngD");
            }
      */

    }

    // ■■■■■■■■■■■■■■■■■■■■■■■■
    u8g2.sendBuffer();          // OLEDに送信（描画する）

    digitalWrite(LED_OUT, HIGH);
    /////////////////////////////////////

  }
#endif  // SSD1309_OLED

  // =======================================================================
#ifdef OPTION_ILI9341_TFT

  // ■■■■■■■■■■■■■■■■■■■■■■■■
  //   TFT への表示処理
  // ■■■■■■■■■■■■■■■■■■■■■■■■
  /*
     2020/06/02:表示処理時間の実測実験（DSOにて測定）
     7行＋ファンクション行の表示　330ｍSec
     7行の表示　＊280～330ｍSec
     ＊ 0Hz時は280mSec、測定周波数を表示（10MHz）すると、330mSec
     7行の内、変化しない表示を1回のみ表示すると ＊205～270mSec
     ＊ 0Hz時は205mSec、測定周波数を表示すると、270mSec
      （表示文字数に応じて長短有り）

    // 以下の if ((digitalRead (ciPPS) == LOW)......... は、PPS信号のアクティブ時間(100mS)後の
    // 非アクティブ時間(900mS)の最初に1回のみ表示させるため。
    if ((digitalRead (ciPPS) == LOW) && (dispOnce == 0)) { // 表示時間がかかるので、1回／秒の表示
    dispOnce = 1;
  */
if (digitalRead (ciPPS) == HIGH) tIntCount1 = 0;

  // ciPPSが、LOWになった直後のタイミングで表示を実施する。（1回目）
  if ((digitalRead (ciPPS) == LOW) && (dispOnce == 0) && (tIntCount1 >= 20)) { // 表示時間がかかるので、1回／秒の表示と
// 加えて＄GNGGAの受信衛星数がNWO-M8Nのバラツキにより取り切れないことがあったので、
// 表示処理を200mS遅らせる遅延を挿入。：遅らせる時間 200ｍS(10mS単位)
// 2.01の３個体をチェックして、50,80,100mSecそれぞれ遅延が必要だったので、余裕を見て200mS=20 とする。(2022/09/16)
    dispOnce++;  // この時点では = 1
    dispGo = 1;
    // tIntCount1 = 0;
  }

  if (dispGo == 1) {  // 表示ルーチンを実行する。
    dispGo = 0;

#ifdef OPTION_USE_TEMPERATURE_SENSOR
    if (temptuRead < 1) {  // 最初は処理
      tempt_chk();  // 温度を取得する
      temptuRead = 3;      // その後は、10回(10秒）毎に温度をチェック ==> 5回
    } else {
      temptuRead--;
    }
#endif

    digitalWrite(LED_OUT, LOW);

    if ( formDisp == 0) {

      fixedForm();    // 固定フォームはここで表示
      formDisp = 1;   // 以前の funcDisp から分離
    }

    if (Counting != 0)  {
      if (funcDisp == 0) {
        FuncEnable = B00000001;
        func_print(3, "STOP");
        funcDisp = 1;
      }
    } else {
      if (tIntCount2 < stopTime) {    // #define stopTime = 1100 で設定（現1.1秒）
        FuncEnable = B00001100;   // stopTime 中は、STRT ボタンのみ非表示
      } else {
        FuncEnable = B00001110;   // "MENU"・"GATE"・"STRT"を表示
        tIntCount2 = 5000;
      }
      if (funcDisp != B01110) {          // ★★func表示が揃っていないときは、以下を実行
        if (FuncEnable & 0x08) {        // "MENU"の表示ビット＝１
          if ((funcDisp & 0x08) == 0) {  // 表示済みか？
            func_print(0, "MENU");      // "MENU"の実表示
            funcDisp |= B01000;          // 表示済みビットをセット
          }
        }
        if (FuncEnable & 0x04) {
          if ((funcDisp & 0x04) == 0) {
            func_print(1, "GATE");
            funcDisp |= B00100;
          }
        }
        if (FuncEnable & 0x02) {
          if ((funcDisp & 0x02) == 0) {
            func_print(2, "STRT");
            funcDisp |= B00010;
          }
        }
      }          // ★★ここまで func 表示が揃っていないときの処理
    }

    dFRQtoStrV(fmtFrq, dFrqReg); // 周波数データを表示形式に変換
    TFT_setCursorB(1, 0);     // 桁・行
    //    u8g2_for_adafruit_gfx.setFont(u8g2_font_profont29_mr);  // 大きい文字  W16xH29 (A19) _mn数字だけ
    fonSet_BIG();
    u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_YELLOW);
    u8g2_for_adafruit_gfx.print(fmtFrq);
    u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
    TFT_setCursorB(17, 0);   // 桁・行

    u8g2_for_adafruit_gfx.print("Hz");    // 試しにHzを小さなフォントに変更＆戻す(2021/01/25)
    //    u8g2_for_adafruit_gfx.setFont(u8g2_font_profont22_mf);  // OK (A14)  W12xH22
    fonSet_MID();
    TFT_setCursor(1, 1);     // 桁・行
    u8g2_for_adafruit_gfx.print(useGPS);

    /* GNGGA が受け取れなかったときのため、「使用GPS数」を表示後毎回クリヤーする。*/
    strcpy(useGPS, "  ");        // スペース2文字

    TFT_setCursor(3, 1);     // 桁・行
    u8g2_for_adafruit_gfx.print("(");
    if (gpsLoss >= 100) gpsLoss = 99;   // ２桁 99 が上限とする。
    sprintf(tmptmp, "% 2u", gpsLoss);   //  2桁指定
    u8g2_for_adafruit_gfx.print(tmptmp);
    //    u8g2_for_adafruit_gfx.print(ppsLoss);      // PPS信号が欠落したとき　　　デバッグ用
    //    u8g2_for_adafruit_gfx.print(picLoss);
    u8g2_for_adafruit_gfx.print(") ");

#ifdef OPTION_OCXO_Vcon  // I2C 0x60      

    TFT_setCursor(8, 1);     // 桁・行

    if (VconI2C == 0) {
      u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_MAGENTA);

      if (VconAUTO == 1) {   // AUTO = 1 / MANUAL = 0
        sprintf(tmptmp, "A.%04u", VconValue);  //  4桁指定  0000-4095
      } else {
        sprintf(tmptmp, "M.%04u", VconValue);  //  4桁指定  0000-4095
      }
      u8g2_for_adafruit_gfx.print(tmptmp);
      //     B4VconValue = VconValue;
      u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
    } else {
      VconAUTO = 0;           // 2022/09/16：「I2Cが未接続なら、VconAUTO を強制的にOFF'0'にする」
      TFT_setCursor(8, 1);    // 桁・行
      u8g2_for_adafruit_gfx.print("vcOFF ");
    }
    /*    // デバッグ用（2021/04/27）
        TFT_setCursor(14, 3);     // 桁・行
        if (VconAUTO == 1) {      // Vcon AUTO MODE の時に（2021/04/27）
          char ee = 0x30 | stable;
          u8g2_for_adafruit_gfx.print(ee);
        } else {
          u8g2_for_adafruit_gfx.print("M");
        }
    */
#endif

    TFT_setCursor(16, 1);    // 桁・行
    u8g2_for_adafruit_gfx.print(nowTime);

    gatePRMdisp(gatePRM);    // GATE時間の表示

    sprintf(tmptmp, "%5d", CountTime);  // 計測回数（最小55時間連続でカウントできる桁数）
    TFT_setCursor(13, 2);     // 桁・行
    u8g2_for_adafruit_gfx.print(tmptmp);

    sprintf(tmptmp, "%5d", ppsCnt);   // GATEがOFFするまでの pps回数
    TFT_setCursor(19, 2);     // 桁・行
    u8g2_for_adafruit_gfx.print(tmptmp);

#ifdef OPTION_USE_TEMPERATURE_SENSOR

#ifndef SENSOR_TEST_MODE      // Not define    
    TFT_setCursor(17, 3);     // 桁・行
    u8g2_for_adafruit_gfx.print(temptVal);   // 温度 LM61
    u8g2_for_adafruit_gfx.print(dosuu);   // 温度記号
    u8g2_for_adafruit_gfx.print("C");   // 温度記号
#else
    TFT_setCursor(15, 3);     // 桁・行
    u8g2_for_adafruit_gfx.print(temptVal);   // 温度
    u8g2_for_adafruit_gfx.print("mV");   // 電圧記号
#endif    // end ===  #ifndef SENSOR_TEST_MODE      // Not define

#endif

    // ************************************************************************
    //sprintf(tmptmp, "%2d", tIntCount3); // PPS信号の間隔(0.1Sec単位)デバッグ用
    //TFT_setCursor(14, 3);     // 桁・行
    //  u8g2_for_adafruit_gfx.print(tmptmp);
    // ************************************************************************
    //      sprintf(tmptmp, "%2d", picLoss); // PICデータのシリアル受信エラー回数 デバッグ用
    //      TFT_setCursor(19, 3);     // 桁・行
    //      u8g2_for_adafruit_gfx.print(tmptmp);

    if (AddCnt > 0) {
      dTmpReg = dAvgReg / (double)AddCnt;
      dFRQtoStr(fmtFrq, dTmpReg); // 周波数データを表示形式に変換
      TFT_setCursor(6, 4);     // 桁・行
      u8g2_for_adafruit_gfx.print(fmtFrq);
    }

    dFRQtoStrV(fmtFrq, dMaxReg); // 周波数データを表示形式に変換
    TFT_setCursor(6, 5);     // 桁・行
    u8g2_for_adafruit_gfx.print(fmtFrq);

    dFRQtoStrV(fmtFrq, dMinReg); // 周波数データを表示形式に変換
    TFT_setCursor(6, 6);     // 桁・行
    u8g2_for_adafruit_gfx.print(fmtFrq);

    dFRQtoStrV(fmtFrq, dP_PReg); // 周波数データを表示形式に変換
    TFT_setCursor(6, 7);     // 桁・行
    u8g2_for_adafruit_gfx.print(fmtFrq);

    // ■■■■■■■■■■■■■■■■■■■■■■■■

    digitalWrite(LED_OUT, HIGH);

  }

#endif  // OPTION_ILI9341_TFT

} // Loop End
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

// =======================================================================
#ifdef OPTION_SSD1309_OLED

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
//   GATE時間の表示を独立させて 1回／秒 表示と、切り替え時に表示する。
//   gatePRMdisp(uint8_t)
//   V50_53         （OLED & TFT 共に）
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void gatePRMdisp(uint8_t prm)
{
  Text_setCursor(1, 2);     // 桁・行
  u8g2.print("GATE=");
  Text_setCursor(6, 2);     // 桁・行
  switch (prm) {
    case 1:
      u8g2.print("10s   ");
      break;
    case 2:
      u8g2.print("100s  ");
      break;
    case 3:
      u8g2.print("1000s ");
      break;
    case 4:     // OLEDでは実験的な扱い（長押しで選択）
      u8g2.print("10^4s ");    // 表示桁数節約のため 10の4乗 表記
      break;
    default:
      u8g2.print("1s    ");
      break;
  }
}
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
//   「ファンクション」表示関数
//   表示器の最下行はファンクション表示エリアとして、４つのボタンに割り当てる。
//   一つのボタンは４文字で、左1文字分を空けてBox描画で反転表示する。
//   fnmb : ボタン番号
//   str  : 表示文字（4文字以内）
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void func_print(uint8_t fnmb, const char* str)
{
  int8_t boxORframe = 1;    // 0 = Box描画,  1 = フレーム描画　切替予定
  int8_t len;
  len = strlen(str);
  if ((len == 0) || (len > 4)) return;

  switch (fnmb) {
    case 0:
      u8g2.setDrawColor(1);          // 反転用の処理のため
      if (boxORframe == 0)  u8g2.drawBox(5, 50, 27, 13);   // Box描画
      else
        u8g2.drawFrame(5, 50, 27, 13);   // フレーム描画
      u8g2.setDrawColor(2);          //  color 2 set
      Text_setCursor(1, 4);   // 桁・行
      u8g2.print(str);
      //      u8g2.drawLine(6, 63, 31, 63);   // (x1, y1, x2, y2)
      break;
    case 1:
      u8g2.setDrawColor(1);          // 反転用の処理のため
      if (boxORframe == 0)  u8g2.drawBox(35, 50, 27, 13);  // Box描画
      else
        u8g2.drawFrame(35, 50, 27, 13);
      u8g2.setDrawColor(2);          //  color 2 set
      Text_setCursor(6, 4);   // 桁・行
      u8g2.print(str);
      //      u8g2.drawLine(36, 63, 61, 63);   // (x1, y1, x2, y2)
      break;
    case 2:
      u8g2.setDrawColor(1);          // 反転用の処理のため
      if (boxORframe == 0)  u8g2.drawBox(65, 50, 27, 13);  // Box描画
      else
        u8g2.drawFrame(65, 50, 27, 13);
      u8g2.setDrawColor(2);          //  color 2 set
      Text_setCursor(11, 4);   // 桁・行
      u8g2.print(str);
      //      u8g2.drawLine(66, 63, 91, 63);   // (x1, y1, x2, y2)
      break;
    case 3:
      u8g2.setDrawColor(1);           // 反転用の処理のため
      if (boxORframe == 0)  u8g2.drawBox(95, 50, 27, 13);   // Box描画
      else
        u8g2.drawFrame(95, 50, 27, 13);
      u8g2.setDrawColor(2);           //  color 2 set
      Text_setCursor(16, 4);   // 桁・行
      u8g2.print(str);
      //      u8g2.drawLine(96, 63, 121, 63);   // (x1, y1, x2, y2)
      break;
  }
}
#endif
// ------------------------------
#ifdef OPTION_ILI9341_TFT

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
//   GATE時間の表示を独立させて 1回／秒 表示と、切り替え時に表示する。
//   gatePRMdisp(uint8_t)
//   V50_53         （OLED & TFT 共に）
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void gatePRMdisp(uint8_t prm)
{
  TFT_setCursor(1, 2);     // 桁・行
  u8g2_for_adafruit_gfx.print("GATE=");
  TFT_setCursor(6, 2);     // 桁・行
  switch (prm) {
    case 1:
      u8g2_for_adafruit_gfx.print("10s   ");
      break;
    case 2:
      u8g2_for_adafruit_gfx.print("100s  ");
      break;
    case 3:
      u8g2_for_adafruit_gfx.print("1000s ");
      break;
    case 4:
      u8g2_for_adafruit_gfx.print("10^4s ");    // 表示桁数節約のため 10の4乗 表記
      break;
    default:
      u8g2_for_adafruit_gfx.print("1s    ");
      break;
  }
}
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
//   「ファンクション」表示関数
//   表示器の最下行はファンクション表示エリアとして、４つのボタンに割り当てる。
//   一つのボタンは４文字で、左1文字分を空けてBox描画で反転表示する。
//   fnmb : ボタン番号
//   str  : 表示文字（4文字以内）
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void func_print(uint8_t fnmb, const char* str)
{
  int8_t len;
  uint8_t xx;
  uint8_t yy = 208;
  uint8_t ww = 64;
  uint8_t hh = 32;
  uint8_t rro = 5;
  uint8_t rri = 3;

  len = strlen(str);
  if ((len == 0) || (len > 4)) return;

  switch (fnmb) {
    case 0:
      display.fillRoundRect(20, yy, ww, hh, rro, ILI9341_CYAN);   // フレーム描画
      display.fillRoundRect(20 + 4, yy + 4, ww - 8, hh - 8, rri, ILI9341_BLACK);   // フレーム描画
      TFT_setCursor(2, 8);   // 桁・行
      u8g2_for_adafruit_gfx.print(str);
      break;
    case 1:
      display.fillRoundRect(92, yy, ww, hh, rro, ILI9341_CYAN);
      display.fillRoundRect(92 + 4, yy + 4, ww - 8, hh - 8, rri, ILI9341_BLACK);
      TFT_setCursor(8, 8);   // 桁・行
      u8g2_for_adafruit_gfx.print(str);
      break;
    case 2:
      display.fillRoundRect(164, yy, ww, hh, rro, ILI9341_CYAN);
      display.fillRoundRect(164 + 4, yy + 4, ww - 8, hh - 8, rri, ILI9341_BLACK);
      TFT_setCursor(14, 8);   // 桁・行
      u8g2_for_adafruit_gfx.print(str);
      break;
    case 3:
      display.fillRoundRect(236, yy, ww, hh, rro, ILI9341_CYAN);
      display.fillRoundRect(236 + 4, yy + 4, ww - 8, hh - 8, rri, ILI9341_BLACK);
      TFT_setCursor(20, 8);   // 桁・行
      u8g2_for_adafruit_gfx.print(str);
      break;
  }
}

// 01234567890123456789012345
//   xxxx  xxxx  xxxx  xxxx

// ファンクションキーの領域だけをクリヤーする。
void func_fillCler(void) {
  display.fillRect(0, 208, 320, 32, ILI9341_BLACK);   // フレーム描画 210,30 ==>208,32
}

#endif
// =======================================================================
#ifdef OPTION_SSD1309_OLED
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
//  文字出力の位置を、文字単位で指定する。
//  左上が０桁(0column)０行(0line)（合計：21文字x5行）
//  対象とするフォント：   u8g2.setFont(u8g2_font_6x12_mf);
//  ＊1文字目の左1ドットは、空ける。
//   cc   : 桁　0から20
//   ll   : 行　0から4
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void Text_setCursor(uint8_t cc, uint8_t ll) // cc:0 to 20, ll:0 to 4  Use
{
  uint8_t cp, lp;
  cp = cc * 6 + 1;
  lp = (ll + 1) * 12 - 1;

  u8g2.setCursor(cp, lp);
}
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
//  上記同様にテキストの行を指定して反転させる。
//  行は、０から４の5行として指定する。
//  対象とするフォント：   u8g2.setFont(u8g2_font_6x12_mf);
//  ＊1文字目の左1ドットは、空ける。
//   cc   : 桁　0から20
//   ll   : 行　0から4
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void drawTxline(uint8_t ll)
{
  uint8_t l1;
  l1 = ll * 12 + 1;
  u8g2.drawBox(0, l1, 127, 12);
}
#endif
// -------------------------------
#ifdef OPTION_ILI9341_TFT
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
//  文字出力の位置を、文字単位で指定する。color bit=18 x 320x240
//  左上が０桁(0column)０行(0line)（合計：26文字x9行）
//  対象とするフォント：
//          u8g2_for_adafruit_gfx.setFont(u8g2_font_profont22_mf);
//                                 Width 12, Height 22, Capital A 14
//            代替候補：u8g2_font_VCR_OSD_mf      Width:12, Height:17
//            代替候補：u8g2_font_courB14_tf
//  ＊1文字目の左4ドットは、空ける。    // 桁・行
//   cc   : 桁　0から25
//   ll   : 行　0から8
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

void TFT_setCursor(uint8_t cc, uint8_t ll) // cc:0 to 25, ll:0 to 8  Use
{
  uint16_t cp, lp;
  cp = cc * 12 + 4;
  if (ll > 7) {
    lp = (ll + 1 ) * 25 + 6;
  } else {
    lp = (ll + 1 ) * 25;
  }
  u8g2_for_adafruit_gfx.setCursor(cp, lp);
}
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// 大きい文字用
//  対象とするフォント：
//          u8g2_for_adafruit_gfx.setFont(u8g2_font_profont29_mf);
//                Width:16, Height:29, Capital A 19
//              代替候補：u8g2_font_VCR_OSD_mf      Width:12, Height:17
//  ＊1文字目の左8ドットは、空ける。    // 桁・行
//   cc   : 桁　0から20
//   ll   : 行　0から8
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

void TFT_setCursorDUMY(uint8_t cc, uint8_t ll) // cc:0 to 25, ll:0 to 8  Use
{
  uint16_t cp, lp;
  cp = cc * 12 + 4;
  if (ll > 7) {
    lp = (ll + 1 ) * 25 + 6;
  } else {
    lp = (ll + 1 ) * 25;
  }
  u8g2_for_adafruit_gfx.setCursor(cp, lp);
}
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// 大きい文字用
//  対象とするフォント：
//          u8g2_for_adafruit_gfx.setFont(u8g2_font_profont29_mf);
//                Width:16, Height:29, Capital A 19
//              代替候補：u8g2_font_VCR_OSD_mf      Width:12, Height:17
//  ＊1文字目の左8ドットは、空ける。    // 桁・行
//   cc   : 桁　0から20
//   ll   : 行　0から8
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

void TFT_setCursorB(uint8_t cc, uint8_t ll) // cc:0 to 20, ll:0 to 8  Use
{
  uint16_t cp, lp;
  cp = cc * 16 + 8;
  lp = (ll + 1 ) * 25 + 2;
  u8g2_for_adafruit_gfx.setCursor(cp, lp);
}

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// 想定した行に、指定の色ボックスを描画する。
//
// 指定は、スタート位置の桁、行、最初の桁からの長さ、色
//
// iro は、ILI9341_WHITE　　ILI9341_BLACK　　ILI9341_CYAN ........
//
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void TFT_LineCursor(uint8_t cc, uint8_t ll, uint8_t leng, uint16_t iro)
{
  uint16_t cp, lp, clen;
  cp = cc * 12;
  if (ll > 7) {
    lp = ll * 25 + 6;
  } else {
    lp = ll  * 25 + 4;
  }
  clen = leng * 12 + cp;
  display.fillRect(cp, lp, clen, 28, iro);   // フレーム描画
}

#endif
// =======================================================================

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// バイト単位で文字列を抽出するMid関数
// 128文字以内を制限しないため、int8_t ではなく int 型を使用する。
//　buf[]:抽出先
//  str[]:抽出される文字列
//  pos  :抽出開始位置
//  len  :文字数
// return は、抽出した文字数を返す。
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
int mid(char *buf, char *str, int pos, int len)
{
  int i;
  int j = 0;
  for (i = 0; str[i] != '\0'; i++) {  // 対象の文字列の文字数分を末尾まで順に
    if (i >= pos && i < pos + len) {  // 指定位置と長さ分を
      buf[j] = str[i];                // buf[j]に格納
      j++;
    }
  }
  buf[j] = '\0'; //bufの末尾にNULL文字を格納
  return strlen(buf);
}

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// int cmmSTRcmm(char buf[], char str[], int cnt)
// 128文字以内を制限しないため、int8_t ではなく int 型を使用する。
//
// ','で挟まれた文字列を抽出する。
//
// buf[] :抽出先文字列
// str[] :検索対象文字列
// cnt   :何個目の ',' 以後の文字列以降を抽出するのか指定
//        注）指定個数＋１以上の ',' が発見できない「不成立」
// return
// 0～n:抽出した文字数
// -1  :不成立
//
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
int cmmSTRcmm(char *buf, char *str, int cnt)
{
  int c, i, h = 0, p1 = 0, p2 = 0;
  int j = 0;
  c = strlen(str);
  if (cnt >= c) return -1;   // 文字列が探したい位置より短い (-1)
  for (i = 0; c > i; i++)
  {
    if (str[i] == ',') {
      h++;
      if (h == cnt) p1 = i;       // p1に',' cnt個目の位置セット
      if (h == cnt + 1) p2 = i;   // p2に',' cnt+1個目の位置セット
    }
  }
  if (h < cnt + 1)  return -1;    // 指定個数＋１以上の ',' が発見できない (-1)
  if (p1 + 1 == p2) {             // ','間に文字列ゼロ
    buf[0] = 0x00;
    return 0;
  }
  for (i = 1; (p1 + i) < p2 ; i++)
  {
    buf[j] = str[p1 + i];
    j++;
  }
  buf[j] = 0x00;
  return p2 - p1 - 1;           // 変換した文字数を返す
}

// ★★★★★★★★★★★★★★★★★★★★★★★★★★★★★★★★

// =======================================================================
#ifdef OPTION_AE_GNSS
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ＧＰＳデータから時分秒と使用中のＧＰＳ数を抽出して
// useGPS,nowTime にセットする。
// 対象データ：$GPGGA
// $GPGGA,001251.000,3445.9102,N,13527.9952,E,1,13,0.72,36.5,M,34.2,M,,*50<CR><LF>
// 時分秒.000, 北緯, 東経,「ＧＰＳのクオリティ0=No,1=OK,2=DGPS」, 使用衛星数, ]ほか
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

int8_t extractGPGGA(char *str) {
  int8_t ci = 0;
  uint8_t maxmin;
  char tmp[16];

  // 使用ＧＰＳ数を抽出
  if (cmmSTRcmm(tmp, str, 6) > 0) {
    switch (tmp[0]) {
      case '2':         // 0:受信不能　1:単独測位　2:DGPS
      case '1':
        ci = cmmSTRcmm(tmp, str, 7);

        if (ci == 0) {
          strcpy(useGPS, "00");        // ゼロ文字　"00"表記
        }
        if (ci == 1) {
          strcpy(useGPS, "0");                    // 1文字　先頭に"0”を付加
          strncat(useGPS, tmp, 1);
        }
        if (ci >= 2) {
          strncpy(useGPS, tmp, 2);    // 先頭二桁のみ抽出
        }
        break;
      default:
        gpsLoss++;
        strcpy(useGPS, "00");        // ゼロ文字　"00"表記
        break;
    }
    maxmin = atoi(useGPS);
    if (SATmin > maxmin) SATmin = maxmin;  //  最小衛星数を更新
    if (SATmax < maxmin) SATmax = maxmin;  //  最大衛星数を更新

  }
  return ci;                                     // 使用ＧＰＳの桁数を返す。0-1-2
}

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ＧＰＳデータから時分秒と年月日を抽出する。
// useGPS,nowTime nowDateにセットする。
// 対象データ：$GPZDA
// $GPZDA,001251.000,04,01,2020,,*54<CR><LF>
// 協定世界時(UTC）時分秒.000, DD,MM,YYYY,,*チェックSUM
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

int8_t extractGPZDA(char *str) {
  int8_t jhh, jdd, jmm, cdd, leapY;
  int16_t jyy;
  char fn[2] = "0";
  char tmp[16];
  char tmps[16];

  cdd = 0;

  // 時分秒を抽出
  if (cmmSTRcmm(tmp, str, 1) >= 10) {  //  HHMMDD.000 十文字以上なら(AE-GNSSは、10文字)
    mid(tmps, tmp, 0, 2);    // 2桁時
    jhh = atoi(tmps);        // UTC時間を数字に
    jhh = jhh + 9;                  // UTC +9H = JST
    if (jhh >= 24) {
      jhh = jhh - 24;  // 24H以上なら24Hを引く
      cdd = 1;
    }
    sprintf(tmps, "%02d", jhh);     // 2文字に変換
    strcpy(nowTime, tmps);          // JST出力
    strcat(nowTime, ":");
    mid(tmps, tmp, 2, 2);    // 2桁分
    strcat(nowTime, tmps);
    strcat(nowTime, ":");
    mid(tmps, tmp, 4, 2);    // 2桁秒
    strcat(nowTime, tmps);
    strcpy(tmpsec, tmps);     // 秒を記憶する■■
  } else {
    return 0;
  }

  if (cmmSTRcmm(tmp, str, 4) == 4) {  //  DDMMYY 6文字なら
    jyy = atoi(tmp);       // UTC年を数字に

    //////////// 閏年の判定スタート
    if (jyy % 4 == 0) {
      if (jyy % 100 == 0) {
        if (jyy % 400 == 0) {
          leapY = 1;
        } else {
          leapY = 0;
        }
      } else {
        leapY = 1;
      }
    } else {
      leapY = 0;
    }
    //////////// 閏年の判定エンド

    if (cmmSTRcmm(tmp, str, 2) == 2) {   // 2桁・日
      jdd = atoi(tmp);       // UTC日を数字に
      jdd = jdd + cdd;        // 24Hのオーバーフロー加算
    } else {
      return 0;
    }
    if (cmmSTRcmm(tmp, str, 3) == 2) {   // 2桁・月
      jmm = atoi(tmp);       // UTC月を数字に

    } else {
      return 0;
    }

    //////////// 月の最終日の判定
    switch (jmm) {
      case 4:
      case 6:
      case 9:
      case 11:
        if (jdd > 30) {
          jdd = 1;
          jmm++;
        }
        break;
      case 2:
        if (leapY == 0) {     // 閏年ではない
          if (jdd > 28) {
            jdd = 1;
            jmm++;
          }
        } else {              // 閏年である
          if (jdd > 29) {
            jdd = 1;
            jmm++;
          }
        }
        break;
      default:            // 1 3 5 7 8 10 12
        if (jdd > 31) {
          jdd = 1;
          jmm++;
        }
        break;
    }
    //////////// 最終月の判定

    if (jmm > 12) {
      jmm = 1;
      jyy++;              // 年を進める
    }
    sprintf(tmps, "%04d", jyy);     // 4文字
    strcpy(nowDate, tmps);          // JST出力
    strcat(nowDate, "/");
    sprintf(tmps, "%02d", jmm);     // 2文字
    strcat(nowDate, tmps);          // JST出力
    strcat(nowDate, "/");
    sprintf(tmps, "%02d", jdd);     // 2文字
    strcat(nowDate, tmps);          // JST出力
    return 1;                      // 正常（時分秒と年月日の両方）抽出時は、1を返す。
  } else {
    return 0;
  }
}
#endif
// -------------------------
#if defined(OPTION_UBLOX_NEO) && defined(SEL_UBLOX_NEO_M8N)
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ＧＰＳデータから時分秒と使用中のＧＰＳ数を抽出して
// useGPS,nowTime にセットする。
// 対象データ：$GNGGA
// $GNGGA,001251.000,3445.9102,N,13527.9952,E,1,13,0.72,36.5,M,34.2,M,,*50<CR><LF>
// 時分秒.000, 北緯, 東経,「ＧＰＳのクオリティ0=No,1=OK,2=DGPS」, 使用衛星数, ]ほか
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

int8_t extractGNGGA(char *str) {
  int8_t ci = 0;
  uint8_t maxmin;
  char tmp[16];

  // 使用ＧＰＳ数を抽出
  if (cmmSTRcmm(tmp, str, 6) > 0) {
    switch (tmp[0]) {
      case '2':         // 0:受信不能　1:単独測位　2:DGPS
      case '1':
        ci = cmmSTRcmm(tmp, str, 7);

        if (ci == 0) {
          strcpy(useGPS, "00");        // ゼロ文字　"00"表記
        }
        if (ci == 1) {
          strcpy(useGPS, "0");                    // 1文字　先頭に"0”を付加
          strncat(useGPS, tmp, 1);
        }
        if (ci >= 2) {
          strncpy(useGPS, tmp, 2);    // 先頭二桁のみ抽出
        }
        break;
      default:
        gpsLoss++;
        strcpy(useGPS, "00");        // ゼロ文字　"00"表記
        break;
    }
    maxmin = atoi(useGPS);
    if (SATmin > maxmin) SATmin = maxmin;  //  最小衛星数を更新
    if (SATmax < maxmin) SATmax = maxmin;  //  最大衛星数を更新

  }
  return ci;                                     // 使用ＧＰＳの桁数を返す。0-1-2
}

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ＧＰＳデータから時分秒と年月日を抽出する。
// useGPS,nowTime nowDateにセットする。    U-Blox NEO で使用する
// 対象データ：$GNRMC
// $GNRMC,001252.000,A,3445.9100,N,13527.9957,E,0.11,275.89,040120,,,A*7C<CR><LF> <==AE_GNSS
// $GNRMC,012222.00,A,3445.90592,N,13528.01464,E,0.025,,260520,,,A*60 <==U-Blox NEO
// 時分秒.000,ステータスA有効:V無効,緯度,経度,速度,方向,日月年,地磁気,,モード*チェックSUM
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

int8_t extractGNRMC(char *str) {
  int8_t jhh, jdd, jmm, cdd, cmm, leapY;
  int16_t jyy;
  char fn[2] = "0";
  char tmp[16];
  char tmps[16];

  cdd = 0;

  // 時分秒を抽出
  if (cmmSTRcmm(tmp, str, 1) >= 9) {  //  HHMMDD.00 9文字以上なら（U-Bloxは9文字）
    mid(tmps, tmp, 0, 2);    // 2桁時
    jhh = atoi(tmps);        // UTC時間を数字に
    jhh = jhh + 9;                  // UTC +9H = JST
    if (jhh >= 24) {
      jhh = jhh - 24;  // 24H以上なら24Hを引く
      cdd = 1;
    }
    sprintf(tmps, "%02d", jhh);     // 2文字に変換
    strcpy(nowTime, tmps);          // JST出力
    strcat(nowTime, ":");
    mid(tmps, tmp, 2, 2);    // 2桁分
    strcat(nowTime, tmps);
    strcat(nowTime, ":");
    mid(tmps, tmp, 4, 2);    // 2桁秒
    strcat(nowTime, tmps);
    strcpy(tmpsec, tmps);     // 秒を記憶する■■
  } else {
    return 0;
  }

  if (cmmSTRcmm(tmp, str, 9) == 6) {  //  DDMMYY 6文字なら
    mid(tmps, tmp, 4, 2);   // 2桁・年
    jyy = atoi(tmps);       // UTC年を数字に
    jyy = jyy + 2000;
    //////////// 閏年の判定スタート
    if (jyy % 4 == 0) {
      if (jyy % 100 == 0) {
        if (jyy % 400 == 0) {
          leapY = 1;
        } else {
          leapY = 0;
        }
      } else {
        leapY = 1;
      }
    } else {
      leapY = 0;
    }
    //////////// 閏年の判定エンド
    mid(tmps, tmp, 0, 2);   // 2桁・日
    jdd = atoi(tmps);       // UTC日を数字に
    jdd = jdd + cdd;        // 24Hのオーバーフロー加算
    mid(tmps, tmp, 2, 2);   // 2桁・月
    jmm = atoi(tmps);       // UTC日を数字に
    //////////// 月の最終日の判定
    switch (jmm) {
      case 4:
      case 6:
      case 9:
      case 11:
        if (jdd > 30) {
          jdd = 1;
          cmm = 1;
        } else {
          cmm = 0;
        }
        break;
      case 2:
        if (leapY == 0) {
          if (jdd > 28) {
            jdd = 1;
            cmm = 1;
          } else {
            cmm = 0;
          }
        } else {
          if (jdd > 29) {
            jdd = 1;
            cmm = 1;
          } else {
            cmm = 0;
          }
        }
        break;
      default:            // 1 3 5 7 8 10 12
        if (jdd > 31) {
          jdd = 1;
          cmm = 1;
        } else {
          cmm = 0;
        }
        break;
    }
    //////////// 月の最終日の判定

    jmm = jmm + cmm;
    if (jmm > 12) {
      jmm = 1;
      jyy++;              // 年を進める
    }
    sprintf(tmps, "%04d", jyy);     // 4文字
    strcpy(nowDate, tmps);          // JST出力
    strcat(nowDate, "/");
    sprintf(tmps, "%02d", jmm);     // 2文字
    strcat(nowDate, tmps);          // JST出力
    strcat(nowDate, "/");
    sprintf(tmps, "%02d", jdd);     // 2文字
    strcat(nowDate, tmps);          // JST出力
    return 1;                      // 正常（時分秒と年月日の両方）抽出時は、1を返す。
  } else {
    return 0;
  }
}
#endif
// =======================================================================

#if defined(OPTION_UBLOX_NEO) && defined(SEL_UBLOX_NEO_7M)
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ＧＰＳデータから時分秒と使用中のＧＰＳ数を抽出して
// useGPS,nowTime にセットする。
// 対象データ：$GNGGA
// $GNGGA,001251.000,3445.9102,N,13527.9952,E,1,13,0.72,36.5,M,34.2,M,,*50<CR><LF>
// 時分秒.000, 北緯, 東経,「ＧＰＳのクオリティ0=No,1=OK,2=DGPS」, 使用衛星数, ]ほか
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

int8_t extractGPGGA(char *str) {
  int8_t ci = 0;
  uint8_t maxmin;
  char tmp[16];

  // 使用ＧＰＳ数を抽出
  if (cmmSTRcmm(tmp, str, 6) > 0) {
    switch (tmp[0]) {
      case '2':         // 0:受信不能　1:単独測位　2:DGPS
      case '1':
        ci = cmmSTRcmm(tmp, str, 7);

        if (ci == 0) {
          strcpy(useGPS, "00");        // ゼロ文字　"00"表記
        }
        if (ci == 1) {
          strcpy(useGPS, "0");                    // 1文字　先頭に"0”を付加
          strncat(useGPS, tmp, 1);
        }
        if (ci >= 2) {
          strncpy(useGPS, tmp, 2);    // 先頭二桁のみ抽出
        }
        break;
      default:
        gpsLoss++;
        strcpy(useGPS, "00");        // ゼロ文字　"00"表記
        break;
    }
    maxmin = atoi(useGPS);
    if (SATmin > maxmin) SATmin = maxmin;  //  最小衛星数を更新
    if (SATmax < maxmin) SATmax = maxmin;  //  最大衛星数を更新

  }
  return ci;                                     // 使用ＧＰＳの桁数を返す。0-1-2
}

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ＧＰＳデータから時分秒と年月日を抽出する。
// useGPS,nowTime nowDateにセットする。    U-Blox NEO で使用する
// 対象データ：$GNRMC
// $GNRMC,001252.000,A,3445.9100,N,13527.9957,E,0.11,275.89,040120,,,A*7C<CR><LF> <==AE_GNSS
// $GNRMC,012222.00,A,3445.90592,N,13528.01464,E,0.025,,260520,,,A*60 <==U-Blox NEO
// 時分秒.000,ステータスA有効:V無効,緯度,経度,速度,方向,日月年,地磁気,,モード*チェックSUM
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

int8_t extractGPRMC(char *str) {
  int8_t jhh, jdd, jmm, cdd, cmm, leapY;
  int16_t jyy;
  char fn[2] = "0";
  char tmp[16];
  char tmps[16];

  cdd = 0;

  // 時分秒を抽出
  if (cmmSTRcmm(tmp, str, 1) >= 9) {  //  HHMMDD.00 9文字以上なら（U-Bloxは9文字）
    mid(tmps, tmp, 0, 2);    // 2桁時
    jhh = atoi(tmps);        // UTC時間を数字に
    jhh = jhh + 9;                  // UTC +9H = JST
    if (jhh >= 24) {
      jhh = jhh - 24;  // 24H以上なら24Hを引く
      cdd = 1;
    }
    sprintf(tmps, "%02d", jhh);     // 2文字に変換
    strcpy(nowTime, tmps);          // JST出力
    strcat(nowTime, ":");
    mid(tmps, tmp, 2, 2);    // 2桁分
    strcat(nowTime, tmps);
    strcat(nowTime, ":");
    mid(tmps, tmp, 4, 2);    // 2桁秒
    strcat(nowTime, tmps);
    strcpy(tmpsec, tmps);     // 秒を記憶する■■
  } else {
    return 0;
  }

  if (cmmSTRcmm(tmp, str, 9) == 6) {  //  DDMMYY 6文字なら
    mid(tmps, tmp, 4, 2);   // 2桁・年
    jyy = atoi(tmps);       // UTC年を数字に
    jyy = jyy + 2000;
    //////////// 閏年の判定スタート
    if (jyy % 4 == 0) {
      if (jyy % 100 == 0) {
        if (jyy % 400 == 0) {
          leapY = 1;
        } else {
          leapY = 0;
        }
      } else {
        leapY = 1;
      }
    } else {
      leapY = 0;
    }
    //////////// 閏年の判定エンド
    mid(tmps, tmp, 0, 2);   // 2桁・日
    jdd = atoi(tmps);       // UTC日を数字に
    jdd = jdd + cdd;        // 24Hのオーバーフロー加算
    mid(tmps, tmp, 2, 2);   // 2桁・月
    jmm = atoi(tmps);       // UTC日を数字に
    //////////// 月の最終日の判定
    switch (jmm) {
      case 4:
      case 6:
      case 9:
      case 11:
        if (jdd > 30) {
          jdd = 1;
          cmm = 1;
        } else {
          cmm = 0;
        }
        break;
      case 2:
        if (leapY == 0) {
          if (jdd > 28) {
            jdd = 1;
            cmm = 1;
          } else {
            cmm = 0;
          }
        } else {
          if (jdd > 29) {
            jdd = 1;
            cmm = 1;
          } else {
            cmm = 0;
          }
        }
        break;
      default:            // 1 3 5 7 8 10 12
        if (jdd > 31) {
          jdd = 1;
          cmm = 1;
        } else {
          cmm = 0;
        }
        break;
    }
    //////////// 月の最終日の判定

    jmm = jmm + cmm;
    if (jmm > 12) {
      jmm = 1;
      jyy++;              // 年を進める
    }
    sprintf(tmps, "%04d", jyy);     // 4文字
    strcpy(nowDate, tmps);          // JST出力
    strcat(nowDate, "/");
    sprintf(tmps, "%02d", jmm);     // 2文字
    strcat(nowDate, tmps);          // JST出力
    strcat(nowDate, "/");
    sprintf(tmps, "%02d", jdd);     // 2文字
    strcat(nowDate, tmps);          // JST出力
    return 1;                      // 正常（時分秒と年月日の両方）抽出時は、1を返す。
  } else {
    return 0;
  }
}
#endif

// ★★★★★★★★★★★★★★★★★★★★★★★★★★★★★★★★

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ４つのキーによるパラメータ設定・変更操作
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

// ◆◆◆◆◆F0キーによる設定等の処理選択◆◆◆◆◆
#ifdef OPTION_SSD1309_OLED
uint8_t jobSelect(uint8_t def)        // F0キーにより、設定等の処理選択
{
  int8_t i, j, p, xl;
  int8_t stateRet = 0;
  char titleMsg[] = "Config MENU";
  const char* selMsg[] = {  "0.UART bps Set   ",      // 17文字以内
                            "1.SEND to HOST   ",
                            "2.Init. Gate time"
                         };   // このアイテム(item)の有効数-1（行数-1）を次に記述。
  const uint8_t item = 2;
  uint8_t tmpprm;
  j = def;
  p = 0;

  while (1)
  {
    u8g2.clearBuffer();        // OLEDクリヤー

    u8g2.setDrawColor(1);         // 反転用の処理のため
    u8g2.drawBox(0, 2, 128, 12);  // Box描画
    u8g2.setDrawColor(2);         //  color 2 set

    Text_setCursor(0, 0);       // 桁・行
    u8g2.print(titleMsg);       // タイトルの行を表示

    for (i = 0; i <= item; i++)
    {
      Text_setCursor(2, i + 1);   // 桁・行
      u8g2.print(selMsg[i]);
    }

    xl = (j + 2) * 12;                // マーカー位置指定
    u8g2.drawGlyph(5, xl , 0x25b6);   //   0x21d2⇒  0x25b6▶

    FuncEnable = B00001111;
    func_print(0, "RETN");
    func_print(1, " UP ");
    func_print(2, "DOWN");
    func_print(3, "ENTR");
    u8g2.sendBuffer();          // OLEDに送信（描画する）

    switch (buttonSW()) {
      case 10:
        stateRet = 1;
        break;
      case 11:
        //     case 21:
        j--;
        if (j < 0) j = item;
        break;
      case 12:
        //     case 22:
        j++;
        if (j > item) j = 0;
        break;
      case 13:
        //     case 23:
        switch (j) {
          case 0:                     // UART BPS set
            tmpprm = bpsPRM;
            bpsPRM = confPRMset(j, selMsg[j], tmpprm);
#ifdef OPTION_AE_GNSS
            switch (bpsPRM) {    // 以前の状態に関係なく速度切換を実行する。(2020/01/28)
              case 1:
                Serial1.println(bps384set);      // <cr><lf>を付けて速度をGPSに指示
                delay(50);
                Serial1.println(bps384set);      // <cr><lf>を付けて速度をGPSに指示(2回目)
                delay(50);
                Serial1.end();                    // シリアルを一度止める
                delay(50);
                Serial1.begin(38400);             // Serial1(PA9 PA10) オープン
                break;
              case 2:
                Serial1.println(bps1152set);      // <cr><lf>を付けて速度をGPSに指示
                delay(50);
                Serial1.println(bps1152set);      // <cr><lf>を付けて速度をGPSに指示(2回目)
                delay(50);
                Serial1.end();                    // シリアルを一度止める
                delay(50);
                Serial1.begin(115200);             // Serial1(PA9 PA10) オープン
                break;
              case 0:
              default:
                Serial1.println(bps96set);       // <cr><lf>を付けて速度をGPSに指示
                delay(50);
                Serial1.println(bps96set);       // <cr><lf>を付けて速度をGPSに指示(2回目)
                delay(50);
                Serial1.end();                    // シリアルを一度止める
                delay(50);
                Serial1.begin(9600);              // Serial1(PA9 PA10) オープン
                break;
            }
#endif  // OPTION_AE_GNSS
            // -----------------------------------------
#ifdef OPTION_UBLOX_NEO
            switch (bpsPRM) {    // NEOに対して、速度切換の指示を出してBBRに保存する。(2020/06/11)
              case 1:
                Serial1.write(bps384set, 28);      // NEOはバイナリー送信
                delay(50);
                Serial1.write(bps384set, 28);      // NEOはバイナリー送信
                delay(50);
                Serial1.end();
                delay(50);
                Serial1.begin(38400);             // Serial1(PA9 PA10) オープン
                delay(50);
                Serial1.write(bpsBBRset, 21);
                break;
              case 2:
                Serial1.write(bps1152set, 28);      // NEOはバイナリー送信
                delay(50);
                Serial1.write(bps1152set, 28);      // NEOはバイナリー送信
                delay(50);
                Serial1.end();
                delay(50);
                Serial1.begin(115200);             // Serial1(PA9 PA10) オープン
                delay(50);
                Serial1.write(bpsBBRset, 21);
                break;
              case 0:
              default:
                Serial1.write(bps96set, 28);      // NEOはバイナリー送信
                delay(50);
                Serial1.write(bps96set, 28);      // NEOはバイナリー送信
                delay(50);
                Serial1.end();
                delay(50);
                Serial1.begin(9600);             // Serial1(PA9 PA10) オープン
                delay(50);
                Serial1.write(bpsBBRset, 21);
                break;
            }
#endif

            if (bpsPRM != tmpprm) {        // 書き替えが有った
              eeprom_write(0, bpsPRM);            // ここにROM書込み
            }
            break;

          case 1:                     // SEND to HOST
            tmpprm = hostPRM;
            hostPRM = confPRMset(j, selMsg[j], tmpprm);   // 新しい方向をセット

            if (hostPRM != tmpprm) {        // 書き替えが有った
              eeprom_write(1, hostPRM);           // ここにROM書込み
            }
            break;

          case 2:                     // GATE time
            tmpprm = gatePRMon;
            gatePRMon = confPRMset(j, selMsg[j], tmpprm);

            if (gatePRMon != tmpprm) {       // 書き替えが有った
              eeprom_write(2, gatePRMon);           // ここにROM書込み
            }
            break;
        }   // switch (j) end
        break;

    }   // switch (buttonSW()) end

    if (stateRet == 1) {     // jobSelect(uint8_t)を抜ける
      return j;
    }
  }  // while (1) end
}
#endif // OPTION_SSD1309_OLED

// =======================================================================
#ifdef OPTION_ILI9341_TFT

uint8_t jobSelect(uint8_t def)        // F0キーにより、設定等の処理選択
{
  int8_t i, j, p, xl;
  int8_t stateRet = 0;
  char titleMsg[] = " Config MENU        ";

#ifdef OPTION_OCXO_Vcon  // I2C 0x60

  const char* selMsg[] = {  " 0.UART bps Set       ",    // 最大22文字
                            " 1.SEND to HOST       ",
                            " 2.Init. Gate time    ",
                            " 3.Touch calibration  ",
                            " 4.Check Touch PRM.   ",
                            " 5.Vcon Setting       "
                         };   // このアイテム(item)の有効数-1（行数-1）を次に記述。
  const uint8_t item = 5;
  uint8_t tmpprm;
  j = def;
  p = 0;
  display.fillScreen(ILI9341_BLACK);
  delay(99);
  funcDisp = 0;

  while (1)
  {

    TFT_setCursor(0, 0);       // 桁・行
    u8g2_for_adafruit_gfx.print(titleMsg);       // タイトルの行を表示

    if (funcDisp == 0) {
      //            012345678   0123
      FuncEnable = (B00111111 * 256) + B00001000; // line2,3,4,5, & F0, F3
      func_print(0, "RETN");
      //   func_print(1, " UP ");
      //   func_print(2, "DOWN");

#else   // else OPTION_OCXO_Vcon

  const char* selMsg[] = {  " 0.UART bps Set       ",    // 最大22文字
                            " 1.SEND to HOST       ",
                            " 2.Init. Gate time    ",
                            " 3.Touch calibration  ",
                            " 4.Check Touch PRM.   "
                         };   // このアイテム(item)の有効数-1（行数-1）を次に記述。
  const uint8_t item = 4;
  uint8_t tmpprm;
  j = def;
  p = 0;
  display.fillScreen(ILI9341_BLACK);
  delay(99);
  funcDisp = 0;

  while (1)
  {

    TFT_setCursor(0, 0);       // 桁・行
    u8g2_for_adafruit_gfx.print(titleMsg);       // タイトルの行を表示

    if (funcDisp == 0) {
      //            012345678   0123
      FuncEnable = (B00111111 * 256) + B00001000; // line2,3,4,5, & F0, F3
      func_print(0, "RETN");
      //   func_print(1, " UP ");
      //   func_print(2, "DOWN");

#endif

#ifdef SOFT_RESET // =================
      FuncEnable |= B0001;
      u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_YELLOW);
      func_print(3, "REST");
      u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
#endif            // =================

      TFT_LineCursor(0, j + 2, 26, ILI9341_YELLOW);   // 黄色の枠線

      funcDisp = 1;
    }

    for (i = 0; i <= item; i++)
    {
      TFT_setCursor(2, i + 2);   // 桁・行
      u8g2_for_adafruit_gfx.print(selMsg[i]);
    }

    switch (touchSW()) {
      case 10:
        stateRet = 1;
        break;

#ifdef SOFT_RESET // =================

      case 13:
        softResetON();
        //  stateRet = 1;
        break;

#endif            // =================

      case 22:                        // 0.UART bps Set
        j = 0;
        tmpprm = bpsPRM;
        bpsPRM = confPRMset(j, selMsg[j], tmpprm);;

#ifdef OPTION_AE_GNSS
        switch (bpsPRM) {    // 以前の状態に関係なく速度切換を実行する。(2020/01/28)
          case 1:
            Serial1.println(bps384set);      // <cr><lf>を付けて速度をGPSに指示
            delay(50);
            Serial1.println(bps384set);      // <cr><lf>を付けて速度をGPSに指示(2回目)
            delay(50);
            Serial1.end();                    // シリアルを一度止める
            delay(50);
            Serial1.begin(38400);             // Serial1(PA9 PA10) オープン
            break;
          case 2:
            Serial1.println(bps1152set);      // <cr><lf>を付けて速度をGPSに指示
            delay(50);
            Serial1.println(bps1152set);      // <cr><lf>を付けて速度をGPSに指示(2回目)
            delay(50);
            Serial1.end();                    // シリアルを一度止める
            delay(50);
            Serial1.begin(115200);             // Serial1(PA9 PA10) オープン
            break;
          case 0:
          default:
            Serial1.println(bps96set);       // <cr><lf>を付けて速度をGPSに指示
            delay(50);
            Serial1.println(bps96set);       // <cr><lf>を付けて速度をGPSに指示(2回目)
            delay(50);
            Serial1.end();                    // シリアルを一度止める
            delay(50);
            Serial1.begin(9600);              // Serial1(PA9 PA10) オープン
            break;
        }
#endif  // OPTION_AE_GNSS
        // -----------------------------------------
#ifdef OPTION_UBLOX_NEO
        switch (bpsPRM) {    // NEOに対して、速度切換の指示を出してBBRに保存する。(2020/06/11)
          case 1:
            Serial1.write(bps384set, 28);      // NEOはバイナリー送信
            delay(50);
            Serial1.write(bps384set, 28);      // NEOはバイナリー送信
            delay(50);
            Serial1.end();
            delay(50);
            Serial1.begin(38400);             // Serial1(PA9 PA10) オープン
            delay(50);
            Serial1.write(bpsBBRset, 21);
            break;
          case 2:
            Serial1.write(bps1152set, 28);      // NEOはバイナリー送信
            delay(50);
            Serial1.write(bps1152set, 28);      // NEOはバイナリー送信
            delay(50);
            Serial1.end();
            delay(50);
            Serial1.begin(115200);             // Serial1(PA9 PA10) オープン
            delay(50);
            Serial1.write(bpsBBRset, 21);
            break;
          case 0:
          default:
            Serial1.write(bps96set, 28);      // NEOはバイナリー送信
            delay(50);
            Serial1.write(bps96set, 28);      // NEOはバイナリー送信
            delay(50);
            Serial1.end();
            delay(50);
            Serial1.begin(9600);             // Serial1(PA9 PA10) オープン
            delay(50);
            Serial1.write(bpsBBRset, 21);
            break;
        }
#endif

        if (bpsPRM != tmpprm) {        // 書き替えが有った
          eeprom_write(0, bpsPRM);            // ここにROM書込み
        }
        break;

      case 23:                     // 1.SEND to HOST
        j = 1;
        tmpprm = hostPRM;
        hostPRM = confPRMset(j, selMsg[j], tmpprm);          // 新しい方向をセット

        if (hostPRM != tmpprm) {        // 書き替えが有った
          eeprom_write(1, hostPRM);           // ここにROM書込み
        }
        break;

      case 26:                      // Check Touch PRM  26
        j = 4;
        touchPrmCheck(selMsg[j]);
        break;

#ifdef OPTION_OCXO_Vcon  // I2C 0x60
      case 27:                      // 5.Vcon Setting
        j = 5;
        tmpprm = vconPRM;
        vconPRM = confPRMset5(j, selMsg[j], tmpprm);
        break;
#endif

      case 24:                     // 2.GATE time
        j = 2;
        tmpprm = gatePRMon;
        gatePRMon = confPRMset(j, selMsg[j], tmpprm);

        if (gatePRMon != tmpprm) {       // 書き替えが有った
          eeprom_write(2, gatePRMon);           // ここにROM書込み
        }
        break;

      case 25:                     // Touch calibration 25
        j = 3;
        uint16_t xxMin = tchXmin;
        uint16_t yyMin = tchYmin;
        uint16_t xxMax = tchXmax;
        uint16_t yyMax = tchYmax;

        if (touchCALB(&xxMin, &yyMin, &xxMax, &yyMax) == 1) { // 1 == 書き変え有り
          eeprom_write_uint16(3, xxMin);
          tchXmin = xxMin;
          eeprom_write_uint16(5, yyMin);
          tchYmin = yyMin;
          eeprom_write_uint16(7, xxMax);
          tchXmax = xxMax;
          eeprom_write_uint16(9, yyMax);
          tchYmax = yyMax;
        }
        break;

    }   // switch (touchSW()) end

    if (stateRet == 1) {     // jobSelect(uint8_t)を抜ける
      return j;
    }
  }  // while (1) end
}

#endif  // end OPTION_ILI9341_TFT

// =======================================================================
#ifdef OPTION_SSD1309_OLED
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ◆◆◆◆◆M ボタンの状態を返す◆◆
// 何れかのボタンが押されて放された(F0=10,F1=11,F2=12,F3=13)を待つ。
// 旧設定
// F0 < 80, 300 < F1 > 430, 580 < F2 < 790, 800 < F3 < 1100
// New   (2020/04/16)
// F0 < 109, 110 < F1 > 548, 549 < F2 < 823, 823 < F3 < 1200,
// ReNew   (2020/06/16)
// F0 = 0V (0), F1 = 0.3V (372), F2 = 0.55V (682), F3 = 0.76V (945)  (AD値)
// F0 < 100, 316 < F1 > 428, 580 < F2 < 784, 900 < F3 < 1150,  <==(±15%)
// F0 < 100, 335 < F1 > 409, 614 < F2 < 750, 851 < F3 < 1040,  <==(±10%)*2021/11/19
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
int8_t buttonSW() {
  int8_t retVal = 0;      // = 0 の記述は必須。
  int8_t repeat10 = 0;
  int8_t count = 0;
  uint16_t asw;

  // while (1) {
  asw = analogRead(SWa_In);
  if (asw < 100) {  // F0ボタンなら return 10

#ifdef BOARD_V5_BUZZER
    beepms(50);     // キー動作のビープ音
#endif

    while (analogRead(SWa_In) < 100) { // F0ボタンを放すまで待つ
      delay(50);
      count++;
      if (count >= 20) {        // 1000mS 以上押されたら10プラス
        repeat10 = 10;

#ifdef BOARD_V5_BUZZER
        beepms(10);     // キー動作のビープ音
#endif

      }
    }
    delay(50);
    return 10 + repeat10;
  }

  if ((asw > 355) && (asw < 409)) { // F1ボタンなら Plus

#ifdef BOARD_V5_BUZZER
    beepms(50);     // キー動作のビープ音
#endif

    while ((analogRead(SWa_In) > 355) && (analogRead(SWa_In) < 409)) { // F1ボタンを放すまで待つ
      delay(50);
      count++;
      if (count >= 20) {        // 1000mS 以上押されたら10プラス
        repeat10 = 10;

#ifdef BOARD_V5_BUZZER
        beepms(10);     // キー動作のビープ音
#endif

      }
    }
    delay(50);
    return 11 + repeat10;
  }

  if ((asw > 612) && (asw < 750)) {   // F2ボタンなら Minus

#ifdef BOARD_V5_BUZZER
    beepms(50);     // キー動作のビープ音
#endif

    while ((analogRead(SWa_In) > 612) && (analogRead(SWa_In) < 750)) { // F2ボタンを放すまで待つ
      delay(50);
      count++;
      if (count >= 20) {        // 1000mS 以上押されたら10プラス
        repeat10 = 10;

#ifdef BOARD_V5_BUZZER
        beepms(10);     // キー動作のビープ音
#endif

      }
    }

    delay(50);
    return 12 + repeat10;
  }

  if ((asw > 851) && (asw < 1041)) {  // F3ボタンなら return 13

#ifdef BOARD_V5_BUZZER
    beepms(50);     // キー動作のビープ音
#endif

    while ((analogRead(SWa_In) > 851) && (analogRead(SWa_In) < 1041)) { // F3ボタンを放すまで待つ
      delay(50);
      count++;
      if (count >= 20) {        // 1000mS 以上押されたら10プラス
        repeat10 = 10;

#ifdef BOARD_V5_BUZZER
        beepms(10);     // キー動作のビープ音
#endif

      }
    }

    delay(50);
    return 13 + repeat10;
  }

  // } // while end
}
#endif

#ifdef OPTION_ILI9341_TFT

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ボタンSWの代替
// 返り値：F0=10, F1=11, F2=12, F3=13
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

int8_t touchSW() {
  uint16_t xtmp = 0;
  uint16_t ytmp = 0;
  int8_t ans2;
  // ans2 = -1;
  // ==================
  //  while (ans2 < 0) {

  ans2 = touchCCxLL(&xtmp, &ytmp);
  if (ytmp == 8) {
    if ((xtmp >= 2) && (xtmp <= 5)) ans2 = 10;
    if ((xtmp >= 8) && (xtmp <= 11)) ans2 = 11;
    if ((xtmp >= 14) && (xtmp <= 17)) ans2 = 12;
    if ((xtmp >= 20) && (xtmp <= 23)) ans2 = 13;
  }
  if ((FuncEnable & 0x8000) > 0) {  // 許可ライン
    if (ytmp == 0) ans2 = 20;
  }
  if ((FuncEnable & 0x4000) > 0) {  // 許可ライン
    if (ytmp == 1) ans2 = 21;
  }
  if ((FuncEnable & 0x2000) > 0) {  // 許可ライン
    if (ytmp == 2) ans2 = 22;
  }
  if ((FuncEnable & 0x1000) > 0) {  // 許可ライン
    if (ytmp == 3) ans2 = 23;
  }
  if ((FuncEnable & 0x0800) > 0) {  // 許可ライン
    if (ytmp == 4) ans2 = 24;
  }
  if ((FuncEnable & 0x0400) > 0) {  // 許可ライン
    if (ytmp == 5) ans2 = 25;
  }
  if ((FuncEnable & 0x0200) > 0) {  // 許可ライン
    if (ytmp == 6) ans2 = 26;
  }
  if ((FuncEnable & 0x0100) > 0) {  // 許可ライン
    if (ytmp == 7) ans2 = 27;
  }
  xtmp = 0;
  ytmp = 0;
  //  } // while end
  // ==================

  return ans2;
}

#endif

#ifdef OPTION_SSD1309_OLED
///////////////////////////////////////////////////////////////////////
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// 処理選択画面を展開し、選択された値を返す。
// 処理のタイトルは、上位区で記述したものを使うよう変更した。(2021/01/30)
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
uint8_t confPRMset(uint8_t jobNo, const char *title0, uint8_t prm)
{
  int8_t i, j, p, xl;
  int8_t stateRet = 0;
  /*  const char* titleMsg[] = {
      "0.UART BPS Set",
      "1.SEND to HOST",
      "2.Init. Gate time"
    };*/
  const char selMsg[3][3][22] = {
    { "9,600 bps(default)",   // jobNo = 0
      "38,400 bps",
      "115,200 bps"
    },
    { "GPS data(default)",    // jobNo = 1
      "LOG data",
      "PIC"
    },
    { "1 sec(default)",        // jobNo = 2
      "10 sec",
      "100 sec"
    }
  };
  int8_t item;
  uint8_t tmpprm = 0;

  switch (jobNo) {    // 選択する項目(selMsg)の数を処理No毎にセットする。012
    case 0:
      item = 2;
      break;
    case 1:
      item = 2;
      break;
    case 2:
      item = 2;
      break;
  };
  j = prm;
  p = 0;

  while (1)
  {
    u8g2.clearBuffer();        // OLEDクリヤー

    u8g2.setDrawColor(1);         // 反転用の処理のため
    u8g2.drawBox(0, 2, 128, 12);  // Box描画
    u8g2.setDrawColor(2);         //  color 2 set
    Text_setCursor(0, 0);         // 桁・行
    u8g2.print(title0);            // タイトルの行を表示

    for (i = 0; i <= item; i++)
    {
      Text_setCursor(2, i + 1);   // 桁・行
      u8g2.print(selMsg[jobNo][i]);
    }

    xl = (j + 2) * 12;                // マーカー位置指定
    u8g2.drawGlyph(5, xl , 0x25b6);   //   0x21d2⇒  0x25b6▶

    func_print(0, "RETN");
    func_print(1, " UP ");
    func_print(2, "Down");
    func_print(3, "ENTR");
    u8g2.sendBuffer();          // OLEDに送信（描画する）

    switch (buttonSW()) {
      case 10:
        tmpprm = prm;            // 初期値で戻る
        stateRet = 1;
        break;
      case 11:
      case 21:
        j--;
        if (j < 0) j = item;
        break;
      case 12:
      case 22:
        j++;
        if (j > item) j = 0;
        break;
      case 13:
        switch (j) {
          case 0:
          case 1:
          case 2:
            tmpprm = j;
            stateRet = 1;       // 終了処理をセット
            break;
        }   // switch (j) end
        break;

    }   // switch (buttonSW()) end

    if (stateRet == 1) {     // jobSelect(int8_t)を抜ける
      return tmpprm;
    }
  }  // while (1) end
}
#endif // OPTION_SSD1309_OLED


#ifdef OPTION_ILI9341_TFT
///////////////////////////////////////////////////////////////////////
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// 処理選択画面を展開し、選択された値を返す。
// 処理のタイトルは、上位区で記述したものを使うよう変更した。(2021/01/30)
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
uint8_t confPRMset(uint8_t jobNo, const char *title0, uint8_t prm)      // TFT
{
  int8_t i, j, p, xl;
  int8_t stateRet = 0;
  /*  const char* titleMsg[] = {
      " 0.UART BPS Set     ",
      " 1.SEND to HOST     ",
      " 2.Init. Gate time  "
    };
  */
  const char selMsg[4][5][25] = {
    { " 9,600 bps(default)  ",   // jobNo = 0
      " 38,400 bps          ",
      " 115,200 bps         "
    },
    { " GPS data(default)   ",    // jobNo = 1
      " LOG data            ",
      " PIC                 "
    },
    { " 1 sec(default)      ",        // jobNo = 2
      " 10 sec              ",
      " 100 sec             "
    }
  };

  int8_t item;
  uint8_t tmpprm = 0;

  switch (jobNo) {    // 選択する項目(selMsg)の「数-1」を処理No毎にセットする。
    case 0:
      item = 2;     // 3アイテムは、2指定
      break;
    case 1:
      item = 2;
      break;
    case 2:
      item = 2;
      break;
  };
  j = prm;
  p = 0;
  display.fillScreen(ILI9341_BLACK);
  delay(99);
  funcDisp = 0;

  while (1)
  {
    TFT_setCursor(1, 0);       // 桁・行
    u8g2_for_adafruit_gfx.print(title0);       // タイトルの行を表示

    if (funcDisp == 0) {
      //            012345678   0123
      FuncEnable = (B00111000 * 256) + B00001000; // line2,3,4, & F0,
      func_print(0, "RETN");
      //  func_print(1, " UP ");
      //  func_print(2, "Down");
      //  func_print(3, "ENTR");

      TFT_LineCursor(0, j + 2, 26, ILI9341_YELLOW);

      funcDisp = 1;
    }

    for (i = 0; i <= item; i++)
    {
      TFT_setCursor(2, i + 2);   // 桁・行
      u8g2_for_adafruit_gfx.print(selMsg[jobNo][i]);
    }

    switch (touchSW()) {
      case 10:              // 「RETN」ボタン
        tmpprm = prm;       // 初期値で戻る
        stateRet = 1;       // 終了処理をセット
        break;
      case 22:              // 2行目
        tmpprm = 0;
        stateRet = 1;       // 終了処理をセット
        break;
      case 23:              // 3行目
        tmpprm = 1;
        stateRet = 1;       // 終了処理をセット
        break;
      case 24:              // 4行目
        tmpprm = 2;
        stateRet = 1;       // 終了処理をセット
        break;
    }   // switch (touchSW()) end

    if (stateRet == 1) {     // jobSelect(int8_t)を抜ける

      TFT_LineCursor(0, j + 2, 26, ILI9341_BLACK);  // 元の下地を黒に戻す
      TFT_setCursor(2, j + 2);   // 桁・行
      u8g2_for_adafruit_gfx.print(selMsg[jobNo][j]);

      TFT_LineCursor(0, tmpprm + 2, 26, ILI9341_YELLOW);  // 新たな選択の下地を入れる
      TFT_setCursor(2, tmpprm + 2);   // 桁・行
      u8g2_for_adafruit_gfx.print(selMsg[jobNo][tmpprm]);
      delay(500);
      TFT_LineCursor(0, tmpprm + 2, 26, ILI9341_BLACK);  // 元の下地を黒に戻す
      funcDisp = 0;
      return tmpprm;
    }
  }  // while (1) end
}

#ifdef OPTION_OCXO_Vcon  // I2C 0x60

///////////////////////////////////////////////////////////////////////
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// 処理選択画面を展開し、選択された値を返す。5行バージョン
// 処理のタイトルは、上位区で記述したものを使うよう変更した。(2021/01/30)
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
uint8_t confPRMset5(uint8_t jobNo, const char *title0, uint8_t prm)      // TFT
{
  int8_t i, j, jb, u8tmp;
  int8_t stateRet = 0;
  /*  const char* titleMsg[] = {
      " 5.Vcon Setting     ",
      " 6.Next JOB Reserve "
    }; */
  const char selMsg[2][6][25] = {
    { "0.Connect & Config    ",        // vconCONNx_Config()
      "1.Auto/Manual MODE    ",        // vconMODEcng()
      "2.DAC value input     ",        // tenkey() & vconDACset()
      "3.mHz(1LSB)Input&Save ",        // tenkey() & vcondmHzSave()
      "4.DACv Write EEPROM   " ,        // vconEEPROMwrite()
      "5.(Measu HELP GUIDE)  "         // vconEEPROMwrite()
    },
    { "0.Reserve             ",        // jobNo = 6
      "1.345678901234567890  ",
      "2.Reserve             ",
      "3.Reserve             ",
      "4.Reserve             ",
      "5.Reserve             "
    }
  };

  int8_t item;
  uint8_t tmpprm = 0;

  switch (jobNo) {    // 選択する項目(selMsg)の「数-1」を処理No毎にセットする。
    case 5:
      item = 5;     // 6アイテムは、5指定
      jb = 0;
      break;
    case 6:
      item = 5;
      jb = 1;
      break;
  };
  j = prm;

  display.fillScreen(ILI9341_BLACK);
  delay(99);
  funcDisp = 0;

  while (1) {
    TFT_setCursor(1, 0);       // 桁・行
    u8g2_for_adafruit_gfx.print(title0);       // タイトルの行を表示

    if (funcDisp == 0) {

      //            012345678   0123
      FuncEnable = (B00111111 * 256) + B00001000; // line2,3,4,5,6,7 & F0,
      func_print(0, "RETN");
      //  func_print(1, " UP ");
      //  func_print(2, "Down");
      //  func_print(3, "ENTR");

      TFT_LineCursor(0, j + 2, 26, ILI9341_YELLOW);

      funcDisp = 1;
    }

    for (i = 0; i <= item; i++)
    {
      TFT_setCursor(1, i + 2);   // 桁・行
      sprintf(tmptmp, "% 1d", jobNo);
      u8g2_for_adafruit_gfx.print(tmptmp);        // jobNo の数字を表示して
      u8g2_for_adafruit_gfx.print(selMsg[jb][i]); // 各行の内容を表示
    }

    switch (touchSW()) {  // 押された行の値を返す

      case 10:                  // 「RETN」ボタン
        tmpprm = j;         // 選択値を記憶
        stateRet = 1;
        break;
      case 22:              // 2行目    vconCONNx_Config(recall)
        tmpprm = 0;
        vconCONNx_Config(selMsg[jb][tmpprm]);
        break;
      case 23:
        tmpprm = 1;
        u8tmp = VconAUTO;
        VconAUTO = vconMODEcng(selMsg[jb][tmpprm], u8tmp);
        if (VconAUTO != u8tmp) {
          eeprom_write(0x18, VconAUTO);      // 後でエラー判定を入れる
        }
        //     funcDisp = 0;
        break;
      case 24:
        tmpprm = 2;
        VconValue = (uint16_t)tenkey(0, 4095, (double)VconValue);
        if (VconI2C == 0) {     // Vcon 接続中なら
          vconDACset(VconValue);   // ■■ Vconへ変更後の値をセット■■
        }
        break;
      case 25:
        tmpprm = 3;
        UNI.dmHz = tenkey(0, 10, UNI.dmHz);    // 10Key処理へ。値は UNI.dmHz
        vcondmHzSave(selMsg[jb][tmpprm]);   // ■■ 本体EEPROMへ変更後 dmHz の値をセット■■
        break;
      case 26:
        tmpprm = 4;
        vconEEPROMwrite(selMsg[jb][tmpprm]);
        break;
      case 27:
        tmpprm = 5;
        vconHELPdisp(selMsg[jb][tmpprm]);
        break;
    }   // switch (touchSW()) end

    if (stateRet == 1) {

      TFT_LineCursor(0, j + 2, 26, ILI9341_BLACK);  // 元の下地を黒に戻す
      delay(100);    // これは入れておく（御呪い）以前は　500
      //     TFT_setCursor(2, j + 2);   // 桁・行
      //     u8g2_for_adafruit_gfx.print(selMsg[jobNo][j]);
      //     delay(1500);
      //     TFT_LineCursor(0, tmpprm + 2, 26, ILI9341_YELLOW);  // 新たな選択の下地を入れる
      //     TFT_setCursor(2, tmpprm + 2);   // 桁・行
      //     u8g2_for_adafruit_gfx.print(selMsg[jobNo][tmpprm]);
      //     delay(1500);
      //     TFT_LineCursor(0, tmpprm + 2, 26, ILI9341_BLACK);  // 元の下地を黒に戻す

      break;
    }
    j = tmpprm;

  }  // while (1) end
  funcDisp = 0;
  return tmpprm;
}

///////////////////////////////////////////////////////////////////////
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// Vconのスタート＆設定状態の表示　をする
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
uint8_t vconCONNx_Config(const char *title0)      // TFT
{
  uint8_t tmpprm = 0, stateRet = 0;
  //  char titleMsg[] = "Connect & Config   ";

  display.fillScreen(ILI9341_BLACK);
  delay(99);
  funcDisp = 0;

  TFT_setCursor(1, 0);       // 桁・行
  u8g2_for_adafruit_gfx.print("5");
  u8g2_for_adafruit_gfx.print(title0);       // タイトルの行を表示


  VconI2C = vconStartCHK();   // 接続チェック＆EPROM読み出し

  if (funcDisp == 0) {
    //            012345678   0123
    FuncEnable = (B00000000 * 256) + B00001000; //  F0,
    func_print(0, "RETN");
    //  func_print(1, " UP ");
    //  func_print(2, "Down");
    //  func_print(3, "ENTR");

    TFT_setCursor(2, 2);   // 桁・行
    u8g2_for_adafruit_gfx.print("Status   :");
    TFT_setCursor(12, 2);   // 桁・行
    if (VconI2C == 0) {
      u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_MAGENTA);
      u8g2_for_adafruit_gfx.print("CONNECT");
      u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
    } else {
      u8g2_for_adafruit_gfx.print("vcOFF ");
    }

    TFT_setCursor(2, 3);   // 桁・行
    u8g2_for_adafruit_gfx.print("MODE     :");
    TFT_setCursor(12, 3);   // 桁・行
    if (VconAUTO == 1) {
      u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_MAGENTA);
      u8g2_for_adafruit_gfx.print("Auto  ");
      u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
    } else {
      u8g2_for_adafruit_gfx.print("Manual");
    }

    TFT_setCursor(2, 4);   // 桁・行
    u8g2_for_adafruit_gfx.print("DAC value:");
    TFT_setCursor(12, 4);   // 桁・行
    sprintf(tmptmp, "%4d", VconValue);
    u8g2_for_adafruit_gfx.print(tmptmp);

    sprintf(tmptmp, " (cfg=%2X)", getMODE);
    u8g2_for_adafruit_gfx.print(tmptmp);

    TFT_setCursor(2, 5);   // 桁・行
    u8g2_for_adafruit_gfx.print("mHz(1LSB):");
    TFT_setCursor(12, 5);   // 桁・行
    sprintf(tmptmp, "%8.5lf", UNI.dmHz);
    u8g2_for_adafruit_gfx.print(tmptmp);
    u8g2_for_adafruit_gfx.print("mHz");

    TFT_setCursor(2, 6);   // 桁・行
    u8g2_for_adafruit_gfx.print("EEPROM   :");
    TFT_setCursor(12, 6);   // 桁・行
    if (VconI2C == 0) {
      sprintf(tmptmp, "%4d", getEPROM);
      u8g2_for_adafruit_gfx.print(tmptmp);
    } else {
      u8g2_for_adafruit_gfx.print("unknown");
    }
  }
  while (1) {
    switch (touchSW()) {  // 押された行の値を返す

      case 10:                  // 「RETN」ボタン
        stateRet = 1;
        break;
    }   // switch (touchSW()) end
    if (stateRet == 1) break;
  }
  display.fillScreen(ILI9341_BLACK);
  delay(99);
  funcDisp = 0;
  return tmpprm;;

}

///////////////////////////////////////////////////////////////////////
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// Vconの実行モード　自動／手動の切り換え
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
uint8_t vconMODEcng(const char *title0, uint8_t preVal)      // TFT
{
  uint8_t tmpprm = preVal;  // 返り値
  uint8_t stateRet = 0;     // 終了判定

  display.fillScreen(ILI9341_BLACK);
  delay(99);
  funcDisp = 0;

  TFT_setCursor(1, 0);       // 桁・行
  u8g2_for_adafruit_gfx.print("5");
  u8g2_for_adafruit_gfx.print(title0);       // タイトルの行を表示

  while (1) {
    if (funcDisp == 0) {
      //            012345678   0123
      FuncEnable = (B00000000 * 256) + B00001110; //  F0, F1, F2
      func_print(0, "RETN");
      func_print(1, "AUTO");
      func_print(2, "MANL");
      //  func_print(3, "ENTR");

      funcDisp = 1;
    }
    TFT_setCursor(2, 2);   // 桁・行
    u8g2_for_adafruit_gfx.print("MODE     :");
    TFT_setCursor(12, 2);   // 桁・行

    u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_MAGENTA);
    if (tmpprm == 1) {
      u8g2_for_adafruit_gfx.print("Auto  ");
    } else {
      u8g2_for_adafruit_gfx.print("Manual");
    }
    u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);

    switch (touchSW()) {  // 押されたFuncボタンの値を返す

      case 10:                  // 「RETN」ボタン
        stateRet = 1;
        break;
      case 11:                  // 「AUTO」ボタン
        tmpprm = 1;
        break;
      case 12:                  // 「MANL」ボタン
        tmpprm = 0;
        break;

    }   // switch (touchSW()) end

    if (stateRet == 1) break;
  }
  display.fillScreen(ILI9341_BLACK);    // 入れないと黄色い枠が残ることあり？
  delay(99);
  funcDisp = 0;
  return tmpprm;;
}

///////////////////////////////////////////////////////////////////////
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// Vcon 接続確認・dmHz書込み(本体EEPROM)処理画面　2秒間のみ
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void vcondmHzSave(const char *title0)      // TFT
{
  display.fillScreen(ILI9341_BLACK);
  delay(99);
  TFT_setCursor(1, 0);   // 桁・行
  u8g2_for_adafruit_gfx.print("5");
  u8g2_for_adafruit_gfx.print(title0);

  TFT_setCursor(2, 4);   // 桁・行
  u8g2_for_adafruit_gfx.print("mHz(1LSB):");
  TFT_setCursor(12, 4);   // 桁・行

  eeprom_write_double(0x10, UNI.intrC);

  sprintf(tmptmp, "%8.5lf", UNI.dmHz);
  u8g2_for_adafruit_gfx.print(tmptmp);
  if (UNI.dmHz == 0) {
    //    u8g2_for_adafruit_gfx.setFont(u8g2_font_profont29_mr);  // 大きい文字  W16xH29 (A19) WHITE YELLOW
    fonSet_BIG();

    // u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
    u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_YELLOW);
    TFT_setCursor10(1, 4);   // 桁・行
    u8g2_for_adafruit_gfx.print("Value is ZERO ?!");
    delay(2000);
    TFT_setCursor10(1, 4);   // 桁・行
    u8g2_for_adafruit_gfx.print("                ");
    //    u8g2_for_adafruit_gfx.setFont(u8g2_font_profont22_mf);  // OK (A14)  W12xH22
    fonSet_MID();
    u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
  }
  delay(2000);
  funcDisp = 0;
}

///////////////////////////////////////////////////////////////////////
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// Vcon EEPROM への書込み処理画面 vconALLMEMset  Read failure　2秒間のみ
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void vconEEPROMwrite(const char *title0)      // TFT
{
  display.fillScreen(ILI9341_BLACK);
  delay(99);
  TFT_setCursor(1, 0);   // 桁・行
  u8g2_for_adafruit_gfx.print("5");
  u8g2_for_adafruit_gfx.print(title0);

  TFT_setCursor(2, 5);   // 桁・行
  u8g2_for_adafruit_gfx.print("EEPROM:");
  TFT_setCursor(9, 5);   // 桁・行
  if (VconI2C == 0) {
    vconALLMEMset(VconValue);   // ■■■書込み
    if (vconALLread() == 0) {
      if (VconValue == getEPROM) {
        sprintf(tmptmp, "%4d", getEPROM);     // 正常時は値を表示
        u8g2_for_adafruit_gfx.print(tmptmp);
        u8g2_for_adafruit_gfx.print(" SUCCESS");
        TFT_setCursor(9, 6);   // 桁・行
        sprintf(tmptmp, "%4X", getMODE);     // 正常時は値を表示
        u8g2_for_adafruit_gfx.print(tmptmp);
      } else {
        u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_RED);
        u8g2_for_adafruit_gfx.print("Bad writing");
        u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
      }
    } else {
      u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_RED);
      u8g2_for_adafruit_gfx.print("Read failure");
      u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
    }
  } else {
    u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_RED);
    u8g2_for_adafruit_gfx.print("disconnected");
    u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
  }
  delay(2000);
  funcDisp = 0;
}

///////////////////////////////////////////////////////////////////////
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// Vconのヘルプの表示をする
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
uint8_t vconHELPdisp(const char *title0)      // TFT
{
  uint8_t tmpprm = 0, stateRet = 0;

  display.fillScreen(ILI9341_BLACK);
  delay(99);
  funcDisp = 0;

  TFT_setCursor(1, 0);   // 桁・行
  u8g2_for_adafruit_gfx.print("5");
  u8g2_for_adafruit_gfx.print(title0);       // タイトルの行を表示

  if (funcDisp == 0) {
    //            012345678   0123
    FuncEnable = (B00000000 * 256) + B00001000; //  F0,
    func_print(0, "RETN");
    //  func_print(1, " UP ");
    //  func_print(2, "Down");
    //  func_print(3, "ENTR");

    TFT_setCursor(1, 2);   // 桁・行
    u8g2_for_adafruit_gfx.print("0.GATE=1000s/Manual SEL ");
    TFT_setCursor(1, 3);   // 桁・行
    u8g2_for_adafruit_gfx.print("1.DAC=1000 > [STRT] > f1");
    TFT_setCursor(1, 4);   // 桁・行
    u8g2_for_adafruit_gfx.print("2.DAC=3000 > [STRT] > f2");
    TFT_setCursor(1, 5);   // 桁・行
    u8g2_for_adafruit_gfx.print("3.fd=(f1-f2)*1000 (mHz) ");
    TFT_setCursor(1, 6);   // 桁・行
    u8g2_for_adafruit_gfx.print("4.mHz(1LSB)=fd/2000     ");
    TFT_setCursor(1, 7);   // 桁・行
    u8g2_for_adafruit_gfx.print("5.MODE:Auto > [STRT]    ");
    //                              0123456789012345678901
  }
  while (1) {
    switch (touchSW()) {  // 押された行の値を返す

      case 10:                  // 「RETN」ボタン
        stateRet = 1;
        break;
    }   // switch (touchSW()) end

    if (stateRet == 1) break;
  }
  display.fillScreen(ILI9341_BLACK);
  delay(99);
  funcDisp = 0;
  return tmpprm;;
}


///////////////////////////////////////////////////////////////////////
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// エラー発生時のメッセージを表示をする
//
// 例： errDisp("#00 vconStartCHK() ");
// メッセージ文字数＝Max19文字
// #00 連番
// +15文字（関数名等）
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
uint8_t errDisp(const char *eMessage)      // TFT
{
  uint8_t tmpprm = 0, stateRet = 0;

  display.fillScreen(ILI9341_BLACK);
  delay(99);
  funcDisp = 0;

  if (funcDisp == 0) {
    //            012345678   0123
    FuncEnable = (B00000000 * 256) + B00001000; //  F0,
    func_print(0, "RETN");
    //  func_print(1, " UP ");
    //  func_print(2, "Down");
    //  func_print(3, "ENTR");
    funcDisp = 1;
  }
  //  u8g2_for_adafruit_gfx.setFont(u8g2_font_profont29_mr);  // 大きい文字  W16xH29 (A19) WHITE YELLOW
  fonSet_BIG();
  u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
  TFT_setCursorB(1, 0);   // 桁・行
  u8g2_for_adafruit_gfx.print("==ERROR Mssg.==");
  u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_YELLOW);

  TFT_setCursorB(0, 3);   // 桁・行
  u8g2_for_adafruit_gfx.print(eMessage);

  while (1) {

    switch (touchSW()) {  // 押された行の値を返す

      case 10:                  // 「RETN」ボタン
        stateRet = 1;
        break;
    }   // switch (touchSW()) end

    if (stateRet == 1) break;
  } // while

  display.fillScreen(ILI9341_BLACK);
  delay(99);
  funcDisp = 0;
  //  u8g2_for_adafruit_gfx.setFont(u8g2_font_profont22_mf);  // OK (A14)  W12xH22
  fonSet_MID();

  u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
  // return 0;
}



#endif      // end OPTION_OCXO_Vcon

#ifdef SOFT_RESET // =================

///////////////////////////////////////////////////////////////////////
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// SOFT_RESETを実行する
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
void softResetON()          // TFT
{
  int32_t timeprm = 5;     // ソフトリセット猶予カウント Sec
  int32_t dmSecEnd;
  float flttmp;
  uint8_t stateRet = 0;

  display.fillScreen(ILI9341_BLACK);
  delay(99);
  funcDisp = 0;

  if (funcDisp == 0) {
    //            012345678   0123
    FuncEnable = (B00000000 * 256) + B00001000; //  F0,
    func_print(0, "RETN");
    //  func_print(1, " UP ");
    //  func_print(2, "Down");
    //  func_print(3, "ENTR");
    funcDisp = 1;
  }

  //  u8g2_for_adafruit_gfx.setFont(u8g2_font_profont29_mr);  // 大きい文字  W16xH29 (A19) WHITE YELLOW
  fonSet_BIG();


  u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
  TFT_setCursorB(1, 0);   // 桁・行
  u8g2_for_adafruit_gfx.print("==SOFT RESET==");
  u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_YELLOW);

  TFT_setCursorB(1, 3);   // 桁・行
  u8g2_for_adafruit_gfx.print("Run in ");

  tIntCount1 = 0;     // 10mSecカウンタ
  timeprm *= 100;     // 10mSecに変換
  dmSecEnd = timeprm; // 10mSec単位の終了カウント数をセット

  while (1) {
    timeprm = dmSecEnd - tIntCount1;
    flttmp = (float)timeprm / 100;   // 秒に変換

    if (timeprm <= 0) {
      digitalWrite(RESET_PORT, HIGH);  // リセット実行
      // 次の命令は実行されないはず
      stateRet = 1;   // もし回路が付いていなかったら戻る
    }

    TFT_setCursorB(9, 3);   // 桁・行
    sprintf(tmptmp, "%4.1f", flttmp);   // 小数点以下１桁までに変更(2021/02/28)
    u8g2_for_adafruit_gfx.print(tmptmp);
    u8g2_for_adafruit_gfx.print(" Sec");

    switch (touchSW()) {  // 押された行の値を返す

      case 10:                  // 「RETN」ボタン
        stateRet = 1;
        break;
    }   // switch (touchSW()) end

    if (stateRet == 1) break;
  } // while

  display.fillScreen(ILI9341_BLACK);
  delay(99);
  funcDisp = 0;
  //  u8g2_for_adafruit_gfx.setFont(u8g2_font_profont22_mf);  // OK (A14)  W12xH22
  fonSet_MID();
  u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
  // return;
}

#endif          // =================

#endif    // OPTION_ILI9341_TFT

///////////////////////////////////////////////////////////////////////
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// EEPROM 24C256 等の外部EEPROMのI2C読み書き関数。
//
// rc 送信結果 (byte)
// 0: 成功
// 1: 送ろうとしたデータが送信バッファのサイズを超えた
// 2: スレーブ・アドレスを送信し、NACKを受信した
// 3: データ・バイトを送信し、NACKを受信した
// 4: その他のエラー
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// 1バイト書込
uint8_t eeprom_write(uint16_t address, uint8_t data)  {
  uint8_t rc;
  Wire.beginTransmission(EEPROM_ADDRESS);
  Wire.write((byte)(address >> 8));   // アドレス上位
  Wire.write((byte)(address & 0xFF)); // アドレス下位
  Wire.write(data);
  rc = Wire.endTransmission();
  delay(6);
  return rc;
}

// 1バイト読込
//  戻り値：受信データ (byte)
uint8_t eeprom_read(uint16_t address)  {
  Wire.beginTransmission(EEPROM_ADDRESS);
  Wire.write((byte)(address >> 8));   // アドレス上位
  Wire.write((byte)(address & 0xFF)); // アドレス下位
  Wire.endTransmission();
  Wire.requestFrom(EEPROM_ADDRESS, 1);
  return  Wire.read();
}

// =======================================================================
#ifdef OPTION_ILI9341_TFT
// 2バイト書込
void eeprom_write_uint16(int addr, uint16_t i) {
  byte tmp;
  tmp = lowByte(i);
  eeprom_write(addr, tmp);
  delay(5);
  tmp = highByte(i);
  eeprom_write(addr + 1, tmp);
  delay(5);
}
// 2バイト読込
uint16_t eeprom_read_uint16(int addr) {
  uint16_t h, l, ret;
  l = eeprom_read(addr);
  delay(5);
  h = eeprom_read(addr + 1);
  delay(5);
  ret = (h << 8) + l;
  return (ret);
}

#ifdef OPTION_OCXO_Vcon  // I2C 0x60

// 8バイト書込 共用体を使用して引き渡し (double)UNI.dmHz <==> char UNI.intrC[8]
void eeprom_write_double(int addr, char* ccc) {
  byte adtmp = addr, tmp;

  for (uint8_t i = 0; i <= 7; i++) {
    tmp = ccc[i];
    eeprom_write(adtmp, tmp);
    adtmp++;
    delay(5);
  }
}

// 8バイト読込 共用体を使用して引き渡し (double)UNI.dmHz <==> char UNI.intrC[8]
void eeprom_read_double(int addr) {
  uint8_t adtmp = addr;
  for (uint8_t i = 0; i <= 7; i++) {
    UNI.intrC[i] = eeprom_read(adtmp);
    adtmp++;
    delay(5);
  }
}

#endif    // end OPTION_OCXO_Vcon  // I2C 0x60

#endif  // end OPTION_ILI9341_TFT
// =======================================================================

// =======================================================================
#ifdef OPTION_ILI9341_TFT
//////////////////////////////////////////////////////////////////////////////
// ペンタッチの座標を校正する
// ILI9341_TFT　の専用機能として、追加メニューで処理する。
// 返り値を引数を使って返すため、ポインター指定を使う(受取：*abc  呼ぶ時：&abc )
// 校正をキャンセルしたときは、return 0; を返す。
// 完了時は、return 1;  と　左上xyと右下xyの値を返す。
//////////////////////////////////////////////////////////////////////////////

int8_t touchCALB(uint16_t *xMin, uint16_t *yMin, uint16_t *xMax, uint16_t *yMax) {
  uint16_t xtmp;
  uint16_t ytmp;
  int8_t ans2;
  while (1) {

    display.fillScreen(ILI9341_BLACK);

    display.fillTriangle(0, 0, 0, 10, 10, 0, ILI9341_YELLOW);  // ▲

    TFT_setCursor(1, 1);   // 桁・行
    u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_YELLOW);      // apply Adafruit GFX color
    u8g2_for_adafruit_gfx.print("Touch the upper left");
    // ==================
    while (1) {
      if (ts.read_XY(xy)) { //If the touch screen is preesed, read the X,Y
        xtmp = xy[0];  // ■■重要 Touchセンサーのxy情報を読み込んだ後は、すぐに別変数に移す■■
        ytmp = xy[1];  // ■■重要 Touchセンサーのxy情報を読み込んだ後は、すぐに別変数に移す■■

#ifdef BOARD_V5_BUZZER
        beepms(50);     // キー動作のビープ音
#else
        delay(50);
#endif

        if ( !ts.read_XY(xy)) {   // Touchペンを放した。

          if ((200 < xtmp) && (xtmp < 550)) {   // 入力制限
            if  ((150 < ytmp) && ( ytmp < 450)) {   // 入力制限
              break;
            }
          }
        }
      }
    }
    // ==================
    TFT_setCursor(1, 3);      // 桁・行
    sprintf(tmpX, "%5d", xtmp);
    sprintf(tmpY, "%5d", ytmp);
    u8g2_for_adafruit_gfx.print("xMin =");
    u8g2_for_adafruit_gfx.print(tmpX);
    u8g2_for_adafruit_gfx.print(", yMin =");
    u8g2_for_adafruit_gfx.print(tmpY);
    *xMin = xtmp;
    *yMin = ytmp;
    delay(200);
    display.fillTriangle(0, 0, 0, 10, 10, 0, ILI9341_BLACK);  // ▲
    TFT_setCursor(1, 1);   // 桁・行
    u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_BLACK);      // apply Adafruit GFX color
    u8g2_for_adafruit_gfx.print("                    ");
    u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);      // apply Adafruit GFX color

    display.fillTriangle(319, 239, 319, 229, 309, 239, ILI9341_YELLOW);  // ▲
    TFT_setCursor(4, 7);   // 桁・行
    u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_YELLOW);      // apply Adafruit GFX color
    u8g2_for_adafruit_gfx.print("Touch the lower right");
    // ==================
    while (1) {
      if (ts.read_XY(xy)) { //If the touch screen is preesed, read the X,Y
        xtmp = xy[0];  // ■■重要 Touchセンサーのxy情報を読み込んだ後は、すぐに別変数に移す■■
        ytmp = xy[1];  // ■■重要 Touchセンサーのxy情報を読み込んだ後は、すぐに別変数に移す■■

#ifdef BOARD_V5_BUZZER
        beepms(50);     // キー動作のビープ音
#else
        delay(50);
#endif

        if ( !ts.read_XY(xy)) {   // Touchペンを放した。

          if ((3600 < xtmp) && (xtmp < 4000)) {   // 入力制限
            if  ((3600 < ytmp) && (ytmp < 4000)) {   // 入力制限
              break;
            }
          }
        }
      }   // end_if
    }   // end_while
    // ==================
    TFT_setCursor(1, 4);      // 桁・行
    sprintf(tmpX, "%5d", xtmp);
    sprintf(tmpY, "%5d", ytmp);
    u8g2_for_adafruit_gfx.print("xMax =");
    u8g2_for_adafruit_gfx.print(tmpX);
    u8g2_for_adafruit_gfx.print(", yMax =");
    u8g2_for_adafruit_gfx.print(tmpY);
    *xMax = xtmp;
    *yMax = ytmp;
    delay(200);

    display.fillTriangle(319, 239, 319, 229, 309, 239, ILI9341_BLACK);  // ▲
    TFT_setCursor(4, 7);   // 桁・行
    u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_BLACK);      // apply Adafruit GFX color
    u8g2_for_adafruit_gfx.print("                      ");
    u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);      // apply Adafruit GFX color

    //            012345678   0123
    FuncEnable = (B00111000 * 256) + B00000111; // F1, F2, F3
    func_print(1, " OK ");
    func_print(2, "AGIN");
    func_print(3, "RETN");
    ans2 = -1;
    // ==================

    while (ans2 < 0) {

      touchCCxLL(&xtmp, &ytmp);
      if (ytmp == 8) {
        if ((xtmp >= 8) && (xtmp <= 11)) ans2 = 1;
        if ((xtmp >= 14) && (xtmp <= 17)) ans2 = 2;
        if ((xtmp >= 20) && (xtmp <= 23)) ans2 = 3;
      }
      xtmp = 0;
      ytmp = 0;
    } // while end
    // ==================

    if (ans2 == 1)  break;      // "OK"
    if (ans2 == 2)  ans2 = -1;  // "AGIN"
    if (ans2 == 3)  break;      // "RETN"
  }
  display.fillScreen(ILI9341_BLACK);
  funcDisp = 0;
  return ans2;
}

//////////////////////////////////////////////////////////////////////////////
// eeprom の内容をTFTに出力する。
// touchPrm
//////////////////////////////////////////////////////////////////////////////

void touchPrmCheck(const char *title0) {
  uint16_t xtmp;
  uint16_t ytmp;
  int8_t ans2;
  while (1) {

    display.fillScreen(ILI9341_BLACK);

    TFT_setCursor(1, 0);      // 桁・行
    u8g2_for_adafruit_gfx.print("5");
    u8g2_for_adafruit_gfx.print(title0);

    // ==================
    TFT_setCursor(1, 3);      // 桁・行
    sprintf(tmpX, "%5d", tchXmin);
    sprintf(tmpY, "%5d", tchYmin);
    u8g2_for_adafruit_gfx.print("xMin =");
    u8g2_for_adafruit_gfx.print(tmpX);
    u8g2_for_adafruit_gfx.print(", yMin =");
    u8g2_for_adafruit_gfx.print(tmpY);

    TFT_setCursor(1, 4);      // 桁・行
    sprintf(tmpX, "%5d", tchXmax);
    sprintf(tmpY, "%5d", tchYmax);
    u8g2_for_adafruit_gfx.print("xMax =");
    u8g2_for_adafruit_gfx.print(tmpX);
    u8g2_for_adafruit_gfx.print(", yMax =");
    u8g2_for_adafruit_gfx.print(tmpY);

    TFT_setCursor(1, 6);      // 桁・行
    u8g2_for_adafruit_gfx.print("[ OK ]or[INIT]ialize");

    //            012345678   0123
    FuncEnable =  B00000110; // F1, F2,
    func_print(1, " OK ");
    func_print(2, "INIT");
    //    func_print(3, "RETN");
    ans2 = -1;
    // ==================
    while (ans2 < 0) {

      touchCCxLL(&xtmp, &ytmp);
      if (ytmp == 8) {
        if ((xtmp >= 8) && (xtmp <= 11)) ans2 = 1;
        if ((xtmp >= 14) && (xtmp <= 17)) ans2 = 2;
      }
      xtmp = 0;
      ytmp = 0;
    } // while end
    // ==================

    if (ans2 == 1)  break;      // "OK"
    if (ans2 == 2) {
      tchXmin = InitXmin;
      tchYmin = InitYmin;
      tchXmax = InitXmax;
      tchYmax = InitYmax;
      eeprom_write_uint16(3, tchXmin);
      eeprom_write_uint16(5, tchYmin);
      eeprom_write_uint16(7, tchXmax);
      eeprom_write_uint16(9, tchYmax);
    }
  }
  display.fillScreen(ILI9341_BLACK);
  funcDisp = 0;
  return;
}

// =======================================================================
// TFT画面の固定表示項目をまとめて表示する。
// =======================================================================

void fixedForm() {
  TFT_setCursor(1, 2);     // 桁・行
  u8g2_for_adafruit_gfx.print("GATE=");

  TFT_setCursor(1, 3);     // 桁・行
  u8g2_for_adafruit_gfx.print("SEND=");
  TFT_setCursor(6, 3);     // 桁・行
  switch (hostPRM) {
    case 0:
      u8g2_for_adafruit_gfx.print("GPS");
      break;
    case 1:
      u8g2_for_adafruit_gfx.print("LOG");
      break;
    case 2:
      u8g2_for_adafruit_gfx.print("PIC");
      break;
  }
  // 仮■■　変更したパラメータを表示する。デバッグ用
  char aa = 0x30 | jobPRM;
  char bb = 0x30 | bpsPRM;
  char cc = 0x30 | hostPRM;
  char dd = 0x30 | gatePRMon;

  TFT_setCursor(10, 3);    // 桁・行
  u8g2_for_adafruit_gfx.print(aa);
  u8g2_for_adafruit_gfx.print(bb);
  u8g2_for_adafruit_gfx.print(cc);
  u8g2_for_adafruit_gfx.print(dd);

#ifdef OPTION_OCXO_Vcon  // I2C 0x60
  char ee = 0x30 | stable;
  u8g2_for_adafruit_gfx.print(ee);
#endif

  TFT_setCursor(1, 4);     // 桁・行
  u8g2_for_adafruit_gfx.print("Ave:");
  TFT_setCursor(22, 4);   // 桁・行
  u8g2_for_adafruit_gfx.print("Hz");
  //
  TFT_setCursor(1, 5);     // 桁・行
  u8g2_for_adafruit_gfx.print("Max:");
  TFT_setCursor(22, 5);   // 桁・行
  u8g2_for_adafruit_gfx.print("Hz");
  //
  TFT_setCursor(1, 6);     // 桁・行
  u8g2_for_adafruit_gfx.print("Min:");

  TFT_setCursor(22, 6);   // 桁・行
  u8g2_for_adafruit_gfx.print("Hz");
  //
  TFT_setCursor(1, 7);     // 桁・行
  u8g2_for_adafruit_gfx.print("P-P:");

  TFT_setCursor(22, 7);   // 桁・行
  u8g2_for_adafruit_gfx.print("Hz");

}
#endif

// =======================================================================
#ifdef OPTION_ILI9341_TFT
//////////////////////////////////////////////////////////////////////////////
// ペンタッチの座標を使って数字入力をする
// ILI9341_TFT　の専用機能として、追加メニューで処理する。
// 最小値と最大値を指定して呼び出す。
// RET キャンセルしたときは、入力前の値を返す。
// ENT 完了時は、double 倍精度浮動小数点型の値を返す。
//
// 入力は、整数部５桁：100000未満～0
// 小数点以下は、５桁：0.00000
// 入力範囲外は、RET ENT CLRのみ有効
//////////////////////////////////////////////////////////////////////////////

double tenkey(int16_t minVal, int16_t maxVal, double inputval) {
  uint16_t xtmp;
  uint16_t ytmp;
  double inptmp;
  double RETvalue;
  int8_t ans2, inbusy = 1;
  int8_t decimalp = -1;   // 少数点以下１桁目＝１、２桁目＝２、、、、、

  inptmp = 0;

  display.fillScreen(ILI9341_BLACK);
  //  u8g2_for_adafruit_gfx.setFont(u8g2_font_profont29_mr);  // 大きい文字  W16xH29 (A19) WHITE YELLOW
  fonSet_BIG();

  u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
  // u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_YELLOW);

  // 10Keyの数字キー
  TFT_setCursor10(1, 1);   // 桁・行
  u8g2_for_adafruit_gfx.print("  7   8   9  CLR");
  TFT_setCursor10(1, 2);   // 桁・行
  u8g2_for_adafruit_gfx.print("  4   5   6     ");
  TFT_setCursor10(1, 3);   // 桁・行
  u8g2_for_adafruit_gfx.print("  1   2   3     ");
  TFT_setCursor10(1, 4);   // 桁・行
  u8g2_for_adafruit_gfx.print("  0   .  RET ENT");
  //  TFT_setCursor10(1, 5);   // 桁・行
  //  u8g2_for_adafruit_gfx.print("     10Key touch");
  // 10Keyの枠組み
  TFT_10keyHline(1, 0, 16);
  TFT_10keyHline(1, 1, 16);
  TFT_10keyHline(1, 2, 16);
  TFT_10keyHline(1, 3, 16);
  TFT_10keyHline(1, 4, 16);
  //  TFT_10keyHline(1, 5, 4);
  TFT_10keyVline(1, 0, 4);
  TFT_10keyVline(5, 0, 4);
  TFT_10keyVline(9, 0, 4);
  TFT_10keyVline(13, 0, 4);
  TFT_10keyVline(17, 0, 4);
  //  TFT_10keyVline(5, 4, 1);

  while (inbusy) {

    TFT_setCursor(3, 0);   // 桁・行
    switch (decimalp) {
      case 0:
        sprintf(tmptmp, "%5.lf", inptmp);   // 一つ桁を少なく
        strcat(tmptmp, ".");
        break;
      case 1:
        sprintf(tmptmp, "%7.1lf", inptmp);
        break;
      case 2:
        sprintf(tmptmp, "%8.2lf", inptmp);
        break;
      case 3:
        sprintf(tmptmp, "%9.3lf", inptmp);
        break;
      case 4:
        sprintf(tmptmp, "%10.4lf", inptmp);
        break;
      case 5:
        sprintf(tmptmp, "%11.5lf", inptmp);
        break;
      case -1:
        sprintf(tmptmp, "%6.lf", inptmp);
        break;
      default:
        break;
    }

    //    sprintf(tmptmp, "%10.5lf", inptmp);
    u8g2_for_adafruit_gfx.print(tmptmp);
    // タッチ入力を調べる
    ans2 = touchSW10();
    if (decimalp == -1) {    // 整数部を入力中。小数点を押すと decimalp++する。
      if (inptmp >= 100000) {  // 入力制限■■■
        inptmp = 0;
        errBeep();    // エラー音を鳴らす（この関数の下の位置にあり）
      }
      switch (ans2) {
        case 10:
          inptmp = inptmp * 10 + 7;
          break;
        case 11:
          inptmp = inptmp * 10 + 8;
          break;
        case 12:
          inptmp = inptmp * 10 + 9;
          break;
        case 20:
          inptmp = inptmp * 10 + 4;
          break;
        case 21:
          inptmp = inptmp * 10 + 5;
          break;
        case 22:
          inptmp = inptmp * 10 + 6;
          break;
        case 30:
          inptmp = inptmp * 10 + 1;
          break;
        case 31:
          inptmp = inptmp * 10 + 2;
          break;
        case 32:
          inptmp = inptmp * 10 + 3;
          break;
        case 40:
          inptmp = inptmp * 10 + 0;
          break;
        case 41:      // [.]
          decimalp++;
          break;
        case 42:      // RETurn
          RETvalue = inputval;
          inbusy = 0;
          break;
        case 43:      // ENTer
          if ((inptmp >= minVal) && (inptmp <= maxVal)) {
            RETvalue = inptmp;
            inbusy = 0;
          } else {        // 入力範囲外■■■
            inptmp = 0;
            decimalp = -1;
            errBeep();    // エラー音を鳴らす（この関数の下の位置にあり）
          }
          break;
        case 13:      // CLR
          inptmp = 0;
          decimalp = -1;
          TFT_setCursor(3, 0);   // 桁・行
          u8g2_for_adafruit_gfx.print("                ");
          break;
        default:
          break;
      }

    } else if (decimalp >= 5) {       // 小数点以下 5桁以内指定
      switch (ans2) {                 // 5桁超えは、次のキーしか受け付けない。
        case 42:      // RETurn
          RETvalue = inputval;
          inbusy = 0;
          break;
        case 43:      // ENTer
          if ((inptmp >= minVal) && (inptmp <= maxVal)) {
            RETvalue = inptmp;
            inbusy = 0;
          } else {        // 入力範囲外■■■
            inptmp = 0;
            decimalp = -1;
            errBeep();    // エラー音を鳴らす（この関数の下の位置にあり）
          }
          break;
        case 13:      // CLR
          inptmp = 0;
          decimalp = -1;
          TFT_setCursor(3, 0);   // 桁・行
          u8g2_for_adafruit_gfx.print("                ");
          break;
      }
    } else {
      switch (ans2) {
        case 10:
          decimalp++;
          inptmp = inptmp + (1 / pow(10, (double)decimalp) * 7);
          break;
        case 11:
          decimalp++;
          inptmp = inptmp + (1 / pow(10, (double)decimalp) * 8);
          break;
        case 12:
          decimalp++;
          inptmp = inptmp + (1 / pow(10, (double)decimalp) * 9);
          break;
        case 20:
          decimalp++;
          inptmp = inptmp + (1 / pow(10, (double)decimalp) * 4);
          break;
        case 21:
          decimalp++;
          inptmp = inptmp + (1 / pow(10, (double)decimalp) * 5);
          break;
        case 22:
          decimalp++;
          inptmp = inptmp + (1 / pow(10, (double)decimalp) * 6);
          break;
        case 30:
          decimalp++;
          inptmp = inptmp + (1 / pow(10, (double)decimalp) * 1);
          break;
        case 31:
          decimalp++;
          inptmp = inptmp + (1 / pow(10, (double)decimalp) * 2);
          break;
        case 32:
          decimalp++;
          inptmp = inptmp + (1 / pow(10, (double)decimalp) * 3);
          break;
        case 40:      // [0]
          decimalp++;
          break;
        case 42:      // RETurn
          RETvalue = inputval;
          inbusy = 0;
          break;
        case 43:      // ENTer
          if ((inptmp >= minVal) && (inptmp <= maxVal)) {
            RETvalue = inptmp;
            inbusy = 0;
          } else {        // 入力範囲外■■■
            inptmp = 0;
            decimalp = -1;
            errBeep();    // エラー音を鳴らす（この関数の下の位置にあり）
          }
          break;
        case 13:      // CLR
          inptmp = 0;
          decimalp = -1;
          TFT_setCursor(3, 0);   // 桁・行
          u8g2_for_adafruit_gfx.print("                ");
          break;
        default:
          break;
      }
    }
  }
  display.fillScreen(ILI9341_BLACK);
  //  u8g2_for_adafruit_gfx.setFont(u8g2_font_profont22_mf);  // OK (A14)  W12xH22
  fonSet_MID();

  u8g2_for_adafruit_gfx.setForegroundColor(ILI9341_WHITE);
  funcDisp = 0;
  return RETvalue;
}

// ==============================================
// tenkey() の入力ミス・オーバーのエラー音
void errBeep() {
#ifdef BOARD_V5_BUZZER
  delay(200);
  beepms(500);     // キー動作のビープ音
  delay(200);
  beepms(100);     // キー動作のビープ音
  TFT_setCursor(3, 0);   // 桁・行
  u8g2_for_adafruit_gfx.print("                ");
#else
  delay(50);
#endif
}
// ==============================================

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// 大きい文字用 １０キー用　１０Ｋｅｙ 320 x 240
//  対象とするフォント：
//          u8g2_for_adafruit_gfx.setFont(u8g2_font_profont29_mf);
//                Width:16, Height:29, Capital A 19
//
//  ＊1文字目の左8ドットは、空ける。    // 桁・行
// 行は38ドットごとにするVertical line
// 桁は16ドットそのまま
//
//
//   cc   : 桁　0から18
//   ll   : 行　0から5
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

void TFT_setCursor10(uint8_t cc, uint8_t ll) // cc:0 to 20, ll:0 to 8  Use
{
  uint16_t cp, lp;
  cp = cc * 16 + 8;
  lp = (ll + 1 ) * 38 - 6;
  u8g2_for_adafruit_gfx.setCursor(cp, lp);
}
// 横線を引く
void TFT_10keyHline(uint8_t cc, uint8_t ll, uint8_t nn) // cc:0 to 20, ll:0 to 8  Use
{
  uint16_t cp, lp, lng;
  cp = cc * 16 + 14;
  lp = (ll + 1 ) * 38 + 2;
  lng = 16 * nn + 3;
  display.fillRoundRect(cp, lp, lng, 3, 0, ILI9341_CYAN);
  //  u8g2_for_adafruit_gfx.setCursor(cp, lp);
}
// 縦線を引く
void TFT_10keyVline(uint8_t cc, uint8_t ll, uint8_t nn) // cc:0 to 20, ll:0 to 8  Use
{
  uint16_t cp, lp, lng;
  cp = cc * 16 + 14;
  lp = (ll + 1 ) * 38 + 2;
  lng = 38 * nn;
  display.fillRoundRect(cp, lp, 3, lng, 0, ILI9341_CYAN);
  //  u8g2_for_adafruit_gfx.setCursor(cp, lp);
}

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ボタンSWの代替 １０Ｋｅｙ用
// 返り値：１０Ｋｅｙ専用に新規定義した。(2021/01/26)
//   ＝＝返り値＝＝     ＝＝想定キー＝＝
//   00  01  02  03
//   10  11  12  13  =>  7   8   9  CLR
//   20  21  22  23  =>  4   5   6
//   30  31  32  33  =>  1   2   3
//   40  41  42  43  =>  0   .  RET ENT
//   50  51  52  53
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■

int8_t touchSW10() {
  uint16_t xtmp = 0;
  uint16_t ytmp = 0;
  int8_t ans2;
  // ans2 = -1;
  // ==================
  //  while (ans2 < 0) {

  ans2 = touchCCxLL10(&xtmp, &ytmp);
  switch (ytmp) {
    case 1:
      if ((xtmp >= 2) && (xtmp <= 4)) ans2 = 10;
      if ((xtmp >= 6) && (xtmp <= 8)) ans2 = 11;
      if ((xtmp >= 10) && (xtmp <= 12)) ans2 = 12;
      if ((xtmp >= 14) && (xtmp <= 16)) ans2 = 13;
      break;
    case 2:
      if ((xtmp >= 2) && (xtmp <= 4)) ans2 = 20;
      if ((xtmp >= 6) && (xtmp <= 8)) ans2 = 21;
      if ((xtmp >= 10) && (xtmp <= 12)) ans2 = 22;
      if ((xtmp >= 14) && (xtmp <= 16)) ans2 = 23;
      break;
    case 3:
      if ((xtmp >= 2) && (xtmp <= 4)) ans2 = 30;
      if ((xtmp >= 6) && (xtmp <= 8)) ans2 = 31;
      if ((xtmp >= 10) && (xtmp <= 12)) ans2 = 32;
      if ((xtmp >= 14) && (xtmp <= 16)) ans2 = 33;
      break;
    case 4:
      if ((xtmp >= 2) && (xtmp <= 4)) ans2 = 40;
      if ((xtmp >= 6) && (xtmp <= 8)) ans2 = 41;
      if ((xtmp >= 10) && (xtmp <= 12)) ans2 = 42;
      if ((xtmp >= 14) && (xtmp <= 16)) ans2 = 43;
      break;
    default:
      ans2 = 99;
      break;
#ifdef BOARD_V5_BUZZER
      beepms(50);     // キー動作のビープ音
#else
      delay(50);
#endif
  }
  xtmp = 0;
  ytmp = 0;
  //  } // while end
  // ==================

  return ans2;
}

//////////////////////////////////////////////////////////////////////////////
// １０Ｋｅｙ用
//  対象とするフォント：
//          u8g2_for_adafruit_gfx.setFont(u8g2_font_profont29_mf);
//                Width:16, Height:29, Capital A 19
// ペンタッチ押して放した時の文字位置を返す。
// 返り値を引数を使って返すため、ポインター指定を使う(受取：*abc  呼ぶ時：&abc )
// 押されて無かった時は、留まらず return 0; を返す。
// 押された時は、return 1;  と　cc桁,  ll行,  に（文字換算の）位置情報を返す。
//
// ■■重要 Touchセンサーのxy情報を読み込んだ後は、すぐに別変数に移す■■
// xy情報は続けてすぐに別変数に移さないと、yの読取値が化ける。
//////////////////////////////////////////////////////////////////////////////
int8_t touchCCxLL10(uint16_t *cc, uint16_t *ll) {
  uint16_t xtmp;
  uint16_t ytmp;
  uint16_t ptmp;
  float ftmp1;
  int16_t ftmp2;

  if (ts.read_XY(xy)) { // true = 1 / false = 0
    xtmp = xy[0];  // ■■重要 Touchセンサーのxy情報を読み込んだ後は、すぐに別変数に移す■■
    ytmp = xy[1];  // ■■重要 Touchセンサーのxy情報を読み込んだ後は、すぐに別変数に移す■■

#ifdef BOARD_V5_BUZZER
    beepms(150);    // キー動作のビープ音
#endif

    while (1) {   // この行以下4行を // コメントにすれば、押した時の取り込み。
      if ( !ts.read_XY(xy)) { // 放すのを待つ。
        break;
      }
    }

    if (xtmp <= tchXmin) xtmp = tchXmin + 1;  // タッチ位置が最小座標より小なら
    if (xtmp >= tchXmax) xtmp = tchXmax - 1;  // タッチ位置が最小座標より大なら
    ftmp1 = (float)(xtmp - tchXmin) / (float)(tchXmax - tchXmin); // 横、左から何％の位置
    ftmp2 = ftmp1 * (float)pixWidth - 1; // ＊桁の位置調整
    if (ftmp2 <= 0) ftmp2 = 1;      // ＊最左桁の調整
    if (ftmp2 >= 311) ftmp2 = 311;  // ＊最右桁の調整(20桁目を無くす)
    *cc = (ftmp2 - 8) / 16;
    //
    if (ytmp <= tchYmin) ytmp = tchYmin + 1;  // タッチ位置が最小座標より小なら
    if (ytmp >= tchYmax) ytmp = tchYmax - 1;  // タッチ位置が最小座標より大なら
    ftmp1 = (float)(ytmp - tchYmin) / (float)(tchYmax - tchYmin); // 縦、上から何％の位置
    ftmp2 = ftmp1 * (float)pixHeight - 1; // ＊行の位置調整
    if (ftmp2 <= 0) ftmp2 = 1;      // ＊最上位行の調整
    if (ftmp2 >= 239) ftmp2 = 239;  // ＊最下行の調整
    *ll = ftmp2 / 38;
    return 1;
  } else {
    return 0;
  }
}
#endif

// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
// ■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
